/**
* @file AEventHandler.h
* @brief Abstract Application Event Handler class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef AEventHandler_h
#define AEventHandler_h

#ifdef WIN32
#pragma warning(disable:4311 4312 4267 4018 4244 4996) 
#endif 

#include <string>
#include "srllib.h"
#include "gclib.h"

#define APPEVENT_ON_MEDIA_CONNECTED             0x0001
#define APPEVENT_ON_MEDIA_DISCONNECTED          0x0002
#define APPEVENT_IPM_GET_RX_PORT_INFO_COMPLETE  0x0003
#define APPEVENT_IPM_GET_TX_PORT_INFO_COMPLETE  0x0004
#define APPEVENT_IPM_PORT_CONNECTED             0x0005
#define APPEVENT_IPM_PORT_DISCONNECTED          0x0006
#define APPEVENT_MM_OPEN_COMPLETE               0x0007
#define APPEVENT_MM_GET_RX_PORT_INFO_COMPLETE   0x0008
#define APPEVENT_MM_GET_TX_PORT_INFO_COMPLETE   0x0009
#define APPEVENT_MM_PORT_CONNECTED              0x000A
#define APPEVENT_MM_PORT_DISCONNECTED           0x000B
#define APPEVENT_MM_PLAY_COMMAND                0x000C
#define APPEVENT_MM_STOP_COMMAND                0x000D
#define APPEVENT_PLAY_COMPLETE                  0x000E
#define APPEVENT_PLAY_FAIL                      0x000F
#define APPEVENT_MM_RECORD_COMMAND              0x0010
#define APPEVENT_RECORD_COMPLETE                0x0011
#define APPEVENT_RECORD_FAIL                    0x0012

#define APPEVENT_ON_CLOSE_CNF_PARTY_COMPLETE    0x0020
#define APPEVENT_ON_OPEN_CNF_PARTY_COMPLETE     0x0021
#define APPEVENT_ON_CNF_STARTED    	 				0x0022
#define APPEVENT_ON_CNF_STOPPED    	 				0x0023
#define APPEVENT_ON_JOIN_CNF_COMPLETE           0x0024
#define APPEVENT_ON_LEAVE_CNF_COMPLETE          0x0025
#define APPEVENT_ON_LEAVE_CNF_FAILED				0x0026
#define APPEVENT_CNF_CONNECTED                  0x0027
#define APPEVENT_CNF_DISCONNECTED               0x0028
#define APPEVENT_CNF_GET_RX_PORT_INFO_COMPLETE  0x0029
#define APPEVENT_CNF_GET_TX_PORT_INFO_COMPLETE  0x0030
#define APPEVENT_CNF_PORT_CONNECTED             0x0031
#define APPEVENT_CNF_PORT_DISCONNECTED          0x0032

#define APPEVENT_RESERVE_RESOURCE_COMPLETE      0x0040
#define APPEVENT_RELEASE_RESOURCE_COMPLETE      0x0041
#define APPEVENT_GET_LOCAL_MEDIA_INFO_COMPLETE  0x0042
#define APPEVENT_START_MEDIA_COMPLETE           0x0043
#define APPEVENT_STOP_MEDIA_COMPLETE            0x0044
#define APPEVENT_IPM_CONNECTED                  0x0045
#define APPEVENT_IPM_DISCONNECTED               0x0046

#define APPEVENT_CNF_SET_VIDEO_LAYOUT_COMPLETE  		0x0050
#define APPEVENT_CNF_SET_VISIBLE_PARTY_LIST_COMPLETE 	0x0051
#define APPEVENT_DTMF_DIGIT                     		0x0052

#define APPEVENT_DROPCALL_COMMAND               0x0060
#define APPEVENT_CLOSE_COMMAND                  0x0061
#define APPEVENT_DISPLAY_CALLERID_COMMAND       0x0062
#define APPEVENT_DISPLAY_LAYOUT_COMMAND         0x0063

#define APPEVENT_FEATURE_TIMEOUT                0x0070

#define APPEVENT_RTSP_EVENT                     0x0080

//*****************************************************************************
// Class: ::AEventHandler
// Description: Abstract Event Handler class
//    Encapsulate variables and functions common to classes that handle events
//*****************************************************************************
class AEventHandler
{
public:
   AEventHandler() {}
   virtual ~AEventHandler() {}
   virtual void ProcessEvent(METAEVENT& a_metaEvent)=0;

   virtual int Stop() = 0;

   void PostEvent(int a_event, int a_data = 0)
   {
      METAEVENT l_metaEvent;
      l_metaEvent.evttype =  a_event;
      l_metaEvent.evtdatap =  (void*) a_data;
      l_metaEvent.evtlen = 0;
      l_metaEvent.evtdev = 0;
      ProcessEvent(l_metaEvent);
   }

   void PostEvent(METAEVENT& a_metaEvent)
   {
      ProcessEvent(a_metaEvent);
   }

   virtual bool ShouldClose() {return true;}

private:
};

#endif


