/**
* @file CConference.cpp
* @brief Conference class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#include <time.h>

#endif

#include <srllib.h>

#include "LogUtil.h"
#include "CConference.h"
#include "CSipSession.h"
#include "CConferenceBoardDevice.h"
#include "CConferencePartyDevice.h"
#include "CConferenceDevice.h"
#include "CGcManager.h"
#include "CSystemConfiguration.h"
#include "CVirtualCaller.h"
#include "CRecorder.h"

#include "CRtspConnector.h"

using namespace std;

#ifdef USE_RTSP
extern void rtspconnector_event_callback( RtspEvent* event );
RtspConnector CConference::g_rtspConnector("VideoConfDemo",rtspconnector_event_callback);
#endif

//CConference* CConference::m_theConference[MAX_CONF_COUNT]= {0};
CConferenceBoardDevice* CConference::m_cnfConferenceBoardDevice;
// SKS for multiple conference maximum 10 at this moment
//CConferenceDevice* CConference::m_cnfConferenceDevice[MAX_CONF_COUNT]={0};
CSystemConfiguration* CConference::m_systemConfiguration;
std::map<std::string,CConference* > CConference::m_theConference;
std::map<std::string,CConferenceDevice* > CConference::m_cnfConferenceDevice;

int CConference::m_allowedLayouts[] = {1,2,4,6,9};
int CConference::m_allowedLayoutsCount = 5;
int CConference::m_initialLayout = 1;

//*****************************************************************************
// Function: CConference::CConference()
// Description: Default constructor
// Return:  CConference*
// Parameters: none 
//*****************************************************************************
CConference::CConference()
   :m_cnfPartyCount(0)
   ,m_realPartyCount(0)
   ,m_virtualPartyCount(0)
   ,m_layout(0)
   ,m_bConfTimeOut(true)
   ,m_bRecorderInConf(false)
{
   m_listRealCallerInConf.clear();
   for (int i=0;i<MAX_CONF_COUNT;i++)
   {
      m_cnfParties[i] = 0;
   }

   for (int i=0;i<MAX_CONF_COUNT;i++)
   {
      m_realParties[i] = 0;
   }

   for (int i=0;i<MAX_REGION_COUNT-1;i++)
   {
      m_virtualParties[i] = 0;
   }

   for (int i=0;i<m_allowedLayoutsCount;i++)
   {
      if (m_allowedLayouts[i] == m_initialLayout)
      {
         m_layout = i;
      }
   }

}

//*****************************************************************************
// Function: void CConference::SetConferenceBoard(CConferenceBoardDevice *a_cnfBoard)
// Description: Set the conference board used to create confernce device objects
// Return: void 
// Parameters: CConferenceBoardDevice *a_cnfBoard 
//*****************************************************************************
void CConference::SetConferenceBoard(CConferenceBoardDevice* a_cnfBoard)
{
   m_cnfConferenceBoardDevice = a_cnfBoard;
}

//*****************************************************************************
// Function: CConferenceBoardDevice* CConference::GetConferenceBoard()
// Description: Return a pointer to the conference board device object
// Return: CConferenceBoardDevice* 
// Parameters: none 
//*****************************************************************************
CConferenceBoardDevice* CConference::GetConferenceBoard()
{
   return m_cnfConferenceBoardDevice;
}

//*****************************************************************************
// Function: CConference* CConference::GetConference(string a_conferenceId)
// Description: Return a pointer to a conference object for a confernce ID
//    Only one conference is currently supported so the ID is ignored
// Return: CConference* 
// Parameters: string a_conferenceId 
//*****************************************************************************
CConference* CConference::GetConference(std::string a_conferenceId)
{
// SKS adding multiple conference up to 10 currently
   LogMsg("JM::CONF", "CConference::GetConference():" + AsString(a_conferenceId));	
   if (!m_theConference[a_conferenceId])
   {
      LogMsg("JM::CONF", "CConference::GetConference() create new Conference with ConfID=" + AsString(a_conferenceId));
      m_theConference[a_conferenceId] = new CConference();    
      m_theConference[a_conferenceId]->SetConferencePIN(a_conferenceId);
      // SKS Now get ready with virtual parties for the conference 
      m_theConference[a_conferenceId]->OpenVirtualParties(a_conferenceId);
      m_theConference[a_conferenceId]->StartRecorder(a_conferenceId);
   }
   return m_theConference[a_conferenceId];
}

void CConference::SetConferencePIN(std::string confID)
{
   m_vConferencePIN[confID]= m_systemConfiguration->getConfPIN(confID);
   m_systemConfiguration->SetConfIDandPIN(confID,m_vConferencePIN[confID]);
}
//void CConference::SetConferencePIN(std::string confID)
//{
//   std::string l_baseConfID = m_systemConfiguration->getBaseConfID();
//   m_vConferencePIN[confID]= AsString(m_systemConfiguration->getBaseConfPIN()) + confID.substr((l_baseConfID.length()),confID.length()-l_baseConfID.length());
//   m_systemConfiguration->SetConfIDandPIN(confID,m_vConferencePIN[confID]);
//}

std::string CConference::GetConferencePIN(std::string confID)
{
   std::string pinID;

   pinID=m_vConferencePIN[confID];
   return pinID;
}

int CConference::GetConferencePINLen(std::string confID)
{
   int pinLen = 0;

   pinLen=m_vConferencePIN[confID].length();
   return pinLen;
}

bool CConference::IsConfRoomExist(std::string confID)
{
   if (m_theConference.find(confID) != m_theConference.end()) {
      return true;
   }
   return false;
}

std::string CConference::GetConferenceID()
{
   std::string cnfID;
   map<std::string,CConference* >::iterator itCnf;
   itCnf=m_theConference.begin();
   cnfID=itCnf->first;
   for(;itCnf!=m_theConference.end();itCnf++) {
      if(itCnf->second==this) {
         cnfID=itCnf->first;
      }
   }
   return cnfID;
}

std::string CConference::GetDefaultConferenceID()
{
   std::string cnfID = "no_default";
   map<std::string,CConference* >::iterator itCnf;
   if (!m_theConference.empty()){
      itCnf=m_theConference.begin();
      cnfID=itCnf->first;
   }
   return cnfID;
}

//*****************************************************************************
// Function: void CConference::SystemConfiguration(CSystemConfiguration *a_systemConfiguration)
// Description: Set the system configuration 
// Return: void 
// Parameters: CSystemConfiguration *a_systemConfiguration 
//*****************************************************************************
void CConference::SystemConfiguration(CSystemConfiguration* a_systemConfiguration)
{
   m_systemConfiguration = a_systemConfiguration;
}

//*****************************************************************************
// Function: int CConference::AddConferenceParty(CConferencePartyDevice *a_conferenceParty)
// Description: Add a party to the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConference::AddConferenceParty(CConferencePartyDevice* a_conferenceParty)
{
   LogMsg("SKS::CONF", "CConference::AddConferenceParty() " + AsString(GetConferenceID()));	
// SKS to make sure multiple cnf work
   return m_cnfConferenceDevice[GetConferenceID()]->AddConferenceParty(a_conferenceParty);
}

//*****************************************************************************
// Function: int CConference::OnConferencePartyAdded(CConferencePartyDevice *a_conferenceParty)
// Description: Handle conference party added completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConference::OnConferencePartyAdded(CConferencePartyDevice* a_conferenceParty)
{
   m_cnfPartyCount++;
   if(a_conferenceParty->IsPartyRecorder()) {
      // SKS TODO
      m_bRecorderInConf=true;
   } else if (a_conferenceParty->IsVirtualParty()) {
      m_virtualParties[m_virtualPartyCount] = a_conferenceParty;
      m_virtualPartyCount++;
   } else {
// SKS adding virtual parties when conf has one party at least
      m_realParties[m_realPartyCount] = a_conferenceParty;
      m_realPartyCount++;
      if(m_realPartyCount==1) {
//       if(m_systemConfiguration->isRecordConferenceEnabled()==true) {
         AddRecorderToConference();
//       }
         AddVirtualPartiesToConference();
         if(m_systemConfiguration->getConfDuration()>0) {
            createConferenceTimerThread();
         }
      }
      m_conferenceParties.push_back(a_conferenceParty);
      a_conferenceParty->SetCallerId(m_conferenceParties.size()-1);
   }

   RefreshVisibleParties();

   return 0;
}

void CConference::createConferenceTimerThread()
{
#ifdef WIN32
   
   m_confTimerThreadH = ((HANDLE) _beginthreadex( (void *)        (NULL), 
                                       (unsigned)      (0), 
                                       CConference::confTimerThreadFunc, 
                                       reinterpret_cast<void*>(this), 
                                       (unsigned)      (0), 
                                       (unsigned *)    (NULL)));

   if ( m_confTimerThreadH == INVALID_HANDLE_VALUE )   

#else
   
   if(pthread_create(&m_confTimerThreadH,NULL,CConference::confTimerThreadFunc,reinterpret_cast<void*>(this))!=0)
#endif

   
   {
      LogMsg( "SKS::CONF", "Unable to create Conference timer thread " + AsString(GetConferenceID()));
   }
}

#ifdef WIN32
	unsigned int WINAPI CConference::confTimerThreadFunc(void * pContext)
#else
	void *CConference::confTimerThreadFunc(void *pContext)
#endif
{
   CConference *pConf=(CConference*)pContext;

   time_t startTime;
   time_t currTime;
   double timeElapsed;

   time(&startTime);

   pConf->m_bConfTimeOut=false;
   while(pConf->m_realPartyCount>=1 && pConf->m_bConfTimeOut==false) {
#ifdef WIN32
	   Sleep(5);
#else
      usleep(5000);
#endif
      time(&currTime);
      timeElapsed=difftime(currTime,startTime);
      if(timeElapsed>=pConf->m_systemConfiguration->getConfDuration()) {
         LogMsg( "SKS::CONF", "Conference Time Out!!!!!!!!!!!!!!!!!!!! " + AsString(pConf->GetConferenceID()));
         pConf->m_recorder->Play();
#ifdef WIN32
		Sleep(20000);
#else
         sleep(20);	// 20 sec
#endif
         pConf->m_bConfTimeOut=true;
      }
   }
   
   if(pConf->m_realPartyCount>=1) {
      // Timeout occurs lets remove real parties and drop the call, so virtual will follow automatically
      pConf->RemoveRealPartiesFromConference();
   }
   return 0;
}

//*****************************************************************************
// Function: int CConference::RemoveConferenceParty(CConferencePartyDevice *a_conferenceParty)
// Description: Remove a party from the conference
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConference::RemoveConferenceParty(CConferencePartyDevice* a_conferenceParty)
{
// SKS to make sure multiple cnf work
   return m_cnfConferenceDevice[GetConferenceID()]->RemoveConferenceParty(a_conferenceParty);
}

//*****************************************************************************
// Function: int CConference::OnConferencePartyRemoved(CConferencePartyDevice *a_conferenceParty)
// Description: Handle remove party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: CConferencePartyDevice *a_conferenceParty 
//*****************************************************************************
int CConference::OnConferencePartyRemoved(CConferencePartyDevice* a_conferenceParty)
{
   // SKS to make sure multiple cnf work
   m_cnfConferenceDevice[GetConferenceID()]->OnConferencePartyRemoved(a_conferenceParty);
   CConferencePartyDevice* l_newCnfParties[MAX_CONF_COUNT] = {0};
   int j(0);
   for (int i=0;i<j;i++)
   {
      m_cnfParties[i] = l_newCnfParties[i];
   }
   m_cnfPartyCount--;
   if (a_conferenceParty->IsPartyRecorder()) {
      // SKS TODO
      m_bRecorderInConf=false;
   } else if (a_conferenceParty->IsVirtualParty()){
      // SKS
      m_virtualPartyCount--;
   } else {
      CConferencePartyDevice* l_newCnfParties[MAX_CONF_COUNT] = {0};
      int j(0);
      for (int i=0; i<m_realPartyCount;i++)
      {
         if (m_realParties[i] != a_conferenceParty)
         {
            l_newCnfParties[j++] = m_realParties[i];
         }
         m_realParties[i] = 0;
      }
      for (int i=0;i<j;i++)
      {
         m_realParties[i] = l_newCnfParties[i];
      }
      m_realPartyCount--;

      if(m_realPartyCount==0) {
         // SKS
         if(m_bRecorderInConf==true) {
            RemoveRecorderFromConference();
         }
         RemoveVirtualPartiesFromConference();
         m_listRealCallerInConf.clear();
      }
      m_conferenceParties.remove(a_conferenceParty);
   }

   int l_callerId(0);
   std::list<CConferencePartyDevice*>::iterator l_partyDevice;
   for ( l_partyDevice=m_conferenceParties.begin();
         l_partyDevice!=m_conferenceParties.end();
         l_partyDevice++)
   {
      (*l_partyDevice)->SetCallerId(l_callerId++);
   }

   RefreshVisibleParties();

   return 0;
}

void CConference::AddRecorderToConference()
{
	m_recorder->StartRecorder();
}
void CConference::RemoveRecorderFromConference()
{
	m_recorder->LeaveConference();
}

void CConference::AddVirtualPartiesToConference()
{
	LogMsg("vCaller", "AddVirtualPartiesFromConference()");	
   vector<CVirtualCaller*>::iterator itVCaller;
   CVirtualCaller *l_vCaller;
   for(itVCaller=v_VirtualCallerInConf.begin();itVCaller!=v_VirtualCallerInConf.end();itVCaller++) {
      l_vCaller=(*itVCaller);
      l_vCaller->StartVirtualCaller();
   }
}

void CConference::RemoveVirtualPartiesFromConference()
{
	vector<CVirtualCaller*>::iterator itVCaller;
	CVirtualCaller *l_vCaller;
   for(itVCaller=v_VirtualCallerInConf.begin();itVCaller!=v_VirtualCallerInConf.end();itVCaller++) {
		l_vCaller=(*itVCaller);
		LogMsg(l_vCaller->Name(), "RemoveVirtualPartiesFromConference()");	
		l_vCaller->LeaveConference();
   }
}

void CConference::SetRealPartiesSipSessionInfo(CSipSession *sipSession)
{
	m_listRealCallerInConf.push_back(sipSession);
}

void CConference::RemoveRealPartiesSipSessionInfo(CSipSession *sipSession)
{
	LogMsg("sipCaller", "RemoveRealPartiesSipSessionInfo()");	
	list<CSipSession*>::iterator itRealCaller;
	cout<<"Before removing the current SIP session object"<<endl;
	for(itRealCaller=m_listRealCallerInConf.begin();itRealCaller!=m_listRealCallerInConf.end();itRealCaller++)
		 cout<<"SipSessionInfo="<<(*itRealCaller)<<"|"<<sipSession<<endl;
	cout<<"RemoveRealPartiesSipSessionInfo"<<endl;
	m_listRealCallerInConf.remove(sipSession);
/*
	int nCtr=0;
	vector<CSipSession*>::iterator itRealCaller;
	for(itRealCaller=v_RealCallerInConf.begin();itRealCaller!=v_RealCallerInConf.end();itRealCaller++,nCtr++) {
		cout<<"SipSessionInfo="<<(*itRealCaller)<<"|"<<sipSession<<endl;
		if((*itRealCaller)==sipSession) {
   		v_RealCallerInConf.erase(v_RealCallerInConf.begin()+nCtr);
			break;
		}
	}
*/

// SKS print remaining values;
	cout<<"After removing the current SIP session object"<<endl;
	for(itRealCaller=m_listRealCallerInConf.begin();itRealCaller!=m_listRealCallerInConf.end();itRealCaller++)
		 cout<<"SipSessionInfo="<<(*itRealCaller)<<"|"<<sipSession<<endl;


}

void CConference::RemoveRealPartiesFromConference()
{
	LogMsg("sipCaller", "RemoveRealPartiesFromConference()");	
   list<CSipSession*>::iterator itRealCaller;
   CSipSession *l_sipCaller;
   for(itRealCaller=m_listRealCallerInConf.begin();itRealCaller!=m_listRealCallerInConf.end();itRealCaller++) {
      l_sipCaller=(*itRealCaller);
		LogMsg(l_sipCaller->SessionName(), "RemoveRealPartiesFromConference()");	
      l_sipCaller->LeaveConference();
   }
	m_listRealCallerInConf.clear();
}

//*****************************************************************************
// Function: int CConference::OnConferenceBoardDeviceOpenComplete()
// Description: Handle an open conference board device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConference::OnConferenceBoardDeviceOpenComplete()
{
   int nCtr;
   std::string confName;
   // SKS Open more than one conference here.
   for(nCtr=1;nCtr<=m_systemConfiguration->getNoOfConfRooms();nCtr++)
   {
      //confName = m_systemConfiguration->getBaseConfID() + AsString(nCtr);
      confName = m_systemConfiguration->getConfID(nCtr);
      m_cnfConferenceDevice[confName] = new CConferenceDevice(confName, m_cnfConferenceBoardDevice);
      CGcManager::RegisterDevice(m_cnfConferenceDevice[confName]);
      m_cnfConferenceDevice[confName]->Observer(GetConference(confName));
      m_cnfConferenceDevice[confName]->Open();
   }

#ifdef USE_RTSP
   LogMsg("CONF", ": Start RtspConnector Object");
   //JM - Start RtspConnector
   g_rtspConnector.start();
#endif

   return 0;
}

void CConference::StartRecorder(std::string confID)
{
   LogMsg("CConference::StartRecorder confID=" + confID);
   m_recorder = new CRecorder(confID,m_systemConfiguration->isRecordConferenceEnabled());
}

void CConference::OpenVirtualParties(std::string confID)
{
   CVirtualCaller* l_virtualCaller(0);
   char l_devName[30];
	int nCtr;
   for (nCtr=1; nCtr<=m_systemConfiguration->NumVirtualCallers(); nCtr++)
   {
      sprintf(l_devName, "vCall_%.2d", nCtr);
      l_virtualCaller = new CVirtualCaller(AsString(l_devName),confID,nCtr);
		v_VirtualCallerInConf.push_back(l_virtualCaller);
   }
}


//*****************************************************************************
// Function: int CConference::OnConferenceDeviceOpenComplete()
// Description: Handle an open conference device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConference::OnConferenceDeviceOpenComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CConference::OnConferencePartyDeviceOpenComplete()
// Description: Handle an open conference party device completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConference::OnConferencePartyDeviceOpenComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CConference::RefreshVisibleParties()
// Description: Refresh the visible parties in a conference
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConference::RefreshVisibleParties()
{
   LogMsg("cc1", "RefreshVisibleParties - real party count:"+AsString(m_realPartyCount));

				// SKS to make sure multiple cnf work
   LB_FRAME_HANDLE* l_regionList = m_cnfConferenceDevice[GetConferenceID()]->RegionList();
   int l_regionIndex(0);
   int l_visiblePartyCount(0);
   int l_visiblePartyIndex(0);

   // if active talker mode, don't assign visible party to 1st region in 1 and 6 region layouts
   if (m_systemConfiguration->ActiveTalkerMode())
   {
      LogMsg("cc1", "RefreshVisibleParties - Active Talker Mode");
      switch (m_allowedLayouts[m_layout])
      {
         case 1:
         case 6:
            m_visiblePartyInfoList[l_visiblePartyIndex].PartyHandle = -1;
            m_visiblePartyInfoList[l_visiblePartyIndex].RegionHandle = l_regionList[l_regionIndex];
            l_visiblePartyIndex++;
            l_visiblePartyCount++;
            l_regionIndex++;
            break;
      }
   }

   // use real conference parties first
   int l_realPartyIndex(0);
				// SKS to make sure multiple cnf work
   for ( ;  (l_visiblePartyIndex < m_cnfConferenceDevice[GetConferenceID()]->RegionListCount())
            && (l_realPartyIndex<m_realPartyCount) 
            && (l_regionIndex<m_cnfConferenceDevice[GetConferenceID()]->RegionListCount())
         ;)
   {
      LogMsg( "R:" + AsString(l_visiblePartyIndex), AsHexString(m_realParties[l_realPartyIndex]->Device())+":"+AsHexString(l_regionList[l_regionIndex]));
      m_visiblePartyInfoList[l_visiblePartyIndex].PartyHandle = m_realParties[l_realPartyIndex]->Device();
      m_visiblePartyInfoList[l_visiblePartyIndex].RegionHandle = l_regionList[l_regionIndex];
      l_visiblePartyIndex++;
      l_visiblePartyCount++;
      l_realPartyIndex++;
      l_regionIndex++;
   }

   // use virtual conference parties second
   LogMsg("cc1", "RefreshVisibleParties - virtual party count:"+AsString(m_virtualPartyCount));
   int l_virtualPartyIndex(0);
				// SKS to make sure multiple cnf work
   for ( ;  (l_visiblePartyIndex < m_cnfConferenceDevice[GetConferenceID()]->RegionListCount())
            && (l_virtualPartyIndex<m_virtualPartyCount)
            && (l_regionIndex<m_cnfConferenceDevice[GetConferenceID()]->RegionListCount())
         ;)
   {
      LogMsg( "V:" + AsString(l_virtualPartyIndex), AsHexString(m_virtualParties[l_virtualPartyIndex]->Device())+":"+AsHexString(l_regionList[l_regionIndex]));
      m_visiblePartyInfoList[l_visiblePartyIndex].PartyHandle = m_virtualParties[l_virtualPartyIndex]->Device();
      m_visiblePartyInfoList[l_visiblePartyIndex].RegionHandle = l_regionList[l_regionIndex];
      l_visiblePartyIndex++;
      l_visiblePartyCount++;
      l_virtualPartyIndex++;
      l_regionIndex++;
   }

   // initialize rest of info list
				// SKS to make sure multiple cnf work
   for ( ;  (l_visiblePartyIndex < m_cnfConferenceDevice[GetConferenceID()]->RegionListCount())
            && (l_regionIndex<m_cnfConferenceDevice[GetConferenceID()]->RegionListCount())
         ;)
   {
     m_visiblePartyInfoList[l_visiblePartyIndex].PartyHandle = -1;
     m_visiblePartyInfoList[l_visiblePartyIndex].RegionHandle = l_regionList[l_regionIndex];
     l_visiblePartyIndex++;
     l_visiblePartyCount++;
     l_regionIndex++;
   }

   m_visiblePartyList.unCount = l_visiblePartyCount;
   m_visiblePartyList.pPartyList = m_visiblePartyInfoList;
   if (m_realPartyCount>0)
   {
				// SKS to make sure multiple cnf work
      return m_cnfConferenceDevice[GetConferenceID()]->SetVisibleParties(m_visiblePartyList);
   }
   return 0;
}

//*****************************************************************************
// Function: int CConference::CycleLayout()
// Description: Cycle the conference layout to the next configured layout
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConference::CycleLayout()
{
   int a_lastLayout = m_layout;
   m_layout = ++m_layout % m_allowedLayoutsCount;

				// SKS to make sure multiple cnf work
   int res = m_cnfConferenceDevice[GetConferenceID()]->SetConferenceLayout(m_allowedLayouts[m_layout]);

   return res;
}

//*****************************************************************************
// Function: int CConference::RotateParties()
// Description: Rotate the visible parties within the conference layout
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CConference::RotateParties()
{
   if (m_realPartyCount<m_allowedLayouts[m_layout])
   {
#ifdef WIN32
      SRL_DEVICE_HANDLE  l_visibleParties[MAX_REGION_COUNT];
#else
	   SRL_DEVICE_HANDLE l_visibleParties[m_visiblePartyList.unCount];
#endif

      int l_start(0);
      // if active talker mode, don't assign visible party to 1st region in 1 and 6 region layouts
      if (m_systemConfiguration->ActiveTalkerMode())
      {
         switch (m_allowedLayouts[m_layout])
         {
            case 1:
            case 6:
               l_start = 1;
               break;
         }
      }

      for (int i=l_start;i<m_visiblePartyList.unCount;i++)
      {
         l_visibleParties[i] = m_visiblePartyInfoList[i].PartyHandle;
      }

      for (int i=l_start+1;i<m_visiblePartyList.unCount;i++)
      {
         m_visiblePartyInfoList[i-1].PartyHandle = l_visibleParties[i];
      }
      m_visiblePartyInfoList[m_visiblePartyList.unCount-1].PartyHandle = l_visibleParties[l_start];

      if (m_realPartyCount>0)
      {
				// SKS to make sure multiple cnf work
         return m_cnfConferenceDevice[GetConferenceID()]->SetVisibleParties(m_visiblePartyList);
      }
      return 0;
   }
#ifdef WIN32
   CConferencePartyDevice* l_realParties[MAX_CONF_COUNT];
#else
   CConferencePartyDevice* l_realParties[m_realPartyCount];
#endif
   for (int i=0;i<m_realPartyCount;i++)
   {
      l_realParties[i] = m_realParties[i];
   }

   for (int i=1;i<m_realPartyCount;i++)
   {
      m_realParties[i-1] = l_realParties[i];
   }
   m_realParties[m_realPartyCount-1] = l_realParties[0];
   return RefreshVisibleParties();
}

//*****************************************************************************
// Function: void CConference::AllowedLayouts(int a_layoutCount, int *a_layouts)
// Description: Set the allowed conference video layouts
// Return: void 
// Parameters: int a_layoutCount 
//             int *a_layouts 
//*****************************************************************************
void CConference::AllowedLayouts(int a_layoutCount, int* a_layouts)
{
   if (a_layoutCount > MAX_LAYOUT_COUNT)
   {
      m_allowedLayoutsCount = MAX_LAYOUT_COUNT;
   }
   else
   {
      m_allowedLayoutsCount = a_layoutCount;
   }

   for (int i=0; i<m_allowedLayoutsCount; i++)
   {
      m_allowedLayouts[i] = a_layouts[i];
   }
}

//*****************************************************************************
// Function: void CConference::InitialLayout(int a_initialLayout)
// Description: Set the initial video conference layout
// Return: void 
// Parameters: int a_initialLayout 
//*****************************************************************************
void CConference::InitialLayout(int a_initialLayout)
{
   m_initialLayout = a_initialLayout;
}

//*****************************************************************************
// Function: void CConference::RefreshOverlays()
// Description: Refresh the image overlays.
// Return: void 
// Parameters: none 
//*****************************************************************************
void CConference::RefreshOverlays()
{
   std::list<CConferencePartyDevice*>::iterator l_partyDevice;
   for ( l_partyDevice=m_conferenceParties.begin();
         l_partyDevice!=m_conferenceParties.end();
         l_partyDevice++)
   {
      (*l_partyDevice)->DisplayCallerId();
   }
}

//*****************************************************************************
// Function: void CConference::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process an event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CConference::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case APPEVENT_CNF_SET_VIDEO_LAYOUT_COMPLETE:
         if (m_systemConfiguration->FeatureDelayTime())
         {
            LogMsg("cc1:", "Schedule feature timeout");
            CScheduler<CConference>::Instance()->ScheduleTask( this, &CConference::OnFeatureTimeout, m_systemConfiguration->FeatureDelayTime());
         }
// SKS
			if (m_cnfPartyCount>0)
      	{
         	RefreshVisibleParties();
			}
         break;

      case APPEVENT_CNF_SET_VISIBLE_PARTY_LIST_COMPLETE:
         RefreshOverlays();
         break;

      case APPEVENT_FEATURE_TIMEOUT:
         LogMsg("cc1:", "APPEVENT_FEATURE_TIMEOUT");
         break;

      default:
         LogMsg("cc1", "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}

//*****************************************************************************
// Function: int CConference::OnFeatureTimeout()
// Description: Process a feature timeout
// Return: int 
// Parameters: none 
//*****************************************************************************
int CConference::OnFeatureTimeout()
{
   LogMsg("cc1:", "Feature timeout");
   switch(m_systemConfiguration->DemoFeature())
   {
      case 1:
				// SKS to make sure multiple cnf work
         return m_cnfConferenceDevice[GetConferenceID()]->SetPipDemoFeature1Layout();
         break;

      case 2:
				// SKS to make sure multiple cnf work
         return m_cnfConferenceDevice[GetConferenceID()]->SetPipDemoFeature2Layout();
         break;

      case 3:
				// SKS to make sure multiple cnf work
         return m_cnfConferenceDevice[GetConferenceID()]->SetPipDemoFeature3Layout();
         break;

      default:
         CycleLayout();
   }
   return 0;
}

