/**
* @file CConference.h
* @brief Conference class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CConference_h
#define CConference_h

#ifdef WIN32
#include <process.h>
#endif
#include <string>
#include <vector>
#include <map>
#include <list>
#include "CConferenceDevice.h"
#include "CMediaDevice.h"
#include "CRtspConnector.h"

using namespace std;

#define MAX_LAYOUT_COUNT   5

class CSipSession;
class CConferenceBoardDevice;
class CConferenceDevice;
class CConferencePartyDevice;
class CSystemConfiguration;

//*****************************************************************************
// Class: ::CConference
// Description: Class to control a conference
//*****************************************************************************
class CConference : public AEventHandler
{
public:
   CConference();
   virtual ~CConference() {}

   void ProcessEvent(METAEVENT& a_metaEvent);

   static void SetConferenceBoard(CConferenceBoardDevice* a_cnfBoard);
   static CConferenceBoardDevice* GetConferenceBoard();
   static CConference* GetConference(std::string a_conferenceId);
   static void SystemConfiguration(CSystemConfiguration* a_systemConfiguration);
   static CSystemConfiguration* SystemConfiguration() {return m_systemConfiguration;}

   static void AllowedLayouts(int a_layoutCount, int* a_layouts);
   static void InitialLayout(int a_initialLayout);
   static int InitialLayout() {return m_initialLayout;};

	// SKS for multiple conference
	std::string GetConferenceID();
	void StartRecorder(std::string confID);
	void AddRecorderToConference();
	void RemoveRecorderFromConference();
	static bool IsConfRoomExist(std::string cnfID);
	static std::string GetDefaultConferenceID();
	void SetConferencePIN(std::string confID);
	std::string GetConferencePIN(std::string confID);
        int GetConferencePINLen(std::string confID);
	void OpenVirtualParties(std::string confID);
	void AddVirtualPartiesToConference();
	void RemoveVirtualPartiesFromConference();
	void RemoveRealPartiesFromConference();
	void RemoveAllPartiesFromConference();
#ifdef WIN32
	static unsigned __stdcall confTimerThreadFunc(void * pContext);
#else
	static void *confTimerThreadFunc(void *context);
#endif
	void createConferenceTimerThread();
	void SetRealPartiesSipSessionInfo(CSipSession *sipSession);
	void RemoveRealPartiesSipSessionInfo(CSipSession *sipSession);

   //JM
   int GetConferenceNumRealParties() {return m_realPartyCount;};
	
   static int OnConferenceBoardDeviceOpenComplete();
   static int OnConferenceDeviceOpenComplete();
   static int OnConferencePartyDeviceOpenComplete();

   int AddConferenceParty(CConferencePartyDevice* a_conferenceParty);
   int OnConferencePartyAdded(CConferencePartyDevice* a_conferenceParty);
   int RemoveConferenceParty(CConferencePartyDevice* a_conferenceParty);
   int OnConferencePartyRemoved(CConferencePartyDevice* a_conferenceParty);


   int CycleLayout();
   int RotateParties();

   int OnFeatureTimeout();

   int Stop() {return 0;}

   //JM - RTSP
   // Get the RtspConnector object.
    static RtspConnector& getRtspConnector() {return g_rtspConnector;}


private:

// SKS adding multiple conference up to 10 currently
//   static CConference* m_theConference[MAX_CONF_COUNT];
   static CConferenceBoardDevice* m_cnfConferenceBoardDevice;
// SKS adding multiple conference up to 10 currently
//   static CConferenceDevice* m_cnfConferenceDevice[MAX_CONF_COUNT];
   static CSystemConfiguration* m_systemConfiguration;

   int RefreshVisibleParties();
   void RefreshOverlays();

//	map<string,vector<string> > mapConfIDList;

	static map<std::string,CConference* > m_theConference;
	static map<string,CConferenceDevice* > m_cnfConferenceDevice;

	map<string,string > m_vConferencePIN;
//	vector<string> vecCnfPartyList;
	
#ifdef WIN32
   // Create and spawn a worker thread for SRL.
   HANDLE m_confTimerThreadH;
#else
   pthread_t m_confTimerThreadH;
#endif
	bool m_bConfTimeOut;

   int m_cnfPartyCount;
   CConferencePartyDevice* m_cnfParties[MAX_CONF_COUNT];

   int m_realPartyCount;
   CConferencePartyDevice* m_realParties[MAX_CONF_COUNT];

   int m_virtualPartyCount;
   CConferencePartyDevice* m_virtualParties[MAX_CONF_COUNT-1];


	bool m_bRecorderInConf;

   vector<CVirtualCaller*> v_VirtualCallerInConf;
   list<CSipSession*> m_listRealCallerInConf;

	CRecorder* m_recorder;

   int m_layout;

   static int m_allowedLayouts[MAX_LAYOUT_COUNT];
   static int m_allowedLayoutsCount;
   static int m_initialLayout;

   //JM - RTSP
   static RtspConnector g_rtspConnector;

   CNF_VISIBLE_PARTY_INFO m_visiblePartyInfoList[MAX_REGION_COUNT];
   CNF_VISIBLE_PARTY_LIST m_visiblePartyList;

   std::list<CConferencePartyDevice*> m_conferenceParties;
};

#endif

