/**
* @file CConferenceDevice.h
* @brief Conference Device class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CConferenceDevice_h
#define CConferenceDevice_h

#ifdef WIN32
#pragma warning(disable:4311 4312 4267 4018 4244 4996 4065 4101) 
#endif 

#include <string>
#include "srllib.h"
#include "cnflib.h"
#include "ADevice.h"

#define MAX_REGION_COUNT      	9
#define MAX_CONF_COUNT   			10
#define MAX_CONF_DURATION			0
#define BASE_CONF_ROOM_ID			3542	// DLGC
#define BASE_CONF_PIN				3542	// DLGC
#define MAX_PARTY_PER_CONF			30
#define MAX_STR_CONF_ROOM_ID		32

class CConferenceBoardDevice;
class CConferencePartyDevice;
class CVirtualCaller;
class CRecorder;

//*****************************************************************************
// Class: ::CConferenceDevice
// Description: Wrapper class for a CNF conference device
//*****************************************************************************
class CConferenceDevice : public ADevice
{
   friend class AConferenceDeviceState;
   friend class CConferenceDeviceClosedState;
   friend class CConferenceDeviceOpeningState;
   friend class CConferenceDeviceOpenedState;

public:
   CConferenceDevice(const std::string& a_deviceName, CConferenceBoardDevice* a_confBoardDevice);
   virtual ~CConferenceDevice() {}

   const std::string& DeviceName() {return m_deviceName;}
//	void SetDeviceName(std::string deviceName) {m_deviceName=deviceName;}
   long ID() {return(long) this;}

   void Observer(AEventHandler* a_observer) {m_observer = a_observer;}

   class AConferenceDeviceState& DeviceState() {return *m_deviceState;}
   void SetDeviceStateTo(AConferenceDeviceState* state);

   int Open();
   int Close();
   int OnCnfOpenConferenceComplete(METAEVENT& a_metaEvent);

   int OnCnfSetVideoLayoutComplete(METAEVENT& a_metaEvent);

   int AddConferenceParty(CConferencePartyDevice* a_conferenceParty);
   int OnConferencePartyAdded(CConferencePartyDevice* a_conferenceParty);
   int SetVisibleParties(CNF_VISIBLE_PARTY_LIST& a_visiblePartyList);
   int RemoveConferenceParty(CConferencePartyDevice* a_conferenceParty);
   int OnConferencePartyRemoved(CConferencePartyDevice* a_conferenceParty);
   int OnCnfSetVisiblePartyList(METAEVENT& a_metaEvent);
   void ProcessEvent(METAEVENT& a_metaEvent);

   int RegionListCount() {return m_regionListCount;}
   LB_FRAME_HANDLE* RegionList() {return m_regionList;}

   int SetConferenceLayout(int a_tiles);
   int SetPipDemoFeature1Layout();
   int SetPipDemoFeature2Layout();
   int SetPipDemoFeature3Layout();
   int SetLayout(LB_FRAME_HANDLE a_layoutTemplateHandle);

   bool ShouldClose() {return true;}

   int Stop() {return 0;}

private:
   // force initialization
   CConferenceDevice();

   // make non-copyable
   CConferenceDevice(const CConferenceDevice&);
   const CConferenceDevice& operator=(const CConferenceDevice&);

   int PrimCnfOpenConference();
   int PrimOnCnfOpenConferenceComplete(METAEVENT& a_metaEvent);
   int PrimCnfCloseConference();

   int RefreshVisibleParties();

	std::string m_confID;

   AConferenceDeviceState* m_deviceState;
   std::string m_deviceName;
   CConferenceBoardDevice* m_cnfBoardDevice;
   AEventHandler* m_observer;
   int m_layoutTileCount;

   LB_FRAME_HANDLE m_layoutHandle;
//   LB_FRAME_HANDLE m_templateLayoutHandle;
   CNF_VIDEO_LAYOUT_INFO m_layoutInfo;
   LB_FRAME_HANDLE m_regionList[MAX_REGION_COUNT];
//   LB_FRAME_HANDLE m_templateRegionList[MAX_REGION_COUNT];
   unsigned int m_regionListCount;
//   unsigned int m_templateRegionListCount;

   int m_featureState1;
   int m_featureState2;
};

#endif


