/**
* @file CConferencePartyDevice.h
* @brief Conference Party Device class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CConferencePartyDevice_h
#define CConferencePartyDevice_h

#include <string>
#include "srllib.h"
#include "cnflib.h"
#include "ADevice.h"
#include "port_connect.h"

class CConferenceBoardDevice;
class CSipSession;

//*****************************************************************************
// Class: ::CConferencePartyDevice
// Description: Wrapper class for a CNF confernce party device
//*****************************************************************************
class CConferencePartyDevice : public ADevice
{
   friend class AConferencePartyDeviceState;
   friend class CConferencePartyDeviceClosedState;
   friend class CConferencePartyDeviceOpeningState;
   friend class CConferencePartyDeviceOpenedState;

public:
   CConferencePartyDevice(const std::string& a_deviceName, CConferenceBoardDevice* a_confBoardDevice, AEventHandler* a_observer);
   virtual ~CConferencePartyDevice() {}

   const std::string& DeviceName() {return m_deviceName;}
   long ID() {return(long) this;}

   class AConferencePartyDeviceState& DeviceState() {return *m_deviceState;}
   void SetDeviceStateTo(AConferencePartyDeviceState* state);
   void Observer(AEventHandler* a_observer) {m_observer = a_observer;}
   void SetVirtualParty(AEventHandler* a_virtualParty) {m_virtualParty=a_virtualParty;}
   AEventHandler* GetVirtualParty() {return m_virtualParty;}
   bool IsVirtualParty() {return(m_virtualParty != 0);}

	void SetRecorderParty(AEventHandler* a_recorderParty) {m_recorderParty=a_recorderParty;}
   AEventHandler* GetRecorderParty() {return m_recorderParty;}
   bool IsPartyRecorder() {return(m_recorderParty != 0);}

   int Open();
   int Close();
   int OnCnfOpenConferencePartyComplete(METAEVENT& a_metaEvent);
   int OnDevConnected(METAEVENT& a_metaEvent);
   int OnDevDisconnected(METAEVENT& a_metaEvent);
   int OnCnfPartyAdded(METAEVENT& a_metaEvent);
   int OnCnfPartyRemoved(METAEVENT& a_metaEvent);
   int OnCnfRemovePartyFailed(METAEVENT& a_metaEvent);

   int Stop() {return 0;}

   void ProcessEvent(METAEVENT& a_metaEvent);

   int GetReceivePortInfo();
   int OnGetReceivePortInfoComplete(METAEVENT& a_metaEvent);
   int GetTransmitPortInfo();
   int OnGetTransmitPortInfoComplete(METAEVENT& a_metaEvent);

   const DM_PORT_INFO_LIST& RxPortInfoList() {return m_rxPortInfoList;}
   const DM_PORT_INFO_LIST& TxPortInfoList() {return m_txPortInfoList;}

   int OnPortConnected(METAEVENT& a_metaEvent);
   int OnPortDisconnected(METAEVENT& a_metaEvent);

   int OnCnfGetVisiblePartyList(METAEVENT& a_metaEvent);

   int DisplayConferenceLayout(int a_layout);
   int DisplayCallerId();
   void SetCallerId(int a_callerId);

   bool ShouldClose() {return true;}

private:
   // force initialization
   CConferencePartyDevice();

   // make non-copyable
   CConferencePartyDevice(const CConferencePartyDevice&);
   const CConferencePartyDevice& operator=(const CConferencePartyDevice&);

   int PrimCnfOpenConferenceParty();
   int PrimOnCnfOpenConferencePartyComplete(METAEVENT& a_metaEvent);
   int PrimCnfCloseConferenceParty();
   int PrimOnDevConnected(METAEVENT& a_metaEvent);
   int PrimOnDevDisconnected(METAEVENT& a_metaEvent);
   int PrimOnCnfPartyAdded(METAEVENT& a_metaEvent);
   int PrimOnCnfPartyRemoved(METAEVENT& a_metaEvent);
   int PrimOnCnfRemovePartyFailed(METAEVENT& a_metaEvent);

   AConferencePartyDeviceState* m_deviceState;
   std::string m_deviceName;
   CConferenceBoardDevice* m_cnfBoardDevice;
   AEventHandler* m_observer;
   AEventHandler* m_virtualParty;
   AEventHandler* m_recorderParty;

   DM_PORT_INFO_LIST m_rxPortInfoList;
   DM_PORT_INFO_LIST m_txPortInfoList;

};

#endif
