/**
* @file CGcManager.h
* @brief GlobalCall Manager class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CGcManager_h
#define CGcManager_h

#include "CSystemConfiguration.h"

#include "gcip.h"
#include "srllib.h"

#include <map>
#include <set>

#include "ADevice.h"
#include "CScheduler.h"
#include "CMediaDevice.h"

class CGcSipDevice;
class CSipSession;
class CConference;

typedef std::map<long, ADevice*> SystemDeviceMap;
typedef std::set<ADevice*> DeviceSet;
typedef std::set<CSipSession*> SipSessionSet;

//*****************************************************************************
// Class: ::CGcManager
// Description: Manage GlobalCall startup, event handling and shutdown
//*****************************************************************************
class CGcManager
{
public:
   CGcManager(CSystemConfiguration& a_systemConfiguration);
   virtual ~CGcManager() {}

   const CSystemConfiguration& SystemConfiguration() {return m_systemConfiguration;}

   int Start();
   void Stop();
   void ProcessEvents();
   void InitializeApplicationObjects();
        
   static void RegisterDevice(ADevice* a_device);
   static void RegisterDevice(long a_id, ADevice* a_device);
   static void UnregisterDevice(long a_id);

   static void AddOpenDevice(ADevice* a_device);
   static void RemoveOpenDevice(ADevice* a_device);
   static bool OpenDevicesExist();

   static void AddActiveSipSession(CSipSession* a_sipSession);
   static void RemoveActiveSipSession(CSipSession* a_sipSession);
   static bool ActiveSipSessionsExist();

private:
   CScheduler<CMediaDevice>& m_mmScheduler;
   CScheduler<CConference>& m_featureScheduler;
   int m_sipPort;
   IPCCLIB_START_DATA m_libData;
   IP_VIRTBOARD m_virtBoard[1];
   CCLIB_START_STRUCT m_ccLibStart[2];
   GC_START_STRUCT m_gcLibStart;

   static SystemDeviceMap m_systemDevices;
   static DeviceSet m_openDevices;
   static SipSessionSet m_activeSipSessions;

   CSystemConfiguration& m_systemConfiguration;
};

#endif

