/**
* @file CGcSipDevice.h
* @brief GlobalCall Device class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CGcSipDevice_h
#define CGcSipDevice_h

#include "srllib.h"
#include "gclib.h"
#include "dtilib.h"

#include <string>

#include "sdpAPI.h"

#include "ADevice.h"

class AGcDeviceState;
class AGcSipCallState;
class CSipSession;

//*****************************************************************************
// Class: ::CGcSipDevice
// Description: Wrapper for GlobalCall SIP device
//*****************************************************************************
class CGcSipDevice : public ADevice
{
   friend class AGcSipDeviceState;
   friend class CGcSipDeviceBlockedState;
   friend class CGcSipDeviceClosedState;
   friend class CGcSipDeviceOpeningState;
   friend class CGcSipDeviceSettingConfigDataState;
   friend class CGcSipDeviceOpenedWaitingForUnblockedState;
   friend class AGcSipDeviceUnblockedState;
   friend class CGcSipDeviceSipRegisteringState;
   friend class CGcSipDeviceSipDeregisteringState;
   friend class CGcSipDeviceResettingState;
   friend class CGcSipDeviceActiveState;

   friend class AGcSipCallState;
   friend class CGcSipCallNullState;
   friend class AGcSipCallActiveState;
   friend class CGcSipCallOfferedState;
   friend class CGcSipCallSendingProceedingState;
   friend class CGcSipCallOfferedProceedingState;
   friend class CGcSipCallAcceptingCallState;
   friend class CGcSipCallAcceptedState;
   friend class CGcSipCallAnsweringCallState;
   friend class CGcSipCallDialingState;
   friend class CGcSipCallProceedingState;
   friend class CGcSipCallAlertingState;
   friend class CGcSipCallWaitingFor200OkState;
   friend class CGcSipCallReceived200OkState;
   friend class CGcSipCallSendingAckState;
   friend class CGcSipCallConnectedState;
   friend class AGcSipCallTeardownState;
   friend class CGcSipCallDisconnectedState;
   friend class CGcSipCallDroppingCallState;
   friend class CGcSipCallDroppedState;
   friend class CGcSipCallReleasingCallState;

public:
   CGcSipDevice(const std::string& a_deviceName, const std::string& a_sipRegistrar, const std::string& a_sipAddress);

   int Stop() {return 0;}

   void ProcessEvent(METAEVENT& a_metaEvent);

   const std::string& DeviceName() {return m_deviceName;}

   long ID() {return(long) this;}

   CRN& CallReference() {return m_crn;}
   void CallReference(CRN crn) {m_crn = crn;}

   int DropCallCause() {return m_dropCallCause;}
   void DropCallCause(int a_cause) {m_dropCallCause = a_cause;}

   class AGcSipDeviceState& DeviceState() {return *m_deviceState;}
   void SetDeviceStateTo(AGcSipDeviceState* a_state); 

   AGcSipCallState& CallState() {return *m_callState;}
   void SetCallStateTo(AGcSipCallState* a_state);

   bool WaitForCallIssued() {return m_waitForCallIssued;}
   void WaitForCallIssued(bool a_flag) {m_waitForCallIssued = a_flag;}

   bool GotEarlyGcUnblockedEvent() {return m_earlyGcUnblockedEvent;}
   void GotEarlyGcUnblockedEvent(bool a_flag) {m_earlyGcUnblockedEvent = a_flag;}

   bool IsHMPBoardDevice() {return DeviceName() == "iptB1";}

   int Open();
   int GcSipRegister();
   int GcSipSetAuthentication(); //JM Added 5/2011, but not tested or enabled.
   int GcSipDeregister();
   int GcResetLineDevice();
   int Close();

   int ProcessNextCall();

   int GcWaitForCall();
   int GcSendProceeding();
   int GcAcceptCall();
   int GcAnswerCall();
   int GcSipAck();
   int GcDropCall();
   int GcReleaseCall();

   int GcSendIFrameRequest();

   int OnGcOpened(METAEVENT& a_metaEvent);
   int OnGcUnblocked(METAEVENT& a_metaEvent);
   int OnGcBlocked(METAEVENT& a_metaEvent);
   int OnGcSipRegisterComplete(METAEVENT& a_metaEvent);
   int OnGcResetLineDevice(METAEVENT& a_metaEvent);
   int OnGcSetConfigData(METAEVENT& a_metaEvent);

   int OnGcCallDisconnected(METAEVENT& a_metaEvent);
   int OnGcCallProceeding(METAEVENT& a_metaEvent);
   int OnGcCallProceedingComplete(METAEVENT& a_metaEvent);
   int OnGcCallAlerting(METAEVENT& a_metaEvent);
   int OnGcSip200Ok(METAEVENT& a_metaEvent);
   int OnGcCallDropped(METAEVENT& a_metaEvent);
   int OnGcCallReleased(METAEVENT& a_metaEvent);
   int OnGcCallOffered(METAEVENT& a_metaEvent);
   int OnGcCallAccepted(METAEVENT& a_metaEvent);
   int OnGcCallAnswered(METAEVENT& a_metaEvent);
   int OnGcSipAck(METAEVENT& a_metaEvent);

   void GcCallInfoMsg();

   int OnGcCallInfo(METAEVENT& a_metaEvent);

   const std::string& SipCallId() {return m_sipCallId;}
   void SipCallId(const std::string& a_sipCallId) {m_sipCallId = a_sipCallId;}
   const std::string& SipCallTo() {return m_sipCallTo;}
   void SipCallTo(const std::string& a_sipCallTo) {m_sipCallTo = a_sipCallTo;}
   const std::string& SipCallFrom() {return m_sipCallFrom;}
   void SipCallFrom(const std::string& a_sipCallFrom) {m_sipCallFrom = a_sipCallFrom;}
   const std::string& SipCallContact() {return m_sipCallContact;}
   void SipCallContact(const std::string& a_sipCallContact) {m_sipCallContact = a_sipCallContact;}
   const std::string& SipCallRequest() {return m_sipCallRequest;}
   void SipCallRequest(const std::string& a_sipCallRequest) {m_sipCallRequest = a_sipCallRequest;}
   const std::string& SipCallOfferSdp() {return m_sipCallOfferSdp;}
   void SipCallOfferSdp(const std::string&  a_sipCallOfferSdp) {m_sipCallOfferSdp = a_sipCallOfferSdp;}
   const std::string& SipCallAnswerSdp() {return m_sipCallAnswerSdp;}
   void SipCallAnswerSdp(const std::string&  a_sipCallAnswerSdp) {m_sipCallAnswerSdp = a_sipCallAnswerSdp;}

   void SipSession(CSipSession* a_sipSession) {m_sipSession = a_sipSession;}

   sdpSessionDescription& OfferSdp() {return m_offerSdp;}
   sdpSessionDescription& AnswerSdp() {return m_answerSdp;}

   bool ShouldClose() {return true;}

private:
   // force initialization
   CGcSipDevice();

   // make non-copyable
   CGcSipDevice(const CGcSipDevice&);
   const CGcSipDevice& operator=(const CGcSipDevice&);

   // member variables
   AGcSipDeviceState* m_deviceState;
   AGcSipCallState* m_callState;
   std::string m_deviceName;
   std::string m_sipRegistrar;
   std::string m_sipAddress;

   CRN m_crn;
   int m_dropCallCause;
   bool m_isBlocked;
   bool m_waitForCallIssued;

   bool m_earlyGcUnblockedEvent;

   char* m_sipInfoType;
   char* m_sipInfoData;

   std::string m_sipCallId;
   std::string m_sipCallTo;
   std::string m_sipCallFrom;
   std::string m_sipCallContact;
   std::string m_sipCallRequest;
   std::string m_sipCallOfferSdp;
   std::string m_sipCallAnswerSdp;

   sdpSessionDescription m_offerSdp;
   sdpSessionDescription m_answerSdp;

   CSipSession* m_sipSession;

   // primative functions
   int PrimGcOpen();
   int PrimGcSipRegister();
   int PrimGcSipDeregister();
   int PrimGcResetLineDevice();
   int PrimGcClose();

   int PrimProcessNextCall();

   int PrimGcWaitForCall();
   int PrimGcAcceptCall();
   int PrimGcSendProceeding();
   int PrimGcAnswerCall();
   int PrimGcSipAck();
   int PrimGcDropCall();
   int PrimGcReleaseCall();

   int PrimGcSendIFrameRequest();

   int PrimOnGcOpened(METAEVENT& a_metaEvent);
   int PrimOnGcUnblocked(METAEVENT& a_metaEvent);
   int PrimOnGcBlocked(METAEVENT& a_metaEvent);
   int PrimOnGcSipRegisterComplete(METAEVENT& a_metaEvent);
   int PrimOnGcSipDeregisterComplete(METAEVENT& a_metaEvent);
   int PrimOnGcResetLineDevice(METAEVENT& a_metaEvent);
   int PrimOnGcSetConfigData(METAEVENT& a_metaEvent);

   int PrimOnGcCallDisconnected(METAEVENT& a_metaEvent);
   int PrimOnGcDropCall(METAEVENT& a_metaEvent);
   int PrimOnGcReleaseCall(METAEVENT& a_metaEvent);
   int PrimOnGcCallProceeding(METAEVENT& a_metaEvent);
   int PrimOnGcCallProceedingComplete(METAEVENT& a_metaEvent);
   int PrimOnGcCallAlerting(METAEVENT& a_metaEvent);
   int PrimOnGcSip200Ok(METAEVENT& a_metaEvent);
   int PrimOnGcCallDropped(METAEVENT& a_metaEvent);
   int PrimOnGcCallReleased(METAEVENT& a_metaEvent);
   int PrimOnGcCallOffered(METAEVENT& a_metaEvent);
   int PrimOnGcCallAccepted(METAEVENT& a_metaEvent);
   int PrimOnGcCallAnswered(METAEVENT& a_metaEvent);
   int PrimOnGcSipAckFromAnswer(METAEVENT& a_metaEvent);

	void GetConferenceIDFromURI();
   void GetSipCallInfo(METAEVENT& a_metaEvent);
   void ParseSdp(sdpSessionDescription& a_sdp, char* a_dataBuffer);
   void GetRegisterationInfo(METAEVENT &a_metaEvent);

   static const char* m_iFrameRequest;
};

#endif


