/**
* @file CMediaDevice.h
* @brief Multimedia Device class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CMediaDevice_h
#define CMediaDevice_h

#include "srllib.h"
#include "gclib.h"
#include "mmlib.h"
#include "port_connect.h"
#include "videodefs.h"
#include "lb_mtklib.h"

#include "ADevice.h"

#include <string>


#ifdef USE_MTK_MP
#include "mp_mtklib.h"
#endif

#define MMRSC_DCI_SIZE 255

static int MM_CLOSE_PLAY      = 0x0081;
static int MM_CLOSE_RECORD    = 0x0082;
static int MM_CLOSE_STOP_ACK  = 0x0083;
static int MM_CLOSE_COMPLETE  = 0x0084;

class AMediaDeviceState;
class CSipSession;

//*****************************************************************************
// Class: ::CMediaDevice
// Description: Wrapper class for a MultiMedia device
//*****************************************************************************
class CMediaDevice : public ADevice
{
   friend class AMediaDeviceState;
   friend class CMediaDeviceClosedState;
   friend class CMediaDeviceOpeningState;
   friend class CMediaDeviceIdleState;
   friend class CMediaDevicePlayingState;
   friend class CMediaDeviceRecordingState;
   friend class CMediaDeviceClosingState;

public:
   CMediaDevice(const std::string& a_deviceName);

   const std::string& DeviceName() {return m_deviceName;}

   long ID() {return(long) this;}
	void GenerateEvent(int aEvent){m_observer->PostEvent(aEvent);}

   AMediaDeviceState& DeviceState() {return *m_deviceState;}
   int ClosingSubState() {return m_closingSubState;}

   void SetDeviceStateTo(AMediaDeviceState* state);
   void Observer(AEventHandler* a_observer) {m_observer = a_observer;}

   const DM_PORT_INFO_LIST& RxPortInfoList() {return m_rxPortInfoList;}
   const DM_PORT_INFO_LIST& TxPortInfoList() {return m_txPortInfoList;}

   int Stop() {return MmStop();}

   void ProcessEvent(METAEVENT& a_metaEvent);

   int Open();
   int OnMmOpened(METAEVENT& a_metaEvent);
   int Close();
   int GetReceivePortInfo();
   int OnGetReceivePortInfoComplete(METAEVENT& a_metaEvent);
   int GetTransmitPortInfo();
   int OnGetTransmitPortInfoComplete(METAEVENT& a_metaEvent);
   int OnPortConnected(METAEVENT& a_metaEvent);
   int OnPortDisconnected(METAEVENT& a_metaEvent);
   int MmPlay();
   int OnMmPlayComplete(METAEVENT& a_metaEvent);
   int OnMmPlayTimeoutComplete();
   int OnMmPlayFail(METAEVENT& a_metaEvent);
   int MmStop();
   int OnMmStopAck(METAEVENT& a_metaEvent);
// SKS
   int MmRecord();
   int OnMmRecordComplete(METAEVENT& a_metaEvent);
   int OnMmRecordTimeoutComplete();
   int OnMmRecordFail(METAEVENT& a_metaEvent);

   static std::string GetNextMediaDeviceName();

   int SetClip(const std::string& a_path, const std::string& a_clip);

#ifdef USE_MTK_MP
   int ParseFileInfo (MP_FILE_INFO_DESC *pFileInfoDesc, MM_VIDEO_CODEC *pVideoCodec, MM_AUDIO_CODEC *pAudioCodec);
#endif

   void PrintMMCodecParameters (MM_VIDEO_CODEC *pVideoCodec, MM_AUDIO_CODEC *pAudioCodec);
   const std::string& GetClip() {return m_audioFile;}
   bool HaveClip() {return m_haveClip;}

   bool ShouldClose() {return true;}

   void SetRecordClip(const std::string& a_path, const std::string& a_clip);
   const std::string& GetRecordClip() {return m_recAudioFile;}
   void SetRecordCoding(eMM_VIDEO_CODING a_recVideoCoding);

private:
   // force initialization
   CMediaDevice();

   // make non-copyable
   CMediaDevice(const CMediaDevice&);
   const CMediaDevice& operator=(const CMediaDevice&);

   // primative functions
   int PrimMmOpen();
   int PrimMmClose();
   int PrimOnMmOpened(METAEVENT& a_metaEvent);
   int PrimMmPlay();
   int PrimOnMmPlayComplete(METAEVENT& a_metaEvent);
   int PrimOnMmPlayTimeoutComplete();
   int PrimOnMmPlayFail(METAEVENT& a_metaEvent);
   int PrimMmStop(int operation);
   int PrimOnMmStopAck(METAEVENT& a_metaEvent);

// SKS
   int PrimMmRecord();
   int PrimOnMmRecordComplete(METAEVENT& a_metaEvent);
   int PrimOnMmRecordTimeoutComplete();
   int PrimOnMmRecordFail(METAEVENT& a_metaEvent);

   // member variables
   AMediaDeviceState* m_deviceState;
   int m_closingSubState;
   std::string m_deviceName;

   AEventHandler* m_observer;

   DM_PORT_INFO_LIST m_rxPortInfoList;
   DM_PORT_INFO_LIST m_txPortInfoList;

   MM_PLAY_INFO m_playInfo;
   MM_PLAY_RECORD_LIST m_playList[2];
   MM_MEDIA_ITEM_LIST m_audioMediaItemList;
   MM_MEDIA_ITEM_LIST m_videoMediaItemList;
   MM_VIDEO_CODEC m_videoCodec;
   MM_AUDIO_CODEC m_audioCodec;

   std::string m_audioFile;
   std::string m_videoFile;
   bool m_haveClip;
   bool m_is3gp;
// SKS
   std::string m_recAudioFile;
   std::string m_recVideoFile;
   eMM_VIDEO_CODING m_recVideoCoding;
   MM_RECORD_INFO m_recordInfo;
   MM_PLAY_RECORD_LIST m_recordList[2];
   MM_MEDIA_ITEM_LIST m_recAudioMediaItemList;
   MM_MEDIA_ITEM_LIST m_recVideoMediaItemList;
   MM_VIDEO_CODEC m_recVideoCodec;
   MM_AUDIO_CODEC m_recAudioCodec;

   static int m_nextMediaDevice;
};

#endif

