/**
* @file imgovl.cpp
* @brief COverlay class implementation
* @date Mar. 27, 2008
*
* DIALOGIC CONFIDENTIAL
* Copyright  2008 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
#include <stdio.h>
#endif
#include "COverlay.h"

/*
typedef struct tagTestOverlay
{
    eMTK_IMAGE_FORMAT imageFormat;
    MTK_HANDLE hImageTemplate; //mtk_CreateImageTemplate()
    MTK_HANDLE hMedia;         //mtk_CreateMediaFileTemplate()
    OB_OVERLAY_HANDLE hOvlTemplate; //ob_CreateImageOverlayTemplate()
    OB_OVERLAY_HANDLE hDeviceOvlSnapshot; //handle provided by SMEV_ADD_OVERLAY event as a result of calling sm_AddOverlays()
// mtk_SetFramePosition(hBoundingFrameTemplate, x, y, ePositionType)
    MTK_FRAME_HANDLE  hBoundingFrameTemplate;
    float framePosition_x;
    float framePosition_y;
    eMTK_POSITION_TYPE ePositionType;

// mtk_SetFrameSize(hBoundingFrameTemplate, frameWidth, frameHeight, eSizeType)
    float frameWidth;
    float frameHeight;
    eMTK_SIZE_TYPE eSizeType;

// hBoundingSnapshot = ob_SetOverlayBoundingFrame(hOvlTemplate, hBoundingFrameTemplate)
    MTK_FRAME_HANDLE  hBoundingSnapshot;

// set overlay duration
// ob_SetOverlayDuration(pTestOverlay->hOvlTemplate, unDuration)
    unsigned int duration_ms;

    unsigned int imgHeight;
    unsigned int imgWidth;
    char imgName[255];
}TestOverlay, *PTestOverlay;
*/

TestOverlay gOverlays[] = { {eMTK_IMAGE_FORMAT_YUV, -1, -1,
   MTK_ERROR, MTK_ERROR, MTK_ERROR,
   10.0, 15.0, eMTK_POSITION_TYPE_PERCENT, 25.0, 20.0, eMTK_SIZE_TYPE_PERCENT,
   MTK_ERROR,MTK_INFINITE,0,0,"/export/mmbb_xcode_demo/image.yuv"},
{eMTK_IMAGE_FORMAT_YUV, -1, -1,
   MTK_ERROR, MTK_ERROR, MTK_ERROR,
   MTK_JUSTIFY_RIGHT, MTK_JUSTIFY_CENTER,
   eMTK_POSITION_TYPE_JUSTIFICATION,
   40.0, 35.0, eMTK_SIZE_TYPE_PERCENT,
   MTK_ERROR,MTK_INFINITE,0,0,""},
{eMTK_IMAGE_FORMAT_JPEG, -1, -1,
   MTK_ERROR, MTK_ERROR, MTK_ERROR,
   MTK_JUSTIFY_RIGHT, MTK_JUSTIFY_CENTER,
   eMTK_POSITION_TYPE_JUSTIFICATION,
   40.0, 35.0, eMTK_SIZE_TYPE_PERCENT,
   MTK_ERROR,MTK_INFINITE,0,0,"/export/hello.jpg"},
{eMTK_IMAGE_FORMAT_JPEG, -1, -1,
   MTK_ERROR, MTK_ERROR, MTK_ERROR,
   10.0, 10.0, eMTK_POSITION_TYPE_PIXEL,
   76.0, 33.0, eMTK_SIZE_TYPE_PIXEL,
   MTK_ERROR,300,0,0,"srcMedia/hello.jpg"},
{eMTK_IMAGE_FORMAT_JPEG, -1, -1,
   MTK_ERROR, MTK_ERROR, MTK_ERROR,
   MTK_JUSTIFY_RIGHT, MTK_JUSTIFY_RIGHT, eMTK_POSITION_TYPE_JUSTIFICATION,
   100.0, 100.0, eMTK_SIZE_TYPE_PERCENT,
   MTK_ERROR,300,0,0,"srcMedia/hello.jpg"}
};

//*****************************************************************************
// Function: COverlay::COverlay(eMTK_IMAGE_FORMAT imgFormat, eMTK_SIZE_TYPE sizeType, unsigned int width, unsigned int height, const char *szImgFileSrc)
// Description: Initializing constructor
// Return:  
// Parameters: eMTK_IMAGE_FORMAT imgFormat 
//             eMTK_SIZE_TYPE sizeType 
//             unsigned int width 
//             unsigned int height 
//             const char *szImgFileSrc 
//*****************************************************************************
COverlay::COverlay(eMTK_IMAGE_FORMAT imgFormat, eMTK_SIZE_TYPE sizeType,
                   unsigned int width, unsigned int height,
                   const char *szImgFileSrc)
{
   m_pTestOverlayInUse = NULL;

   if (szImgFileSrc != NULL)
   {
      m_imgFileSrc.assign (szImgFileSrc);
   }

   if (!(imgFormat == eMTK_IMAGE_FORMAT_YUV
         || imgFormat == eMTK_IMAGE_FORMAT_JPEG))
   {
	  LogMsg(Name(), "Invalid image format: " + AsString(imgFormat));
	  LogMsg(Name(), "   Supported formats are eMTK_IMAGE_FORMAT_YUV and eMTK_IMAGE_FORMAT_JPEG");
      throw "Invalid Image format";
   }

   m_pTestOverlayInUse = new TestOverlay;
   m_pTestOverlayInUse->hMedia = -1;
   strcpy(m_pTestOverlayInUse->imgName,m_imgFileSrc.c_str());
   m_pTestOverlayInUse->frameWidth = width;
   m_pTestOverlayInUse->frameHeight = height;
   m_pTestOverlayInUse->imageFormat = imgFormat;

   switch (imgFormat)
   {
   case eMTK_IMAGE_FORMAT_YUV:
      CreateYUVImageTemplate ();
      if (ApplyMediaToTemplate () != -1)
      {
         CreateOverlayTemplates(m_pTestOverlayInUse, 1);
      }
      break;
   case eMTK_IMAGE_FORMAT_JPEG:
      CreateJPEGImageTemplate();
      if (ApplyMediaToTemplate () != -1)
      {
         CreateOverlayTemplates(m_pTestOverlayInUse, 1);
      }
      break;
   }
}

//*****************************************************************************
// Function: COverlay::COverlay(const TestOverlay *pOverlay)
// Description: Initializing constructor
// Return:  
// Parameters: const TestOverlay *pOverlay 
//*****************************************************************************
COverlay::COverlay(const TestOverlay *pOverlay)
{
   int rc = 0;
   m_pTestOverlayInUse = new TestOverlay;
   memcpy(m_pTestOverlayInUse, pOverlay, sizeof(TestOverlay));
   m_pTestOverlayInUse->hMedia = -1;
   if (m_pTestOverlayInUse->imgName != NULL)
   {
      m_imgFileSrc.assign (m_pTestOverlayInUse->imgName);
   }

   if (!(m_pTestOverlayInUse->imageFormat == eMTK_IMAGE_FORMAT_YUV
         || m_pTestOverlayInUse->imageFormat == eMTK_IMAGE_FORMAT_JPEG))
   {
	  LogMsg(Name(), "Invalid image format: " + AsString(m_pTestOverlayInUse->imageFormat));
	  LogMsg(Name(), "   Supported formats are eMTK_IMAGE_FORMAT_YUV and eMTK_IMAGE_FORMAT_JPEG");
      throw "Invalid Image format";
   }

   switch (m_pTestOverlayInUse->imageFormat)
   {
   case eMTK_IMAGE_FORMAT_YUV:
      rc = CreateYUVImageTemplate ();
      break;
   case eMTK_IMAGE_FORMAT_JPEG:
      rc = CreateJPEGImageTemplate();
      break;
   }
   if (rc != -1)
   {
      if (ApplyMediaToTemplate () != -1)
      {
         CreateOverlayTemplates(m_pTestOverlayInUse, 1);
      }
   }
}

//*****************************************************************************
// Function: COverlay::COverlay(eMTK_IMAGE_FORMAT imgFormat, const char *szImgFile)
// Description: Initializing constructor
// Return:  
// Parameters: eMTK_IMAGE_FORMAT imgFormat 
//             const char *szImgFile 
//*****************************************************************************
COverlay::COverlay(eMTK_IMAGE_FORMAT imgFormat, const char *szImgFile)
{
}

//*****************************************************************************
// Function: COverlay::~COverlay()
// Description: Destructor
// Return:  
// Parameters: none 
//*****************************************************************************
COverlay::~COverlay()
{
   if (m_pTestOverlayInUse)
   {
      // MMP 2.0 build 70 has a problem with mtk_DestroyMediaTemplate()
      // known ot be fixed in build 76
      // mtk_DestroyMediaTemplate(m_pTestOverlayInUse->hMedia);
      ob_DestroyOverlayTemplate(m_pTestOverlayInUse->hOvlTemplate);
      mtk_DestroyFrameTemplate(m_pTestOverlayInUse->hBoundingFrameTemplate);
   }
}

//*****************************************************************************
// Function: int COverlay::CreateJPEGImageTemplate()
// Description: Create a JPEG image template
// Return: int 
// Parameters: none 
//*****************************************************************************
int COverlay::CreateJPEGImageTemplate ()
{
   if ((m_pTestOverlayInUse->hImageTemplate = mtk_CreateImageTemplate (m_pTestOverlayInUse->imageFormat)) == MTK_ERROR)
   {
      LogMsg(Name(), "ERROR: CreateJPEGImageTemplate returned ERROR from mtk_CreateImageTemplate");
      outputLastMTKError ();
      return -1;
   }
   else
   {
      
	  LogMsg(Name(), "Received media JPEG image handle: " + AsString(m_pTestOverlayInUse->hImageTemplate));
      m_bitMapLength = m_pTestOverlayInUse->imgWidth*m_pTestOverlayInUse->imgHeight;
   }
   return 0;
}

//*****************************************************************************
// Function: int COverlay::CreateYUVImageTemplate()
// Description: Create a YUV image template
// Return: int 
// Parameters: none 
//*****************************************************************************
int COverlay::CreateYUVImageTemplate ()
{
   if ((m_pTestOverlayInUse->hImageTemplate = mtk_CreateImageTemplate (m_pTestOverlayInUse->imageFormat)) == MTK_ERROR)
   {
      LogMsg(Name(), "ERROR: CreateYUVImageTemplate returned ERROR from mtk_CreateImageTemplate");
      outputLastMTKError ();
      return -1;
   }
   else
   {
      LogMsg(Name(), "Received media YUV image handle: " + AsString(m_pTestOverlayInUse->hImageTemplate) + "Setting yuv attributes.");
      // set the yuv attributes
      eMTK_YUV_IMAGE_FORMAT eFormat = eMTK_YUV_IMAGE_FORMAT_420;
      if (mtk_SetYUVImageFormat (m_pTestOverlayInUse->hImageTemplate, eFormat) == MTK_ERROR)
      {
         LogMsg(Name(), "ERROR: setting YUV Image format");
         outputLastMTKError ();
         return -1;
      }

      m_bitMapLength = 0;
      if (mtk_SetYUVImageSize (m_pTestOverlayInUse->hImageTemplate, m_pTestOverlayInUse->imgWidth, m_pTestOverlayInUse->imgHeight) == MTK_ERROR)
      {
         LogMsg(Name(), "ERROR: setting YUV Image size");
         outputLastMTKError ();
         return -1;
      }

      m_bitMapLength = m_pTestOverlayInUse->imgWidth*m_pTestOverlayInUse->imgHeight;
#if 1
      unsigned int h = 0;
      unsigned int w = 0;
      LogMsg(Name(), "Getting yuv height and width and comparing");

      if (mtk_GetYUVImageSize (m_pTestOverlayInUse->hImageTemplate, &w, &h) == MTK_ERROR)
      {
         LogMsg(Name(), "ERROR: getting YUV height and width");
         outputLastMTKError ();
         return -1;
      }
      if ((h != m_pTestOverlayInUse->imgHeight) || (w != m_pTestOverlayInUse->imgWidth))
      {
         LogMsg(Name(), "ERROR: YUV height or width values did not match; expected: w(120), h(176) received: w("
			 + AsString(w) + "), h(" + AsString(h) + ")");
         return -1;
      }
#endif
   }
   return 0;
}

//*****************************************************************************
// Function: int COverlay::ApplyMediaToTemplate()
// Description: Apply a media file to an image template
// Return: int 
// Parameters: none 
//*****************************************************************************
int COverlay::ApplyMediaToTemplate ()
{
   if ((m_pTestOverlayInUse->hMedia = mtk_CreateMediaFileTemplate(m_pTestOverlayInUse->hImageTemplate, m_imgFileSrc.c_str()))== MTK_ERROR)
   {
      LogMsg(Name(), "ERROR: returned from createMediaFile");
      outputLastMTKError();
      return -1;
   }
   else
   {
      LogMsg(Name(), "received media file handle: " + AsString(m_pTestOverlayInUse->hMedia));
   }

   return 0;
}

//*****************************************************************************
// Function: int COverlay::CreateOverlayTemplates(TestOverlay *overlays, unsigned int len)
// Description: Create an image overlay template
// Return: int 
// Parameters: TestOverlay *overlays 
//             unsigned int len 
//*****************************************************************************
int COverlay::CreateOverlayTemplates(TestOverlay * overlays, unsigned int len)
{
   // create the overlay templates
   for (unsigned int ii = 0; ii < len; ii++)
   {
      // create a frame for each overlay
      LogMsg(Name(), "creating frame template");
      if ((overlays[ii].hBoundingFrameTemplate = mtk_CreateFrameTemplate())
          == MTK_ERROR)
      {
		  LogMsg(Name(), "ERROR: creating frame template");
         outputLastMTKError();
         return -1;
      }

	  LogMsg(Name(), "setting frame[" + AsString(overlays[ii].hBoundingFrameTemplate) + "] position:");
	  LogMsg(Name(), "\tx: [" + AsString(overlays[ii].framePosition_x) + "]; y: [" 
		                      + AsString(overlays[ii].framePosition_y) + "]; pos enum[" 
							  + AsString(overlays[ii].ePositionType) +"]");
      // set the frame position
      if (mtk_SetFramePosition(overlays[ii].hBoundingFrameTemplate,
                               overlays[ii].framePosition_x,
                               overlays[ii].framePosition_y,
                               overlays[ii].ePositionType) == MTK_ERROR)
      {
		 LogMsg(Name(), "ERROR: setting frame template position values");
         outputLastMTKError();
         return -1;
      }

      
	  LogMsg(Name(), "setting frame[" + AsString(overlays[ii].hBoundingFrameTemplate) + "] size:");
	  LogMsg(Name(), "\tw: [" + AsString(overlays[ii].frameWidth) + "]; h: [" 
		                      + AsString(overlays[ii].frameHeight) + "]; size enum[" 
							  + AsString(overlays[ii].eSizeType) +"]");
      // set the frame size
      if (mtk_SetFrameSize(overlays[ii].hBoundingFrameTemplate,
                           overlays[ii].frameWidth,
                           overlays[ii].frameHeight,
                           overlays[ii].eSizeType) == MTK_ERROR)
      {
		 LogMsg(Name(), "ERROR: setting frame template size values");
         outputLastMTKError();
         return -1;
      }

      // create the image overlay template
      if ((overlays[ii].hOvlTemplate =
           ob_CreateImageOverlayTemplate(m_pTestOverlayInUse->hMedia))
          == MTK_ERROR)
      {
		 LogMsg(Name(), "ERROR: returned from createImageOverlay for " + AsString(ii));
         outputLastMTKError();
         return -1;
      }
      else
      {
         LogMsg(Name(), "received overlay handle [ " + AsString(overlays[ii].hOvlTemplate)
			             + "] for overlay index [" + AsString(ii));
         // set all the attributes on the template
         if (setOverlayAttrs(&overlays[ii]) == -1)
         {
            return -1;
         }
      }
   }
   return 0;
}

//*****************************************************************************
// Function: int COverlay::setOverlayAttrs(TestOverlay *pTestOverlay)
// Description: Set the image overlay attributes
// Return: int 
// Parameters: TestOverlay *pTestOverlay 
//*****************************************************************************
int COverlay::setOverlayAttrs(TestOverlay* pTestOverlay)
{
   LogMsg(Name(), "Setting overlay duration");
   if (ob_SetOverlayDuration(pTestOverlay->hOvlTemplate, pTestOverlay->duration_ms) == MTK_ERROR)
   {
      LogMsg(Name(), "ERROR: setting overlay duration to " + AsString(pTestOverlay->duration_ms));
	  outputLastMTKError();
      return -1;
   }

   LogMsg(Name(), "Setting bounding frame for the overlay template");
   // set the template frame getting the handle of the snapshot frame
   if ((pTestOverlay->hBoundingSnapshot =
        ob_SetOverlayBoundingFrame(pTestOverlay->hOvlTemplate,
                                   pTestOverlay->hBoundingFrameTemplate))
       == MTK_ERROR)
   {
      LogMsg(Name(), "Setting overlay template frame");
      outputLastMTKError();
      return -1;
   }

   LogMsg(Name(), "Setting overlay fill style");
   // set the overlay fill style
   if (ob_SetOverlayFillStyle(pTestOverlay->hOvlTemplate,
                              eOB_FILL_STYLE_RESIZE_TO_FIT) == MTK_ERROR)
   {
	  LogMsg(Name(), "ERROR: Setting overlay fill style");
      outputLastMTKError();
      return -1;
   }

   return 0;
}

//*****************************************************************************
// Function: int COverlay::removeOverlay(int nDeviceHandle, TestOverlay *pOverlay)
// Description: Remove an overlay from a device stream
// Return: int 
// Parameters: int nDeviceHandle 
//             TestOverlay *pOverlay 
//*****************************************************************************
int COverlay::removeOverlay(int nDeviceHandle, TestOverlay* pOverlay)
{
   SM_REMOVE_OVERLAY_LIST overlayList;
   INIT_SM_REMOVE_OVERLAY_LIST(&overlayList);
   overlayList.unCount = 1;
   overlayList.overlayHandles[0] = pOverlay->hDeviceOvlSnapshot;

   LogMsg(Name(), "calling sm_RemoveOverlay for overlay handle[" + AsString(pOverlay->hDeviceOvlSnapshot) + "]");

   if (sm_RemoveOverlays(nDeviceHandle, &overlayList, pOverlay) == MTK_ERROR)
   {
	  LogMsg(Name(), "ERROR: returned from sm_RemoveOverlays()");
      outputLastMTKError();
      return -1;
   }
   return 0;
}

//*****************************************************************************
// Function: int COverlay::removeOverlay(int nDeviceHandle)
// Description: Remove an overlay from a device stream
// Return: int 
// Parameters: int nDeviceHandle 
//*****************************************************************************
int COverlay::removeOverlay(int nDeviceHandle)
{
   if (m_pTestOverlayInUse)
   {
      return removeOverlay(nDeviceHandle, m_pTestOverlayInUse);
   }
   else
   {
      return -1;
   }
}

//*****************************************************************************
// Function: int COverlay::addOverlay(int nDeviceHandle, TestOverlay *pOverlay, eSM_OVERLAY_DIRECTION direction)
// Description: Add an overlay to a device stream
// Return: int 
// Parameters: int nDeviceHandle 
//             TestOverlay *pOverlay 
//             eSM_OVERLAY_DIRECTION direction 
//*****************************************************************************
int COverlay::addOverlay(int nDeviceHandle, TestOverlay* pOverlay, eSM_OVERLAY_DIRECTION direction)
{
   SM_ADD_OVERLAY_LIST overlayList;
   INIT_SM_ADD_OVERLAY_LIST(&overlayList);
   overlayList.unCount = 1;

   pOverlay->hDeviceOvlSnapshot = MTK_ERROR;
   overlayList.addOverlays[0].eDirection = direction;
   overlayList.addOverlays[0].hOverlay = pOverlay->hOvlTemplate;

   LogMsg(Name(), "calling sm_AddOverlay for overlay handle[" + AsString(pOverlay->hOvlTemplate) + "]");

   if (sm_AddOverlays(nDeviceHandle, &overlayList, pOverlay) == MTK_ERROR)
   {
	  LogMsg(Name(), "ERROR: returned from sm_AddOverlays()");
      outputLastMTKError();
      return -1;
   }
   return 0;
}

//*****************************************************************************
// Function: int COverlay::addOverlay(int nDeviceHandle, eSM_OVERLAY_DIRECTION direction)
// Description: Add an overlay to a device stream
// Return: int 
// Parameters: int nDeviceHandle 
//             eSM_OVERLAY_DIRECTION direction 
//*****************************************************************************
int COverlay::addOverlay(int nDeviceHandle, eSM_OVERLAY_DIRECTION direction)
{
   if (m_pTestOverlayInUse)
   {
      return addOverlay(nDeviceHandle, m_pTestOverlayInUse, direction);
   }
   else
   {
      return -1;
   }
}

//*****************************************************************************
// Function: int COverlay::OnOverlayAdd(long evtlen, void *pEvtData, void *evtContext)
// Description: Handle an add overlay completion event
// Return: int 
// Parameters: long evtlen 
//             void *pEvtData 
//             void *evtContext 
//*****************************************************************************
int COverlay::OnOverlayAdd(long evtlen, void* pEvtData, void* evtContext)
{
   PSM_ADD_OVERLAY_RESULT_LIST paddOverlay = (PSM_ADD_OVERLAY_RESULT_LIST)pEvtData;
   TestOverlay* pExpectedOverlayAdd = (TestOverlay*)evtContext;
   if (pExpectedOverlayAdd->hOvlTemplate == paddOverlay->overlayResults[0].hOverlayTemplate)
   {
      pExpectedOverlayAdd->hDeviceOvlSnapshot = paddOverlay->overlayResults[0].hOverlaySnapshot;
      LogMsg(Name(), "received overlay: template handle [" + AsString(paddOverlay->overlayResults[0].hOverlayTemplate) + "];");
	  LogMsg(Name(), "                  snapshot handle [" + AsString(paddOverlay->overlayResults[0].hOverlaySnapshot) + "];");
	  LogMsg(Name(), "                  result[" + AsString(translateOverlayResult(paddOverlay->overlayResults[0].result)) + "]");
   }
   else
   {
      LogMsg(Name(), "ERROR: add overlay event is good but wrong event data");
   }

#if 0
   LogMsg(Name(), "--- List count: " + AsString(paddOverlay->unCount));
   for (int i = 0; i < paddOverlay->unCount; i++)
   {
      LogMsg(Name(), "--- list[0].overlayResults[" + AsString(i) + "].hOverlayTemplate = " + AsString(paddOverlay->overlayResults[i].hOverlayTemplate));
	  LogMsg(Name(), "--- list[0].overlayResults[" + AsString(i) + "].hOverlaySnapshot = " + AsString(paddOverlay->overlayResults[i].hOverlaySnapshot));
	  LogMsg(Name(), "--- list[0].overlayResults[" + AsString(i) + "].result = " + AsString(paddOverlay->overlayResults[i].result));
   }
#endif

   return 0;
}

//*****************************************************************************
// Function: int COverlay::OnOverlayRemove(long evtlen, void *pEvtData, void *evtContext)
// Description: Handle a remove overlay completion event
// Return: int 
// Parameters: long evtlen 
//             void *pEvtData 
//             void *evtContext 
//*****************************************************************************
int COverlay::OnOverlayRemove(long evtlen, void* pEvtData, void* evtContext)
{
   PSM_REMOVE_OVERLAY_LIST premoveOverlay = (PSM_REMOVE_OVERLAY_LIST)pEvtData;
   TestOverlay* pExpectedOverlayRemove = (TestOverlay*)evtContext;

   if (premoveOverlay->unCount != 1)
   {
	  LogMsg(Name(), "remove overlay event is good but count is not 1");

      return -1;
   }

   if (pExpectedOverlayRemove->hDeviceOvlSnapshot == premoveOverlay->overlayHandles[0])
   {
	  LogMsg(Name(), "received overlay remove event");
   }
   else
   {
      LogMsg(Name(), "remove overlay event is good but wrong event data.");
	  LogMsg(Name(), "       Expected hDeviceOvlSnapshot = " + AsString(pExpectedOverlayRemove->hDeviceOvlSnapshot));
	  LogMsg(Name(), "       Received  = " + AsString(premoveOverlay->overlayHandles[0]));
   }

#if 0
   LogMsg(Name(), "--- List count: " + AsString(premoveOverlay->unCount));
   for (int i = 0; i < premoveOverlay->unCount; i++)
   {
	  LogMsg(Name(), "--- list[0].overlayHandles[" + AsString(i) + "] = " + AsString(premoveOverlay->overlayHandles[i]));
	  
   }
#endif
   return 0;
}

//*****************************************************************************
// Function: int COverlay::outputLastMTKError()
// Description: Output the last media toolkit error
// Return: int 
// Parameters: none 
//*****************************************************************************
int COverlay::outputLastMTKError ()
{
   MTK_ERROR_INFO errorInfo;
   INIT_MTK_ERROR_INFO (&errorInfo);

   if (mtk_GetErrorInfo (&errorInfo) == MTK_ERROR)
   {
	  LogMsg(Name(), "ERROR: getting last mtk error");
   }
   else
   {
      LogMsg(Name(), "MTK Last Error:");
	  LogMsg(Name(), "    unErrorCode: " + AsString(errorInfo.unErrorCode));
	  LogMsg(Name(), "    szErrorString: " + AsString(errorInfo.szErrorString));
	  LogMsg(Name(), "    szAdditionalInfo: " + AsString(errorInfo.szAdditionalInfo));
   }
   return 0;
}

//*****************************************************************************
// Function: const char* COverlay::translateOverlayResult(eSM_ADD_OVERLAY_RESULT result)
// Description: Translate an overlay result into a string
// Return: const char* 
// Parameters: eSM_ADD_OVERLAY_RESULT result 
//*****************************************************************************
const char* COverlay::translateOverlayResult(eSM_ADD_OVERLAY_RESULT result)
{
   static const char * cpadd       = "eSM_ADD_OVERLAY_RESULT_ADD";
   static const char * cpmodify    = "eSM_ADD_OVERLAY_RESULT_MODIFY";
   static const char * cpfail      = "eSM_ADD_OVERLAY_RESULT_FAIL";
   static const char * cperror     = "ERROR - invalid add overlay result";

   switch (result)
   {
   case eSM_ADD_OVERLAY_RESULT_ADD:
      return cpadd;

   case eSM_ADD_OVERLAY_RESULT_MODIFY:
      return cpmodify;

   case eSM_ADD_OVERLAY_RESULT_FAIL:
      return cpfail;

   default:
      return cperror;
   };
}

