/**
* @file COverlay.h
* @brief COverlay class declaration
* @date Mar. 27, 2008
*
* DIALOGIC CONFIDENTIAL
* Copyright  2008 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/
#if !defined (_IMGOVL_H_)
   #define _IMGOVL_H_

   #include "LogUtil.h"
   #include "mtklib.h"
   #include "sm_mtklib.h"
#ifdef WIN32
	#include <string>
#endif

typedef struct tagTestOverlay
{
   eMTK_IMAGE_FORMAT imageFormat;
   MTK_HANDLE hImageTemplate; //mtk_CreateImageTemplate()
   MTK_HANDLE hMedia;         //mtk_CreateMediaFileTemplate()
   OB_OVERLAY_HANDLE hOvlTemplate; //ob_CreateImageOverlayTemplate()
   OB_OVERLAY_HANDLE hDeviceOvlSnapshot; //handle provided by SMEV_ADD_OVERLAY event as a result of calling sm_AddOverlays()
// mtk_SetFramePosition(hBoundingFrameTemplate, x, y, ePositionType)
   MTK_FRAME_HANDLE  hBoundingFrameTemplate;
   float framePosition_x;
   float framePosition_y;
   eMTK_POSITION_TYPE ePositionType;

// mtk_SetFrameSize(hBoundingFrameTemplate, frameWidth, frameHeight, eSizeType)
   float frameWidth;
   float frameHeight;
   eMTK_SIZE_TYPE eSizeType;

// hBoundingSnapshot = ob_SetOverlayBoundingFrame(hOvlTemplate, hBoundingFrameTemplate)
   MTK_FRAME_HANDLE  hBoundingSnapshot;

// set overlay duration
// ob_SetOverlayDuration(pTestOverlay->hOvlTemplate, unDuration)
   unsigned int duration_ms;

   unsigned int imgHeight;
   unsigned int imgWidth;
   char imgName[255];
}TestOverlay, *PTestOverlay;

//*****************************************************************************
// Class: ::COverlay
// Description: Concrete class used to manage an overlay
//*****************************************************************************
class COverlay
{
public:
   COverlay(eMTK_IMAGE_FORMAT imgFormat, eMTK_SIZE_TYPE sizeType, unsigned int width, unsigned int height, const char* szImgFileSrc );
   COverlay(eMTK_IMAGE_FORMAT imgFormat,  const char* szImgFile);
   COverlay (const TestOverlay *pOverlay);
   ~COverlay();
   int CreateYUVImageTemplate ();
   int CreateJPEGImageTemplate ();
   int ApplyMediaToTemplate();
   int CreateOverlayTemplates(TestOverlay * overlays, unsigned int len);
   int setOverlayAttrs(TestOverlay* pTestOverlay);
   int addOverlay(int nDeviceHandle, TestOverlay* pOverlay, eSM_OVERLAY_DIRECTION direction);
   int addOverlay(int nDeviceHandle, eSM_OVERLAY_DIRECTION direction);
   int removeOverlay(int nDeviceHandle);
   int removeOverlay(int nDeviceHandle, TestOverlay* pOverlay);
   int OnOverlayAdd(long evtlen, void* pEvtData, void* evtContext);
   int OnOverlayRemove(long evtlen, void* pEvtData, void* evtContext);
   int outputLastMTKError();
   int checkSnapshotOverlayAttrs();
   static const char* translateOverlayResult(eSM_ADD_OVERLAY_RESULT result);

   const std::string Name() {return "Ovrlay";}

   std::string m_imgFileSrc;
   unsigned int m_bitMapLength;
   TestOverlay* m_pTestOverlayInUse;
};

#endif

