/**
* @file CRecorder.h
* @brief Recorder class declaration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef CRecorder_h
#define CRecorder_h

#include <string>

#include "port_connect.h"

#include "config.h"

#include "AEventHandler.h"
#include "CSystemConfiguration.h"

class CMediaDevice;
class CConferencePartyDevice;
class ARecorderState;

//*****************************************************************************
// Class: ::CRecorder
// Description: Class to provide a virtual caller/conference party.  The caller
//    is simulated by recording a multimedia clip to a conference party 
//    connection
//*****************************************************************************
class CRecorder : public AEventHandler
{
   friend class ARecorderState;
   friend class CRecorderIdleState;
   friend class CRecorderSetupState;
   friend class CRecorderAvailableState;
   friend class CRecorderRecordingState;
   friend class CRecorderTeardownState;

public:
   CRecorder(std::string confID, bool bRecordConf);
   virtual ~CRecorder() {}

   void ProcessEvent(METAEVENT& a_metaEvent);

   ARecorderState& State() {return *m_state;}
   void SetStateTo(ARecorderState* state);

   const std::string& Name() {return m_name;}

	void OpenMediaDevice();
   int StartRecorder();
   int OpenConferenceParty();
   int OnOpenConferencePartyComplete();
   int ConnectToConferenceParty();
   int OnConnectToConferencePartyComplete();
   int JoinConference();
   int OnJoinConferenceComplete();
   int LeaveConference();
   int OnLeaveConferenceComplete();
   int OnLeaveConferenceFailed();
   int DisconnectFromConferenceParty();
   int OnDisconnectFromConferencePartyComplete();
   int CloseConferenceParty();
   int OnCloseConferencePartyComplete();
	void SystemConfiguration(CSystemConfiguration* a_systemConfiguration);

   int Play();
   int OnPlayComplete();

   int Record();
   int OnRecordComplete();
   int Stop();

   int OnGetMmRxPortInfoComplete();
   int OnGetMmTxPortInfoComplete();
   int OnGetCnfRxPortInfoComplete();
   int OnGetCnfTxPortInfoComplete();

   int PortInfoSubState() {return m_portInfoSubState;}

   int OnCnfPortConnected();
   int OnMmPortConnected();

   int PortConnectSubState() {return m_portConnectSubState;}

   int OnCnfPortDisconnected();
   int OnMmPortDisconnected();

   int PortDisconnectSubState() {return m_portDisconnectSubState;}

   static int MM_RX_PORT_INFO;
   static int MM_TX_PORT_INFO;
   static int CNF_RX_PORT_INFO;
   static int CNF_TX_PORT_INFO;
   static int PORT_INFO_COMPLETE;

   static int MM_TX_CNF_RX_CONNECT;
   static int CNF_TX_MM_RX_CONNECT;
   static int PORT_CONNECT_COMPLETE;

   static int MM_TX_CNF_RX_DISCONNECT;
   static int CNF_TX_MM_RX_DISCONNECT;
   static int PORT_DISCONNECT_COMPLETE;

private:

   // force initialization
   CRecorder();

   // make non-copyable
   CRecorder(const CRecorder&);
   const CRecorder& operator=(const CRecorder&);

   ARecorderState* m_state;
   std::string m_name;

   CMediaDevice* m_mmDevice;
   AEventHandler* m_observer;

   CConferencePartyDevice* m_conferenceParty;
   int m_portInfoSubState;
   int m_portConnectSubState;
   int m_portDisconnectSubState;
   bool m_stopRecord;
   bool m_stopPlay;
	bool m_bRecordConf;

	std::string m_confID;
	std::string m_DeviceName;
	int m_DeviceH;

   DM_PORT_CONNECT_INFO_LIST m_portConnectInfoListMmTxCnfRx;
   DM_PORT_CONNECT_INFO_LIST m_portConnectInfoListCnfTxMmRx;

   int PrimStartRecorder();
   int PrimOpenConferenceParty();
   int PrimOnOpenConferencePartyComplete();
   int PrimConnectToConferenceParty();
   int PrimOnConnectToConferencePartyComplete();
   int PrimJoinConference();
   int PrimOnJoinConferenceComplete();
   int PrimLeaveConference();
   int PrimOnLeaveConferenceComplete();
   int PrimDisconnectFromConferenceParty();
   int PrimOnDisconnectFromConferencePartyComplete();
   int PrimCloseConferenceParty();
   int PrimOnCloseConferencePartyComplete();
   int PrimOnCallReleased();

   int PrimOnGetMmRxPortInfoComplete();
   int PrimOnGetMmTxPortInfoComplete();
   int PrimOnGetCnfRxPortInfoComplete();
   int PrimOnGetCnfTxPortInfoComplete();
   int PrimPortConnect();

   int PrimOnCnfPortConnected();
   int PrimOnMmPortConnected();

   int PrimOnCnfPortDisconnected();
   int PrimOnMmPortDisconnected();


   int CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                   const DM_PORT_INFO_LIST& a_portInfoList1,
                                   const DM_PORT_INFO_LIST& a_portInfoList2);

   int PrimPlay();
   int PrimOnPlayComplete();
   int PrimRecord();
   int PrimOnRecordComplete();
   int PrimStop();

};

#endif

