/**
* @file CScheduler.h
* @brief Telmpate declarations for a task scheduler
* @date No.v 29, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/
#ifndef CScheduler_h
#define CScheduler_h

#include <sys/timeb.h>
#include <list>

//*****************************************************************************
// Class: ::CScheduledTask
// Description: Template class for a scheduled task
//*****************************************************************************
template< class Class>
class CScheduledTask
{
public:

   typedef int ( Class::*Method)();

   CScheduledTask( Class* object, Method method, long secsFromNow) : m_object( object), m_method( method) 
   {
      struct timeb tstruct;
      ftime( &tstruct );
      m_schedTime = tstruct.time + secsFromNow;
   }

   int operator()()
   {
      return( m_object->*m_method)();
   };

   int operator<( CScheduledTask<Class>& task)
   {
      return( this->ScheduledTime() < task.ScheduledTime());
   };

   long ScheduledTime()
   {
      return m_schedTime;
   }

private:
   long m_schedTime;
   Class* m_object;
   Method m_method;
};

//*****************************************************************************
// Class: ::CScheduler
// Description: Template class for a task scheduler
//*****************************************************************************
template< class Class>
class CScheduler
{
   typedef CScheduledTask<Class> ScheduledTask;
   typedef int ( Class::*Method)();

public:

   static CScheduler<Class>* Instance()
   {
      if (!m_instance)
      {
         m_instance =    new CScheduler();
      }
      return m_instance;
   }

   void Run()
   {
      struct timeb timeInfo;
      ftime( &timeInfo );
      if (!m_taskList.empty())
      {
         ScheduledTask& task = m_taskList.front();
         if (task.ScheduledTime() <= timeInfo.time)
         {
            task();
            m_taskList.pop_front();
         }
      }
   }

   void ScheduleTask( Class* object, Method method, long secsFromNow)
   {
      m_taskList.push_back( ScheduledTask( object, method, secsFromNow));
      m_taskList.sort();
   }

private:
   CScheduler()
   {
      ;
   }
   static CScheduler<Class>* m_instance;
   std::list< ScheduledTask > m_taskList;
};

template< class Class>
CScheduler<Class>* CScheduler<Class>::m_instance = 0;

#endif


