/**
* @file CSipSession.cpp
* @brief SIP Session class implementation
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifdef WIN32
   #pragma warning(disable:4786) 
#include <time.h>
#endif

#include <vector>

#include "config.h"

#include "devmgmt.h"

#include "LogUtil.h"
#include "CSipSession.h"
#include "CSipSessionState.h"
#include "CGcSipDevice.h"
#include "CIpmDevice.h"
#include "CMediaDevice.h"
#include "CConference.h"
#include "CConferencePartyDevice.h"
#include "CGcManager.h"

#include "CConnection.h"

char* g_callerIdFiles[] = {
   "caller1.jpg",
   "caller2.jpg",
   "caller3.jpg",
   "caller4.jpg",
   "caller5.jpg",
   "caller6.jpg",
   "caller7.jpg",
   "caller8.jpg",
   "caller9.jpg"
};

//typedef struct ipm_audio_coder_info_tag
//{
//	   eIPM_CODER_TYPE		eCoderType;				// The coder Type
//	   eIPM_CODER_FRAMESIZE	eFrameSize;				// Frame size supported
//	   unsigned int			unFramesPerPkt;		// No. of Frames per packet
//	   eIPM_CODER_VAD			eVadEnable;				// Flag indicating if VAD is enabled/disabled
//	   unsigned int			unCoderPayloadType;	// Type of coder payload supported
//	   unsigned int			unRedPayloadType;		// Type of Redundancy Payload supported
//}IPM_AUDIO_CODER_INFO, *PIPM_AUDIO_CODER_INFO;

IPM_AUDIO_CODER_INFO g_systemAudioCoderCapabilities_HMP[] = { 
		{CODER_TYPE_G711ULAW64K,(eIPM_CODER_FRAMESIZE)10,1,(eIPM_CODER_VAD)0,0,0}, 
		{CODER_TYPE_G711ULAW64K,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)0,0,0},
		{CODER_TYPE_G711ULAW64K,(eIPM_CODER_FRAMESIZE)30,1,(eIPM_CODER_VAD)0,0,0},

		{CODER_TYPE_G711ALAW64K,(eIPM_CODER_FRAMESIZE)10,1,(eIPM_CODER_VAD)0,8,0}, 
		{CODER_TYPE_G711ALAW64K,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)0,8,0}, 
		{CODER_TYPE_G711ALAW64K,(eIPM_CODER_FRAMESIZE)30,1,(eIPM_CODER_VAD)0,8,0},

		{CODER_TYPE_G7231_5_3K,(eIPM_CODER_FRAMESIZE)30,1,(eIPM_CODER_VAD)0,4,0},
		{CODER_TYPE_G7231_5_3K,(eIPM_CODER_FRAMESIZE)30,2,(eIPM_CODER_VAD)0,4,0},
		{CODER_TYPE_G7231_5_3K,(eIPM_CODER_FRAMESIZE)30,3,(eIPM_CODER_VAD)0,4,0},
		{CODER_TYPE_G7231_5_3K,(eIPM_CODER_FRAMESIZE)30,1,(eIPM_CODER_VAD)1,4,0},
		{CODER_TYPE_G7231_5_3K,(eIPM_CODER_FRAMESIZE)30,2,(eIPM_CODER_VAD)1,4,0},
		{CODER_TYPE_G7231_5_3K,(eIPM_CODER_FRAMESIZE)30,3,(eIPM_CODER_VAD)1,4,0},

		{CODER_TYPE_G7231_6_3K,(eIPM_CODER_FRAMESIZE)30,1,(eIPM_CODER_VAD)0,4,0},
		{CODER_TYPE_G7231_6_3K,(eIPM_CODER_FRAMESIZE)30,2,(eIPM_CODER_VAD)0,4,0},
		{CODER_TYPE_G7231_6_3K,(eIPM_CODER_FRAMESIZE)30,3,(eIPM_CODER_VAD)0,4,0},
		{CODER_TYPE_G7231_6_3K,(eIPM_CODER_FRAMESIZE)30,1,(eIPM_CODER_VAD)1,4,0},
		{CODER_TYPE_G7231_6_3K,(eIPM_CODER_FRAMESIZE)30,2,(eIPM_CODER_VAD)1,4,0},
		{CODER_TYPE_G7231_6_3K,(eIPM_CODER_FRAMESIZE)30,3,(eIPM_CODER_VAD)1,4,0},

      {CODER_TYPE_G729,(eIPM_CODER_FRAMESIZE)10,1,(eIPM_CODER_VAD)0,18,0},
      {CODER_TYPE_G729,(eIPM_CODER_FRAMESIZE)10,2,(eIPM_CODER_VAD)0,18,0},
      {CODER_TYPE_G729,(eIPM_CODER_FRAMESIZE)10,4,(eIPM_CODER_VAD)0,18,0},
      {CODER_TYPE_G729,(eIPM_CODER_FRAMESIZE)10,3,(eIPM_CODER_VAD)0,18,0}, 

		{CODER_TYPE_G729ANNEXA,(eIPM_CODER_FRAMESIZE)10,1,(eIPM_CODER_VAD)0,18,0},
		{CODER_TYPE_G729ANNEXA,(eIPM_CODER_FRAMESIZE)10,2,(eIPM_CODER_VAD)0,18,0},
		{CODER_TYPE_G729ANNEXA,(eIPM_CODER_FRAMESIZE)10,4,(eIPM_CODER_VAD)0,18,0},
		{CODER_TYPE_G729ANNEXA,(eIPM_CODER_FRAMESIZE)10,3,(eIPM_CODER_VAD)0,18,0}, 

		{CODER_TYPE_G729ANNEXAWANNEXB,(eIPM_CODER_FRAMESIZE)10,1,(eIPM_CODER_VAD)1,18,0},
		{CODER_TYPE_G729ANNEXAWANNEXB,(eIPM_CODER_FRAMESIZE)10,2,(eIPM_CODER_VAD)1,18,0},
		{CODER_TYPE_G729ANNEXAWANNEXB,(eIPM_CODER_FRAMESIZE)10,4,(eIPM_CODER_VAD)1,18,0},
		{CODER_TYPE_G729ANNEXAWANNEXB,(eIPM_CODER_FRAMESIZE)10,3,(eIPM_CODER_VAD)1,18,0}, 

      {CODER_TYPE_AMRNB_12_2k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_10_2k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_7_95k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_7_4k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_6_7k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_5_9k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_5_15k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_4_75k,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0},
      {CODER_TYPE_AMRNB_NONE,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,96,0} ,

      {CODER_TYPE_AMRWB_23_85K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_23_05K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_19_85K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_18_25K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_15_85K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_14_25K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_12_65K, (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_8_85K,  (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_6_6K,   (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},
      {CODER_TYPE_AMRWB_NONE,   (eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)1,111,0},

		{CODER_TYPE_G722_64K,(eIPM_CODER_FRAMESIZE)20,1,(eIPM_CODER_VAD)0,9,0}, 
};

//typedef struct ipm_video_coder_info_tag
//{
//	unsigned int			unVersion;		      // structure version
//	eIPM_CODER_TYPE		eCoderType;		      // The coder Type - e.g. CODER_TYPE_H263
//	unsigned int			unCoderPayloadType;	// Static or dynamic range of  96-127
//} IPM_VIDEO_CODER_INFO, *PIPM_VIDEO_CODER_INFO;

IPM_VIDEO_CODER_INFO g_systemVideoCoderCapabilities_HMP[] = {
		{0, CODER_TYPE_H263,34},
      {0, CODER_TYPE_MP4V_ES, 96},
      {0, CODER_TYPE_H264, 106}
};

//*****************************************************************************
// Function: CSipSession::CSipSession(const std::string &a_sessionName, CGcSipDevice *a_sipDevice, CIpmDevice *a_ipmDevice, CMediaDevice *a_mmDevice)
// Description: Initializing constructor
// Return:  
// Parameters: const std::string &a_sessionName 
//             CGcSipDevice *a_sipDevice 
//             CIpmDevice *a_ipmDevice 
//             CMediaDevice *a_mmDevice 
//*****************************************************************************
CSipSession::CSipSession(const std::string& a_sessionName,CGcSipDevice* a_sipDevice, CIpmDevice* a_ipmDevice, CMediaDevice* a_mmDevice, CSystemConfiguration& a_systemConfiguration)
   : m_sessionState(ASipSessionState::SessionState(SipSessionIdleState))
   , m_sessionName(a_sessionName)
   , m_sipDevice(a_sipDevice)
   , m_ipmDevice(a_ipmDevice)
   , m_mmDevice(a_mmDevice)
   , m_systemConfiguration(a_systemConfiguration)
   , m_conferenceParty(0)
   , m_sessionBandwidth(0)
   , m_audioBandwidth(0)
   , m_videoBandwidth(0)
   , m_matchedAudioCoder(0)
   , m_matchedRfc2833Coder(0)
   , m_matchedVideoCoder(0)
   , m_callerIdOverlay(0)
   , m_layoutDescriptionOverlay(0)
   , m_cnfID("2345001")
   , m_callerId(0)
   , m_bPinAccepted(false)
   , m_bMediaStarted(false)
   , m_bMediaPlayerConnected(false)
   , m_bConfPartyConnected(false)
   , m_digitBuf("")
   , m_nDigitCnt(0)
{
   m_sipDevice->SipSession(this);
   m_ipmDevice->Observer(this);
   m_mmDevice->Observer(this);

   m_callerIdOverlayInfo.imageFormat = eMTK_IMAGE_FORMAT_JPEG;

   m_callerIdOverlayInfo.hOvlTemplate = MTK_ERROR;
   m_callerIdOverlayInfo.hDeviceOvlSnapshot = MTK_ERROR; 
   m_callerIdOverlayInfo.hBoundingFrameTemplate = MTK_ERROR;
//   m_callerIdOverlayInfo.framePosition_x = 0;
//   m_callerIdOverlayInfo.framePosition_y = 0;
//   m_callerIdOverlayInfo.ePositionType = eMTK_POSITION_TYPE_PIXEL;

   m_callerIdOverlayInfo.framePosition_x = 35;
   m_callerIdOverlayInfo.framePosition_y = 80;
   m_callerIdOverlayInfo.ePositionType = eMTK_POSITION_TYPE_PERCENT;

//   m_callerIdOverlayInfo.framePosition_x = MTK_JUSTIFY_LEFT;
//   m_callerIdOverlayInfo.framePosition_y = MTK_JUSTIFY_BOTTOM;
//   m_callerIdOverlayInfo.ePositionType = eMTK_POSITION_TYPE_JUSTIFICATION;

//   m_callerIdOverlayInfo.frameWidth = 110.0;
//   m_callerIdOverlayInfo.frameHeight = 32.0;
//   m_callerIdOverlayInfo.eSizeType = eMTK_SIZE_TYPE_PIXEL;

   m_callerIdOverlayInfo.frameWidth = 31;
   m_callerIdOverlayInfo.frameHeight = 11;
   m_callerIdOverlayInfo.eSizeType = eMTK_SIZE_TYPE_PERCENT;

   m_callerIdOverlayInfo.hBoundingSnapshot = MTK_ERROR;

   m_callerIdOverlayInfo.duration_ms = 5000;

   m_callerIdOverlayInfo.imgHeight = 0;
   m_callerIdOverlayInfo.imgWidth = 0;

   // image file
//   std::string l_buffer(m_sessionName);
//   l_buffer.append(".jpg");
//   strcpy(m_callerIdOverlayInfo.imgName, l_buffer.c_str());

//   printf("image overlay file:%s\n", l_buffer.c_str());

//   m_callerIdOverlay = new COverlay(&m_callerIdOverlayInfo);


   m_layoutDescriptionOverlayInfo.imageFormat = eMTK_IMAGE_FORMAT_JPEG;

   m_layoutDescriptionOverlayInfo.hOvlTemplate = MTK_ERROR;
   m_layoutDescriptionOverlayInfo.hDeviceOvlSnapshot = MTK_ERROR; 
   m_layoutDescriptionOverlayInfo.hBoundingFrameTemplate = MTK_ERROR;
//   m_callerIdOverlayInfo.framePosition_x = 0;
//   m_callerIdOverlayInfo.framePosition_y = 0;
//   m_callerIdOverlayInfo.ePositionType = eMTK_POSITION_TYPE_PIXEL;

   m_layoutDescriptionOverlayInfo.framePosition_x = 25;
   m_layoutDescriptionOverlayInfo.framePosition_y = 75;
   m_layoutDescriptionOverlayInfo.ePositionType = eMTK_POSITION_TYPE_PERCENT;

//   m_callerIdOverlayInfo.framePosition_x = MTK_JUSTIFY_LEFT;
//   m_callerIdOverlayInfo.framePosition_y = MTK_JUSTIFY_BOTTOM;
//   m_callerIdOverlayInfo.ePositionType = eMTK_POSITION_TYPE_JUSTIFICATION;

   m_layoutDescriptionOverlayInfo.frameWidth = 20.0;
   m_layoutDescriptionOverlayInfo.frameHeight = 10.0;
   m_layoutDescriptionOverlayInfo.eSizeType = eMTK_SIZE_TYPE_PIXEL;

//   m_layoutDescriptionOverlayInfo.frameWidth = 20;
//   m_layoutDescriptionOverlayInfo.frameHeight = 10;
//   m_layoutDescriptionOverlayInfo.eSizeType = eMTK_SIZE_TYPE_PERCENT;

   m_layoutDescriptionOverlayInfo.hBoundingSnapshot = MTK_ERROR;

   m_layoutDescriptionOverlayInfo.duration_ms = 5000;
   m_layoutDescriptionOverlayInfo.imgHeight = 0;
   m_layoutDescriptionOverlayInfo.imgWidth = 0;

   //printf("<<CSipSession::CSipsession()\n");
}

//*****************************************************************************
// Function: void CSipSession::~CSipSession()
// Description: destructor
// Return: none 
// Parameters: 
//*****************************************************************************
CSipSession::~CSipSession()
{
   if (m_callerIdOverlay)
   {
      delete m_callerIdOverlay;
   }
}

//*****************************************************************************
// Function: void CSipSession::SetSessionStateTo(ASipSessionState *state)
// Description: Set the state of a SIP session object
// Return: void 
// Parameters: ASipSessionState *state 
//*****************************************************************************
void CSipSession::SetSessionStateTo(ASipSessionState* state)
{
   m_sessionState = state;
   LogMsg(SessionName(), AsString("SIP Session State:") + m_sessionState->Desc());
}

//*****************************************************************************
// All session operations and event handling are delegated to the state
// object with the following functions
//*****************************************************************************

int CSipSession::OnInboundCallOffered() {return SessionState().OnInboundCallOffered(*this);}
int CSipSession::InboundCallSendProceeding() {return SessionState().InboundCallSendProceeding(*this);}
int CSipSession::OnInboundCallSendProceedingComplete() {return SessionState().OnInboundCallSendProceedingComplete(*this);}
int CSipSession::ReserveResource() {return SessionState().ReserveResource(*this);}
int CSipSession::OnReserveResourceComplete() {return SessionState().OnReserveResourceComplete(*this);}
int CSipSession::GetLocalMediaInfo() {return SessionState().GetLocalMediaInfo(*this);}
int CSipSession::OnGetLocalMediaInfoComplete() {return SessionState().OnGetLocalMediaInfoComplete(*this);}
int CSipSession::AcceptInboundCall() {return SessionState().AcceptInboundCall(*this);}
int CSipSession::OnAcceptInboundCallComplete() {return SessionState().OnAcceptInboundCallComplete(*this);}
int CSipSession::AnswerInboundCall() {return SessionState().AnswerInboundCall(*this);}
int CSipSession::OnAnswerInboundCallComplete() {return SessionState().OnAnswerInboundCallComplete(*this);}
int CSipSession::StartMedia() {return SessionState().StartMedia(*this);}
int CSipSession::OnStartMediaComplete() {return SessionState().OnStartMediaComplete(*this);}
int CSipSession::OpenConferenceParty() {return SessionState().OpenConferenceParty(*this);}
int CSipSession::OnOpenConferencePartyComplete() {return SessionState().OnOpenConferencePartyComplete(*this);}
int CSipSession::ConnectToConferenceParty() {return SessionState().ConnectToConferenceParty(*this);}
int CSipSession::OnConnectToConferencePartyComplete() {return SessionState().OnConnectToConferencePartyComplete(*this);}
int CSipSession::JoinConference() {return SessionState().JoinConference(*this); }
int CSipSession::OnJoinConferenceComplete(){return SessionState().OnJoinConferenceComplete(*this);}
int CSipSession::LeaveConference() {return SessionState().LeaveConference(*this); }
int CSipSession::OnLeaveConferenceComplete() {return SessionState().OnLeaveConferenceComplete(*this);}
int CSipSession::DisconnectFromConferenceParty() {return SessionState().DisconnectFromConferenceParty(*this);}
int CSipSession::OnDisconnectFromConferencePartyComplete() {return SessionState().OnDisconnectFromConferencePartyComplete(*this);}
int CSipSession::CloseConferenceParty() {return SessionState().CloseConferenceParty(*this);}
int CSipSession::OnCloseConferencePartyComplete() {return SessionState().OnCloseConferencePartyComplete(*this);}
int CSipSession::StopMedia() {return SessionState().StopMedia(*this);}
int CSipSession::OnStopMediaComplete() {return SessionState().OnStopMediaComplete(*this);}
int CSipSession::OnCallDisconnected() {return SessionState().OnCallDisconnected(*this);}
int CSipSession::ReleaseResource() {return SessionState().ReleaseResource(*this);}
int CSipSession::OnReleaseResourceComplete() {return SessionState().OnReleaseResourceComplete(*this);}
int CSipSession::ReleaseCall() {return SessionState().ReleaseCall(*this);}
int CSipSession::OnCallReleased() {return SessionState().OnCallReleased(*this);}
int CSipSession::OnGetIpmRxPortInfoComplete() {return SessionState().OnGetIpmRxPortInfoComplete(*this);}
int CSipSession::OnGetIpmTxPortInfoComplete() {return SessionState().OnGetIpmTxPortInfoComplete(*this);}
int CSipSession::OnGetCnfRxPortInfoComplete() {return SessionState().OnGetCnfRxPortInfoComplete(*this);}
int CSipSession::OnGetCnfTxPortInfoComplete() {return SessionState().OnGetCnfTxPortInfoComplete(*this);}
int CSipSession::OnGetMmRxPortInfoComplete() {return SessionState().OnGetMmRxPortInfoComplete(*this);}
int CSipSession::OnGetMmTxPortInfoComplete() {return SessionState().OnGetMmTxPortInfoComplete(*this);}
int CSipSession::OnCnfPortConnected() {return SessionState().OnCnfPortConnected(*this);}
int CSipSession::OnIpmPortConnected() {return SessionState().OnIpmPortConnected(*this);}
int CSipSession::OnMmPortConnected() {return SessionState().OnMmPortConnected(*this);}
int CSipSession::OnCnfPortDisconnected() {return SessionState().OnCnfPortDisconnected(*this); }
int CSipSession::OnIpmPortDisconnected() {return SessionState().OnIpmPortDisconnected(*this);
}
int CSipSession::OnMmPortDisconnected() {return SessionState().OnMmPortDisconnected(*this);}

int CSipSession::ConnectToMediaPlayer() {return SessionState().ConnectToMediaPlayer(*this);}
int CSipSession::OnConnectToMediaPlayerComplete() {return SessionState().OnConnectToMediaPlayerComplete(*this);}
int CSipSession::DisconnectFromMediaPlayer() {return SessionState().DisconnectFromMediaPlayer(*this);}
int CSipSession::OnDisconnectFromMediaPlayerComplete() {return SessionState().OnDisconnectFromMediaPlayerComplete(*this);}

int CSipSession::PlayGreeting() {return SessionState().PlayGreeting(*this);}
int CSipSession::OnPlayComplete() {return SessionState().OnPlayComplete(*this); }
int CSipSession::OnRecordComplete() {return SessionState().OnRecordComplete(*this); }

int CSipSession::OnDtmf(int a_dtmf) {return SessionState().OnDtmf(*this, a_dtmf);}

int CSipSession::OnIFrameRequest() {return SessionState().OnIFrameRequest(*this);}

int CSipSession::AddCallerIdOverlay() {return SessionState().AddCallerIdOverlay(*this);}
int CSipSession::AddConferenceLayoutOverlay(int a_layout) {return SessionState().AddConferenceLayoutOverlay(*this,a_layout);}
int CSipSession::RemoveCallerIdOverlay() {return SessionState().RemoveCallerIdOverlay(*this);}
int CSipSession::RemoveConferenceLayoutOverlay() {return SessionState().RemoveConferenceLayoutOverlay(*this);}
int CSipSession::OnAddImageOverlay(METAEVENT& a_metaEvent) {return SessionState().OnAddImageOverlay(*this,a_metaEvent);}
int CSipSession::OnRemoveImageOverlay(METAEVENT& a_metaEvent) {return SessionState().OnRemoveImageOverlay(*this,a_metaEvent);}

//*****************************************************************************
// Function: int CSipSession::PrimOnInboundCallOffered()
// Description: Handle an inbound call offered event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnInboundCallOffered()
{
   // TODO: inspect offer SDP and select coder

   //LogMsg("PrimOnInboundCallOffered()\n");

   CGcManager::AddActiveSipSession(this);

   BuildRemoteCapList(m_sipDevice->OfferSdp());
   BuildLocalCapList();

   // match audio, RCF2833 and video coders

   m_matchedAudioCoder = 0;
   m_matchedRfc2833Coder = 0;
   m_matchedVideoCoder = 0;

   //LogMsg(SessionName(), "Negotiate coder");
   std::vector<AMediaCoder*>::iterator l_remoteCoder;
   std::vector<AMediaCoder*>::iterator l_localCoder;

   for( l_remoteCoder=m_remoteMediaCoders.begin();
        l_remoteCoder != m_remoteMediaCoders.end();
        l_remoteCoder++)
   {
      LogMsg(SessionName(), AsString("Remote Offered:") + AsString((*l_remoteCoder)->GetPayloadCoderName()));
      (*l_remoteCoder)->ListProperties();

      // skip matched coders
      if (!strcmp((*l_remoteCoder)->GetPayloadCoderName(),"telephone-event"))
      {
         if (m_matchedRfc2833Coder)
         {
            continue;
         }
      }
      else if ( ((*l_remoteCoder)->GetMediaType() == "audio") && m_matchedAudioCoder)
      {
         continue;
      }
      else if ( ((*l_remoteCoder)->GetMediaType() == "video") && m_matchedVideoCoder)
      {
         continue;
      }

      // match to a local coder
      for( l_localCoder=m_localMediaCoders.begin();
           l_localCoder != m_localMediaCoders.end();
           l_localCoder++)
      {
         //Debug
         //LogMsg(SessionName(), "Compare to:" + AsString((*l_localCoder)->GetPayloadCoderName()));
         if ((*l_remoteCoder)->GetMediaType() != (*l_localCoder)->GetMediaType())
         {
            continue;
         }
         if ((*l_remoteCoder)->MatchCoder(*l_localCoder))
         {
            if (!strcmp((*l_remoteCoder)->GetPayloadCoderName(),"telephone-event"))
            {
               LogMsg(SessionName(), "Found telephony-event match for " + AsString((*l_remoteCoder)->GetPayloadCoderName()));
               m_matchedRfc2833Coder = dynamic_cast<AAudioCoder*>(*l_remoteCoder);
               break;
            }
            if ((*l_remoteCoder)->GetMediaType() == "audio")
            {
               LogMsg(SessionName(), "Found Audio match for " + AsString((*l_remoteCoder)->GetPayloadCoderName()));
               m_matchedAudioCoder = dynamic_cast<AAudioCoder*>(*l_remoteCoder);
               break;
            }
            if ((*l_remoteCoder)->GetMediaType() == "video")
            {
               LogMsg(SessionName(), "Found Video match for " + AsString((*l_remoteCoder)->GetPayloadCoderName()));
               //Debug
               //LogMsg(SessionName(), "Local Match to:");
               //(*l_localCoder)->ListProperties();
               m_matchedVideoCoder = dynamic_cast<AVideoCoder*>(*l_remoteCoder);
               break;
            }
         }
      }
   }

   if(m_matchedVideoCoder){
      //JM - for debugging Update local(encoder) video profile/level based on bitrate
      // Here LocalBitRate is current setting for encoder
      LogMsg(SessionName(), "Matched Video Bandwidth=" + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteBitRate()));
      LogMsg(SessionName(), "AS video Bandwidth=" + AsString(m_videoBandwidth*1000));
      LogMsg(SessionName(), "AS session Bandwidth=" + AsString(m_sessionBandwidth*1000));
      LogMsg(SessionName(), "default Video Bandwidth=" + AsString(m_defaultBandwidth*1000));
      // Update Matched bitrate based on endpoint AS value or application default
      if ((m_videoBandwidth) && (m_videoBandwidth*1000 < m_matchedVideoCoder->GetMatchedCoderRemoteBitRate()))
      {
         LogMsg(SessionName(), "Updating Matched Video Bandwidth based on");
         LogMsg(SessionName(), "video bandwidth b=AS: " + AsString(m_videoBandwidth));
         m_matchedVideoCoder->UpdateMatchedCoder(m_videoBandwidth*1000);
      }
      else if ((m_sessionBandwidth) && (m_sessionBandwidth*1000 < m_matchedVideoCoder->GetMatchedCoderRemoteBitRate()))
      {
         LogMsg(SessionName(), "Updating Matched Video Bandwidth based on");
         LogMsg(SessionName(), "session bandwidth b=AS: " + AsString(m_sessionBandwidth));
         m_matchedVideoCoder->UpdateMatchedCoder(m_sessionBandwidth*1000);
      }
      else if ((m_defaultBandwidth) && (m_defaultBandwidth*1000 < m_matchedVideoCoder->GetMatchedCoderRemoteBitRate()))
      {
         LogMsg(SessionName(), "Updating Matched Video Bandwidth based on");
         LogMsg(SessionName(), "application default bandwidth: " + AsString(m_defaultBandwidth));
         m_matchedVideoCoder->UpdateMatchedCoder(m_defaultBandwidth*1000);
      }
   }

   // for debugging
   if (m_matchedAudioCoder)
   {
      LogMsg(SessionName(), "Matched audio: " + AsString(m_matchedAudioCoder->GetPayloadCoderName()));
      m_matchedAudioCoder->ListProperties();
   }
   if (m_matchedRfc2833Coder)
   {
      LogMsg(SessionName(), "Matched telephone-event: " + AsString(m_matchedRfc2833Coder->GetPayloadCoderName()));
      m_matchedRfc2833Coder->ListProperties();
   }
   if (m_matchedVideoCoder)
   {
      LogMsg(SessionName(), "Matched video: " + AsString(m_matchedVideoCoder->GetPayloadCoderName()));
      m_matchedVideoCoder->ListProperties();
   }

   if (!m_matchedAudioCoder && !m_matchedVideoCoder)
   {
      LogMsg( SessionName(), "could not negotiate coders");
      // this should reject the call
      return -1;
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimInboundCallSendProceeding()
// Description: Handle a GC send proceeding command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimInboundCallSendProceeding()
{
   return m_sipDevice->GcSendProceeding();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnInboundCallSendProceedingComplete()
// Description: Handle an inbound call send proceeding completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnInboundCallSendProceedingComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimReserveResource()
// Description: Handle a reserve coder resources command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimReserveResource()
{
   if (m_matchedAudioCoder)
   {
      return m_ipmDevice->IpmReserveResource(m_matchedAudioCoder->GetDialogicResourceType());
   }
   LogMsg(SessionName(), AsString("no resource to reserve"));
   return -1;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnReserveResourceComplete()
// Description: Handle a reserve coder resources completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnReserveResourceComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimGetLocalMediaInfo()
// Description: Handle a get local media info command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimGetLocalMediaInfo()
{
   return m_ipmDevice->IpmGetLocalMediaInfo();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnGetLocalMediaInfoComplete()
// Description: Handle a get local media info completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetLocalMediaInfoComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimAcceptInboundCall()
// Description: Handle an accept inbound call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimAcceptInboundCall()
{
   return m_sipDevice->GcAcceptCall();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnAcceptInboundCallComplete()
// Description: Handle an accept inbound call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnAcceptInboundCallComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimAnswerInboundCall()
// Description: Handle an answer inbound call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimAnswerInboundCall()
{
   // build answer SDP for the call
   sdpSessionDescription& l_answerSdp = m_sipDevice->AnswerSdp();

   InitializeAnswerSdp(l_answerSdp );

   BuildAnswerSDP(m_sipDevice->OfferSdp(),l_answerSdp);

   return m_sipDevice->GcAnswerCall();

}
//*****************************************************************************
// Function: int CSipSession::PrimOnAnswerInboundCallComplete()
// Description: Handle an answer inbound call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnAnswerInboundCallComplete()
{
   //JM - verify ConfID is valid else set to DefaultConferenceID 
   if( CConference::IsConfRoomExist(m_cnfID)==false) {
      LogMsg( SessionName(), "ERROR Invalid ConfID" + AsString(m_cnfID));
      m_cnfID=CConference::GetDefaultConferenceID();
   }
   
   CConference* l_conference = CConference::GetConference(m_cnfID);
   int l_NumRealParties = l_conference->GetConferenceNumRealParties();
   int l_PartyPerConf =  m_systemConfiguration.getNoOfPartyPerConf();
   // JM - verify party can join the conference, otherwise return error
   // JM - TODO - Add some type of file play 'Conference Full' 
   if (l_NumRealParties+1 > l_PartyPerConf){
      LogMsg( SessionName(), "APP_WARNING Conference Full:RealParties=" + AsString(l_NumRealParties) + ":PartyPerConf=" + AsString(l_PartyPerConf));
      LogMsg( SessionName(), "APP_WARNING Dropping Call!!");
      return -1;
   }
   else {
      return 0;
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimStartMedia()
// Description: Handle a start media streaming command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimStartMedia()
{

   IPM_MEDIA_INFO& l_mediaInfo = m_ipmDevice->RemoteMediaInfo();
   memset(&l_mediaInfo, 0, sizeof(l_mediaInfo));

  //LogMsg(">>PrimStartMedia()\n");

   l_mediaInfo.unCount = 0;

   if (m_matchedAudioCoder)
   {
      // remote audio connection
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_REMOTE_RTP_INFO;
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.unPortId = m_matchedAudioCoder->GetRtpPort();
      strcpy(l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.cIPAddress,m_matchedAudioCoder->GetRtpIpAddress().c_str());
      ++l_mediaInfo.unCount;
   
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_REMOTE_RTCP_INFO;
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.unPortId = m_matchedAudioCoder->GetRtpPort()+1;
      strcpy(l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.cIPAddress,m_matchedAudioCoder->GetRtpIpAddress().c_str());
      ++l_mediaInfo.unCount;

      // remote audio coder
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_REMOTE_CODER_INFO;
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.eCoderType = m_matchedAudioCoder->GetDialogicCoderType();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.eFrameSize = m_matchedAudioCoder->GetMatchedCoderFrameSize();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.unFramesPerPkt = m_matchedAudioCoder->GetMatchedCoderFramesPerPacket();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.eVadEnable = m_matchedAudioCoder->GetMatchedCoderVAD();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.unCoderPayloadType = m_matchedAudioCoder->GetPayloadType();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.unRedPayloadType = m_matchedAudioCoder->GetRedundencyPayloadType();;
      ++l_mediaInfo.unCount;
      
      LogMsg("Remote(Tx/Encoder) Audio Coder: ");
      LogMsg("\t Media Address: " + AsString(m_matchedAudioCoder->GetRtpIpAddress().c_str()) + ":" + AsString(m_matchedAudioCoder->GetRtpPort()));
      LogMsg("\t coder type: " + AsString(AMediaCoder::GetDialogicCoderTypeDesc(m_matchedAudioCoder->GetDialogicCoderType())));
      LogMsg("\t payload type: " + AsString(m_matchedAudioCoder->GetPayloadType()));
      LogMsg("\t frame size: " + AsString(m_matchedAudioCoder->GetMatchedCoderFrameSize()));
      LogMsg("\t frames per packet: " + AsString(m_matchedAudioCoder->GetMatchedCoderFramesPerPacket()));
      LogMsg("\t VAD: " + AsString(m_matchedAudioCoder->GetMatchedCoderVAD()));
   
      if (m_matchedAudioCoder->GetExtendedInformation())
      {
         LogMsg("\t ExInfo: " + AsString(m_matchedAudioCoder->GetExtendedInformation()));
         l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_AUDIO_REMOTE_CODER_OPTIONS_INFO;
         l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.AudioCoderOptionsInfo.unVersion = IPM_AUDIO_CODER_OPTIONS_INFO_VERSION;
         l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.AudioCoderOptionsInfo.unCoderOptions= m_matchedAudioCoder->GetExtendedInformation();
         ++l_mediaInfo.unCount;
      }
   
      // local audio coder
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_LOCAL_CODER_INFO;
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.eCoderType = m_matchedAudioCoder->GetDialogicCoderType();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.eFrameSize = m_matchedAudioCoder->GetMatchedCoderFrameSize();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.unFramesPerPkt = m_matchedAudioCoder->GetMatchedCoderFramesPerPacket();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.eVadEnable = m_matchedAudioCoder->GetMatchedCoderVAD();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.unCoderPayloadType = m_matchedAudioCoder->GetPayloadType();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.CoderInfo.unRedPayloadType = m_matchedAudioCoder->GetRedundencyPayloadType();;
      ++l_mediaInfo.unCount;
 
      LogMsg("Local(Rx/Decoder) Audio Coder: ");
      LogMsg("\t Media Address: " + AsString(m_matchedAudioCoder->GetRtpIpAddress().c_str()) + ":" + AsString(m_matchedAudioCoder->GetRtpPort()));
      LogMsg("\t coder type: " + AsString(AMediaCoder::GetDialogicCoderTypeDesc(m_matchedAudioCoder->GetDialogicCoderType())));
      LogMsg("\t payload type: " + AsString(m_matchedAudioCoder->GetPayloadType()));
      LogMsg("\t frame size: " + AsString(m_matchedAudioCoder->GetMatchedCoderFrameSize()));
      LogMsg("\t frames per packet: " + AsString(m_matchedAudioCoder->GetMatchedCoderFramesPerPacket()));
      LogMsg("\t VAD: " + AsString(m_matchedAudioCoder->GetMatchedCoderVAD()));

      if (m_matchedAudioCoder->GetExtendedInformation())
      {
         LogMsg("\t ExInfo: " + AsString(m_matchedAudioCoder->GetExtendedInformation()));
         l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_AUDIO_LOCAL_CODER_OPTIONS_INFO;
         l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.AudioCoderOptionsInfo.unVersion = IPM_AUDIO_CODER_OPTIONS_INFO_VERSION;
         l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.AudioCoderOptionsInfo.unCoderOptions= m_matchedAudioCoder->GetExtendedInformation();
         ++l_mediaInfo.unCount;
      }
   }
   if (m_matchedVideoCoder)
   {
      // remote video connection
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_VIDEO_REMOTE_RTP_INFO;
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.unPortId = m_matchedVideoCoder->GetRtpPort(); 
      strcpy(l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.cIPAddress, m_matchedVideoCoder->GetRtpIpAddress().c_str());
      ++l_mediaInfo.unCount;

      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_VIDEO_REMOTE_RTCP_INFO;
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.unPortId = m_matchedVideoCoder->GetRtpPort()+1; 
      strcpy(l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.PortInfo.cIPAddress, m_matchedVideoCoder->GetRtpIpAddress().c_str());
      ++l_mediaInfo.unCount;

      // remote video coder
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_VIDEO_REMOTE_CODER_INFO;
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo.eCoderType = m_matchedVideoCoder->GetDialogicCoderType();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo.unCoderPayloadType = m_matchedVideoCoder->GetPayloadType();


      IPM_VIDEO_CODER_INFO_EX* l_remoteVideoCoderInfoEx = &m_ipmDevice->RemoteVideoCoderInfoEx();
      INIT_IPM_VIDEO_CODER_INFO_EX(l_remoteVideoCoderInfoEx);
      l_remoteVideoCoderInfoEx->eProfile = m_matchedVideoCoder->GetMatchedCoderRemoteVideoProfile();
      l_remoteVideoCoderInfoEx->eLevel = m_matchedVideoCoder->GetMatchedCoderRemoteVideoLevel();
      l_remoteVideoCoderInfoEx->eImageWidth = m_matchedVideoCoder->GetMatchedCoderRemoteImageWidth();
      l_remoteVideoCoderInfoEx->eImageHeight = m_matchedVideoCoder->GetMatchedCoderRemoteImageHeight();
      //TODO: Add MPEG4 and H264 Specific Coder Info
      //l_remoteVideoCoderInfoEx->unVisualConfigSize = 0;
      //l_remoteVideoCoderInfoEx->szVisualConfiguration = NULL;
#ifdef USE_H264
      l_remoteVideoCoderInfoEx->eH264_PacketizationMode = m_matchedVideoCoder->GetMatchedPacketizationMode();
      l_remoteVideoCoderInfoEx->eH264_AcceptRedundantSlices  = ACCEPT_REDUNDAND_SLICES_DEFAULT; //DEFAULT=false
      l_remoteVideoCoderInfoEx->eH264_ProfileIOP             = H264_PROFILE_IOP_DEFAULT; //DEFAULT=0
      l_remoteVideoCoderInfoEx->unH264_MaxNalUnitSize      = H264_MAX_NAL_UNIT_SIZE_DEFAULT; //DEFAULT=1400
#endif

      LogMsg("Remote(Tx/Encoder) Video Coder: "); 
      LogMsg("\t Media Address: " + AsString(m_matchedAudioCoder->GetRtpIpAddress().c_str()) + ":" + AsString(m_matchedVideoCoder->GetRtpPort()));
      LogMsg("\t coder type: " + AsString(AMediaCoder::GetDialogicCoderTypeDesc(m_matchedVideoCoder->GetDialogicCoderType())));
      LogMsg("\t payload type: " + AsString(m_matchedVideoCoder->GetPayloadType()));
      LogMsg("\t video profile: " + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteVideoProfile()));
      LogMsg("\t video level: " + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteVideoLevel()));
      LogMsg("\t image width: " + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteImageWidth()));
      LogMsg("\t image height: " + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteImageHeight()));
      LogMsg("\t frames/sec: " + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteFramesPerSecond()));
      LogMsg("\t bandwidth: " + AsString(m_matchedVideoCoder->GetMatchedCoderRemoteBitRate()));
      LogMsg("\t packetization mode: " + AsString(l_remoteVideoCoderInfoEx->eH264_PacketizationMode));
      LogMsg("\t accept redundant slices: " + AsString(l_remoteVideoCoderInfoEx->eH264_AcceptRedundantSlices));
      LogMsg("\t profile iop: 0x" + AsHexString(l_remoteVideoCoderInfoEx->eH264_ProfileIOP));
      LogMsg("\t max nal unit size: " + AsString(l_remoteVideoCoderInfoEx->unH264_MaxNalUnitSize));

      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo.pExtraCoderInfo = l_remoteVideoCoderInfoEx;
      ++l_mediaInfo.unCount;

      // local video coder
      l_mediaInfo.MediaData[l_mediaInfo.unCount].eMediaType = MEDIATYPE_VIDEO_LOCAL_CODER_INFO;
      INIT_IPM_VIDEO_CODER_INFO(&l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo);
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo.eCoderType = m_matchedVideoCoder->GetDialogicCoderType();
      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo.unCoderPayloadType = m_matchedVideoCoder->GetPayloadType();

      IPM_VIDEO_CODER_INFO_EX* l_localVideoCoderInfoEx = &m_ipmDevice->RemoteVideoCoderInfoEx();
      INIT_IPM_VIDEO_CODER_INFO_EX(l_localVideoCoderInfoEx);
      l_localVideoCoderInfoEx->eProfile = m_matchedVideoCoder->GetMatchedCoderLocalVideoProfile();
      l_localVideoCoderInfoEx->eLevel = m_matchedVideoCoder->GetMatchedCoderLocalVideoLevel();
      l_localVideoCoderInfoEx->eImageWidth = m_matchedVideoCoder->GetMatchedCoderLocalImageWidth();
      l_localVideoCoderInfoEx->eImageHeight = m_matchedVideoCoder->GetMatchedCoderLocalImageHeight();
      l_localVideoCoderInfoEx->eFramesPerSec = m_matchedVideoCoder->GetMatchedCoderLocalFramesPerSecond();
      l_localVideoCoderInfoEx->unBitRate = m_matchedVideoCoder->GetMatchedCoderLocalBitRate();
      //TODO: Add MPEG4 and H264 Specific Coder Info
      //l_localVideoCoderInfoEx->unVisualConfigSize = 0;
      //l_localVideoCoderInfoEx->szVisualConfiguration = NULL;
#ifdef USE_H264
      l_localVideoCoderInfoEx->eH264_PacketizationMode = m_matchedVideoCoder->GetMatchedPacketizationMode();
      l_localVideoCoderInfoEx->eH264_AcceptRedundantSlices  = ACCEPT_REDUNDAND_SLICES_DEFAULT; //DEFAULT=false
      l_localVideoCoderInfoEx->eH264_ProfileIOP             = H264_PROFILE_IOP_DEFAULT; //DEFAULT=0
      l_localVideoCoderInfoEx->unH264_MaxNalUnitSize      = H264_MAX_NAL_UNIT_SIZE_DEFAULT; //DEFAULT=1400
#endif
      
      LogMsg("Local(Rx/Decoder) Video Coder: "); 
      LogMsg("\t Media Address: " + AsString(m_matchedAudioCoder->GetRtpIpAddress().c_str()) + ":" + AsString(m_matchedVideoCoder->GetRtpPort()));
      LogMsg("\t coder type: " + AsString(AMediaCoder::GetDialogicCoderTypeDesc(m_matchedVideoCoder->GetDialogicCoderType())));
      LogMsg("\t payload type: " + AsString(m_matchedVideoCoder->GetPayloadType()));
      LogMsg("\t video profile: " + AsString(m_matchedVideoCoder->GetMatchedCoderLocalVideoProfile()));
      LogMsg("\t video level: " + AsString(m_matchedVideoCoder->GetMatchedCoderLocalVideoLevel()));
      LogMsg("\t image width: " + AsString(m_matchedVideoCoder->GetMatchedCoderLocalImageWidth()));
      LogMsg("\t image height: " + AsString(m_matchedVideoCoder->GetMatchedCoderLocalImageHeight()));
      LogMsg("\t frames/sec: " + AsString(m_matchedVideoCoder->GetMatchedCoderLocalFramesPerSecond()));
      LogMsg("\t bandwidth: " + AsString(m_matchedVideoCoder->GetMatchedCoderLocalBitRate()));
      LogMsg("\t packetization mode: " + AsString(l_localVideoCoderInfoEx->eH264_PacketizationMode));
      LogMsg("\t accept redundant slices: " + AsString(l_localVideoCoderInfoEx->eH264_AcceptRedundantSlices));
      LogMsg("\t profile iop: 0x" + AsHexString(l_localVideoCoderInfoEx->eH264_ProfileIOP));
      LogMsg("\t max nal unit size: " + AsString(l_localVideoCoderInfoEx->unH264_MaxNalUnitSize));

      l_mediaInfo.MediaData[l_mediaInfo.unCount].mediaInfo.VideoCoderInfo.pExtraCoderInfo = l_localVideoCoderInfoEx;
      ++l_mediaInfo.unCount;
   }

   if (m_matchedRfc2833Coder)
   {
      LogMsg("Set RFC2833 Payload Type: " + AsString(m_matchedRfc2833Coder->GetPayloadType()));
      m_ipmDevice->SetRfc2833PayloadTypes(DEFAULT_RFC2833_PAYLOAD_TYPE, m_matchedRfc2833Coder->GetPayloadType());
   }

   int ret = m_ipmDevice->IpmStartMedia(DATA_IP_TDM_BIDIRECTIONAL);
   if (ret==0){
      m_bMediaStarted=true;
   }
   else {
      m_bMediaStarted=false;
   }
   return ret;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnStartMediaComplete()
// Description: Handle a start media streaming completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnStartMediaComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimOpenConferenceParty()
// Description: Handle an open conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOpenConferenceParty()
{
   m_conferenceParty = new CConferencePartyDevice(m_sipDevice->DeviceName(), CConference::GetConferenceBoard(), this);
   CGcManager::RegisterDevice(m_conferenceParty);
   return m_conferenceParty->Open();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnOpenConferencePartyComplete()
// Description: Handle an open conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnOpenConferencePartyComplete()
{
   CGcManager::RegisterDevice(m_conferenceParty->Device(), m_conferenceParty);
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimConnectToConferenceParty()
// Description: Handle a connect ot conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimConnectToConferenceParty()
{
   m_portInfoSubState = 0;
   m_ipmDevice->GetReceivePortInfo();
   m_ipmDevice->GetTransmitPortInfo();
   m_conferenceParty->GetReceivePortInfo();
   m_conferenceParty->GetTransmitPortInfo();
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnConnectToConferencePartyComplete()
// Description: Handle a connect ot conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnConnectToConferencePartyComplete()
{
   m_bConfPartyConnected=true;
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimJoinConference()
// Description: Handle a join conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimJoinConference()
{
   LogMsg( SessionName(), "CSipSession::PrimJoinConference() " + AsString(m_cnfID));

   CConference* l_conference = CConference::GetConference(m_cnfID);
   l_conference->SetRealPartiesSipSessionInfo(this);
   return l_conference->AddConferenceParty(m_conferenceParty);
}

//*****************************************************************************
// Function: int CSipSession::PrimOnJoinConferenceComplete()
// Description: Handle a join conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnJoinConferenceComplete()
{
// SKS need to put here	 JOIN CONF
   m_sipDevice->GcSendIFrameRequest();
   CConference* l_conference = CConference::GetConference(m_cnfID);
   return l_conference->OnConferencePartyAdded(m_conferenceParty);
}

//*****************************************************************************
// Function: int CSipSession::PrimLeaveConference()
// Description: Handle a leave conference command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimLeaveConference()
{
   CConference* l_conference = CConference::GetConference(m_cnfID);
// SKS TODO if single person is leaving check if we need to use list
//	l_conference->RemoveRealPartiesSipSessionInfo(this);
   return l_conference->RemoveConferenceParty(m_conferenceParty);
}

//*****************************************************************************
// Function: int CSipSession::PrimOnLeaveConferenceComplete()
// Description: Handle a leave conference completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnLeaveConferenceComplete()
{
   return CConference::GetConference(m_cnfID)->OnConferencePartyRemoved(m_conferenceParty);
}

//*****************************************************************************
// Function: int CSipSession::PrimDisconnectFromConferenceParty()
// Description: Handle a disconnect from conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimDisconnectFromConferenceParty()
{
   m_portDisconnectSubState = 0;
   int res = dev_PortDisconnect(m_ipmDevice->Device(), &m_portConnectInfoListIpmTxCnfRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_ipmDevice->Device(), "dev_PortDisconnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortDisconnect");
   }

   res = dev_PortDisconnect(m_conferenceParty->Device(), &m_portConnectInfoListCnfTxIpmRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_conferenceParty->Device(), "dev_PortDisconnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortDisconnect");
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnDisconnectFromConferencePartyComplete()
// Description: Handle a disconnect from conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnDisconnectFromConferencePartyComplete()
{
   m_bConfPartyConnected=false;
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimCloseConferenceParty()
// Description: Handle a close conference party command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimCloseConferenceParty()
{
   return m_conferenceParty->Close();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnCloseConferencePartyComplete()
// Description: Handle a close conference party completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnCloseConferencePartyComplete()
{
   // TODO: fix teardown
//   CGcManager::UnregisterDevice(m_conferenceParty->ID());
//   CGcManager::UnregisterDevice(m_conferenceParty->CnfPartyDevice());
//   delete m_conferenceParty;
   m_conferenceParty = 0;
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimStopMedia()
// Description: Handle a stop media streaming command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimStopMedia()
{
   int ret = m_ipmDevice->IpmStopMedia();
   if (ret==0){
      m_bMediaStarted=false;
   }
   else {
      m_bMediaStarted=true;
   }
   return ret;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnStopMediaComplete()
// Description: Handle a stop media streaming completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnStopMediaComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnCallDisconnected()
// Description: Handle a call disconnected event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnCallDisconnected()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimReleaseResource()
// Description: Handle a release coder resource command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimReleaseResource()
{
   return m_ipmDevice->IpmReleaseResource();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnReleaseResourceComplete()
// Description: Handle a release coder resource completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnReleaseResourceComplete()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimReleaseCall()
// Description: Handle a release call command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimReleaseCall()
{
   return m_sipDevice->GcDropCall();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnCallReleased()
// Description: Handle a release call completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnCallReleased()
{
   CGcManager::RemoveActiveSipSession(this);

   //JM - Reset Bandwidths to 0 after call released
   m_videoBandwidth = 0;
   m_audioBandwidth = 0;
   m_sessionBandwidth = 0;
   return 0;
}


//*****************************************************************************
// Function: int CSipSession::Device()
// Description: Return the devcie
// Return: int - 0 for success, otherwise error 
// Parameters: none 
// Notes:
//    Use the SIP device for the SIP session
//*****************************************************************************
int CSipSession::Device()
{
   return m_sipDevice->Device();
}

//*****************************************************************************
// Function: int CSipSession::ListenTo(CSipSession *a_sipSession)
// Description: Handle a listen command
// Return: int - 0 for success, otherwise error 
// Parameters: CSipSession *a_sipSession 
//*****************************************************************************
int CSipSession::ListenTo(CSipSession* a_sipSession)
{
   return m_ipmDevice->ListenTo( a_sipSession->IpmDevice());
}

//*****************************************************************************
// Function: int CSipSession::CreatePortConnectInfoList(DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList, const DM_PORT_INFO_LIST &a_txPortInfoList, const DM_PORT_INFO_LIST &a_rxPortInfoList)
// Description: Create a port connect info list
// Return: int - number of items in the port connection info list
// Parameters: DM_PORT_CONNECT_INFO_LIST &a_portConnectionInfoList 
//             const DM_PORT_INFO_LIST &a_txPortInfoList 
//             const DM_PORT_INFO_LIST &a_rxPortInfoList 
//*****************************************************************************
int CSipSession::CreatePortConnectInfoList(  DM_PORT_CONNECT_INFO_LIST& a_portConnectionInfoList,
                                             const DM_PORT_INFO_LIST& a_txPortInfoList,
                                             const DM_PORT_INFO_LIST& a_rxPortInfoList)
{
   INIT_DM_PORT_CONNECT_INFO_LIST(&a_portConnectionInfoList);

   // Loop through all transmit ports of 1st device
   int k = 0;
   int i;
   for (i = 0; i < a_txPortInfoList.unCount; ++i)
   {
      DM_PORT_MEDIA_TYPE type_tx = a_txPortInfoList.port_info[i].port_media_type;
      if (type_tx == DM_PORT_MEDIA_TYPE_NBUP)
      {
         continue;
      }

      // find appropriate RX port on 2nd device
      bool bFound = false;
      int j;
      for (j = 0; j < a_rxPortInfoList.unCount; ++j)
      {
         DM_PORT_MEDIA_TYPE type_rx = a_rxPortInfoList.port_info[j].port_media_type;
         if (type_tx == type_rx)
         {
            bFound = true;
            break;
         }
      }
      if (!bFound)
      {
         continue;
      }

      // create element of connect list
      DM_PORT_CONNECT_INFO& info = a_portConnectionInfoList.port_connect_info[k];
      INIT_DM_PORT_CONNECT_INFO(&info);

      info.unFlags = DMFL_TRANSCODE_ON;
      info.port_info_tx = a_txPortInfoList.port_info[i];
      info.port_info_rx = a_rxPortInfoList.port_info[j];
      ++k;
   }
   a_portConnectionInfoList.unCount = k;
   return k;
}


//*****************************************************************************
// Function: int CSipSession::PrimOnGetIpmRxPortInfoComplete()
// Description: Handle a get RX port info completion event for an IPM device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetIpmRxPortInfoComplete()
{
   m_portInfoSubState |= IPM_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnGetIpmTxPortInfoComplete()
// Description: Handle a get TX port info completion event for an IPM device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetIpmTxPortInfoComplete()
{
   m_portInfoSubState |= IPM_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnGetCnfRxPortInfoComplete()
// Description: Handle a get RX port info completion event for an CNF device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetCnfRxPortInfoComplete()
{
   m_portInfoSubState |= CNF_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnGetCnfTxPortInfoComplete()
// Description: Handle a get TX port info completion event for an CNF device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetCnfTxPortInfoComplete()
{
   m_portInfoSubState |= CNF_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnGetMmRxPortInfoComplete()
// Description: Handle a get RX port info completion event for an MM device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetMmRxPortInfoComplete()
{
   m_portInfoSubState |= MM_RX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnGetMmTxPortInfoComplete()
// Description: Handle a get TX port info completion event for an MM device
// Return: int - value of port info substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnGetMmTxPortInfoComplete()
{
   m_portInfoSubState |= MM_TX_PORT_INFO;
   return m_portInfoSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimIpmMmPortConnect()
// Description: Handle a port conmnect command for an IPM and a MM device
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimIpmMmPortConnect()
{
   m_portConnectSubState = 0;
   int num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListIpmTxCnfRx,
                                                      m_ipmDevice->TxPortInfoList(),
                                                      m_mmDevice->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg( SessionName(), "No matched ports found");
      return -1;
   }

   int res = dev_PortConnect(m_ipmDevice->Device(), &m_portConnectInfoListIpmTxCnfRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_ipmDevice->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortConnect");
   }

   num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListCnfTxIpmRx,
                                                  m_mmDevice->TxPortInfoList(),
                                                  m_ipmDevice->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg( SessionName(), "No matched ports found");
      return -1;
   }

   res = dev_PortConnect(m_mmDevice->Device(), &m_portConnectInfoListCnfTxIpmRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_mmDevice->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortConnect");
   }

   return res;
}

//*****************************************************************************
// Function: int CSipSession::PrimPortConnect()
// Description: Handle a port conmnect command for an IPM and a CNF device
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimPortConnect()
{
   m_portConnectSubState = 0;
   int num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListIpmTxCnfRx,
                                                      m_ipmDevice->TxPortInfoList(),
                                                      m_conferenceParty->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg( SessionName(), "No matched ports found");
      return -1;
   }

   int res = dev_PortConnect(m_ipmDevice->Device(), &m_portConnectInfoListIpmTxCnfRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_ipmDevice->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortConnect");
   }

   num_matched_ports = CreatePortConnectInfoList( m_portConnectInfoListCnfTxIpmRx,
                                                  m_conferenceParty->TxPortInfoList(),
                                                  m_ipmDevice->RxPortInfoList());
   if (0 == num_matched_ports)
   {
      LogMsg( SessionName(), "No matched ports found");
      return -1;
   }

   res = dev_PortConnect(m_conferenceParty->Device(), &m_portConnectInfoListCnfTxIpmRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_conferenceParty->Device(), "dev_PortConnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortConnect");
   }

   return res;
}


//*****************************************************************************
// Function: int CSipSession::PrimOnCnfPortConnected()
// Description: Handle a CNF device port connect completion event
// Return: int - value of port connect substate
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnCnfPortConnected()
{
   m_portConnectSubState |= CNF_TX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnIpmPortConnected()
// Description: Handle an IPM device port connect completion event
// Return: int - value of port connect substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnIpmPortConnected()
{
   m_portConnectSubState |= IPM_TX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnMmPortConnected()
// Description: Handle a MM device port connect completion event
// Return: int - value of port connect substate 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnMmPortConnected()
{
   m_portConnectSubState |= MM_TX_CONNECT;
   return m_portConnectSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnCnfPortDisconnected()
// Description: Handle a CNF device port disconnected completion event
// Return: int - value of port disconnect substate
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnCnfPortDisconnected()
{
   m_portDisconnectSubState |= CNF_TX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnIpmPortDisconnected()
// Description: Handle an IPM device port disconnected completion event
// Return: int - value of port disconnect substate
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnIpmPortDisconnected()
{
   m_portDisconnectSubState |= IPM_TX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnMmPortDisconnected()
// Description: Handle a MM device port disconnected completion event
// Return: int - value of port disconnect substate
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnMmPortDisconnected()
{
   m_portDisconnectSubState |= MM_TX_DISCONNECT;
   return m_portDisconnectSubState;
}

//*****************************************************************************
// Function: int CSipSession::PrimConnectToMediaPlayer()
// Description: Handle a connect to media player command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimConnectToMediaPlayer()
{
   m_portInfoSubState = 0;
   m_ipmDevice->GetReceivePortInfo();
   m_ipmDevice->GetTransmitPortInfo();
   m_mmDevice->GetReceivePortInfo();
   m_mmDevice->GetTransmitPortInfo();
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnConnectToMediaPlayerComplete()
// Description: Handle a connect to media player completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnConnectToMediaPlayerComplete()
{
   m_bMediaPlayerConnected=true;
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimDisconnectFromMediaPlayer()
// Description: Handle a disconnect from media player command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimDisconnectFromMediaPlayer()
{
   m_portDisconnectSubState = 0;
   int res = dev_PortDisconnect(m_ipmDevice->Device(), &m_portConnectInfoListIpmTxCnfRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_ipmDevice->Device(), "dev_PortDisconnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortDisconnect");
   }

   res = dev_PortDisconnect(m_mmDevice->Device(), &m_portConnectInfoListCnfTxIpmRx, NULL);
   if (-1 == res)
   {
      ErrMsg( m_mmDevice->Device(), "dev_PortDisconnect");
      return res;
   }
   else
   {
      LogMsg( SessionName(), "dev_PortDisconnect");
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnDisconnectFromMediaPlayerComplete()
// Description: Handle a disconnect from media player completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnDisconnectFromMediaPlayerComplete()
{
   m_bMediaPlayerConnected=false;
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimPlayGreeting()
// Description: Handle a play greeting command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimPlayGreeting()
{
	m_mmDevice->SetClip(m_systemConfiguration.MediaFilePath(), m_systemConfiguration.GreetingClip());
   return m_mmDevice->MmPlay();
}

//*****************************************************************************
// Function: int CSipSession::PrimPlayInvalidPinClip()
// Description: Handle a play greeting command
// Return: int - 0 for success, otherwise error
// Parameters: none
//*****************************************************************************
int CSipSession::PrimPlayInvalidPinClip()
{
   LogMsg( SessionName(), "PrimPlayInvalidPinClip()");
   m_mmDevice->SetClip(m_systemConfiguration.MediaFilePath(), m_systemConfiguration.InvalidPinClip());
   return m_mmDevice->MmPlay();
}



//*****************************************************************************
// Function: int CSipSession::PrimOnPlayComplete()
// Description: Handle a play completion event
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnPlayComplete()
{
   return 0;
}


//*****************************************************************************
// Function: int CSipSession::GetConfPINLen()
// Description: Get ConfPinLen
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::GetConfPINLen()
{
	int nRetVal = 0;
	CConference* l_conference = CConference::GetConference(m_cnfID);
        nRetVal = l_conference->GetConferencePINLen(m_cnfID);
        return nRetVal;
}

#if 0
//*****************************************************************************
// Function: int CSipSession::VerifyConfPIN()
// Description: Verify Conference PIN matches
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
bool CSipSession::VerifyConfPIN(string cnfPIN)
{
	bool bRet=true;
	CConference* l_conference = CConference::GetConference(m_cnfID);

	if(cnfPIN!=l_conference->GetConferencePIN(m_cnfID))
	{
            ErrMsg( Device(), "VerifyConfPIN() actualPIN=" + l_conference->GetConferencePIN(m_cnfID) + " Entered=" + cnfPIN);
            bRet=false;
	}
	m_bPinAccepted=bRet;

	return bRet;
}
#endif

//*****************************************************************************
// Function: int CSipSession::VerifyConfPIN()
// Description: Verify Conference PIN matches DigitBuf
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
bool CSipSession::VerifyConfPIN()
{
	bool bRet=true;
	CConference* l_conference = CConference::GetConference(m_cnfID);

	if(m_digitBuf!=l_conference->GetConferencePIN(m_cnfID))
	{
           ErrMsg( Device(), "VerifyConfPIN() actualPIN=" + l_conference->GetConferencePIN(m_cnfID) + " Entered=" + m_digitBuf);
           bRet=false;
	}
	m_bPinAccepted=bRet;
	return bRet;
}


//*****************************************************************************
// Function: int CSipSession::collectDigit(std::string digit)
// Description: RecieveDigit into DigitBuf
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::collectDigit(std::string digit)
{
	m_digitBuf = m_digitBuf + digit;
        m_nDigitCnt++;
 
        return 0;
}

//*****************************************************************************
// Function: void CSipSession::clearDigits()
// Description: clear DigitBuf
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
void CSipSession::clearDigits()
{
	m_digitBuf = "";
        m_nDigitCnt = 0;
        return;
}

//*****************************************************************************
// Function: int CSipSession::PrimStopPlay(int a_dtmf)
// Description: Handle a stop play command
// Return: int - 0 for success, otherwise error 
// Parameters: int a_dtmf 
//*****************************************************************************
int CSipSession::PrimStopPlay(int a_dtmf)
{
   return m_mmDevice->MmStop();
}

//*****************************************************************************
// Function: int CSipSession::PrimCycleConferenceLayout()
// Description: Handle a cycle conference layout command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimCycleConferenceLayout()
{
   CConference* l_conference = CConference::GetConference(m_cnfID);
   return l_conference->CycleLayout();
}

//*****************************************************************************
// Function: int CSipSession::PrimRotateConferenceParties()
// Description: Handle a rotate conference parties command
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimRotateConferenceParties()
{
   CConference* l_conference = CConference::GetConference(m_cnfID);
   return l_conference->RotateParties();
}

//*****************************************************************************
// Function: int CSipSession::PrimOnIFrameRequest()
// Description: Handle an IFrame request
// Return: int - 0 for success, otherwise error 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimOnIFrameRequest()
{
   return m_ipmDevice->IpmGenerateIFrame();
}

//*****************************************************************************
// Function: bool CSipSession::HaveGreeting()
// Description: Answer if the SIP session has a greeting to play
// Return: bool 
// Parameters: none 
//*****************************************************************************
bool CSipSession::HaveGreeting()
{
   return (m_mmDevice->HaveClip());
}

//*****************************************************************************
// Function: int CSipSession::BuildRemoteCapList(sdpSessionDescription &a_remoteSdp)
// Description: Build the list of remote capabilities
// Return: int 
// Parameters: sdpSessionDescription &a_remoteSdp 
//*****************************************************************************
int CSipSession::BuildRemoteCapList(sdpSessionDescription& a_remoteSdp)
{
   int  l_attributeCount = 0;
   int  l_mediaFormatCount = 0;

   std::vector<int> l_mediaFormats;


   //clear the remote capabilites list
   for(unsigned int i=0; i<m_remoteMediaCoders.size(); i++)
   {
      if (m_remoteMediaCoders[i])
      {
         delete m_remoteMediaCoders[i];
      }
   }
   m_remoteMediaCoders.clear();

   //Reset payload values
   for(int i=0; i < DYNAMIC_PAYLOAD_MAX; i++)
   {
      m_remotePayloads[i] = -1;
   }

   //c= connection info. 
   std::string l_connectionIpAddr;

   std::string l_attribute;

   bool l_found = false;
   bool l_created = false;
   int l_return = 0;
   sdpMedia* l_sdpMedia = 0;
   sdpMediaDescription* l_sdpMediaDescription = 0;
   sdpAttribute* l_sdpAttribute = 0;
   sdpAttributeList* l_sdpAttributeList = 0;
   sdpConnection* l_sdpMediaConnection = 0;

   sdpBandwidth* l_sessionBandwidth = a_remoteSdp.bandwidth();
   if (strcmp(l_sessionBandwidth->getModifier(),"AS") == 0)
   {
      m_sessionBandwidth = atoi(l_sessionBandwidth->getBandwidthValue());
      LogMsg("Session description bandwidth: " + AsString(l_sessionBandwidth->getModifier()) 
                                               + ":" + AsString(m_sessionBandwidth));
   }
   // JM - CT session bandwidth parameter - same as AS? 
   else if (strcmp(l_sessionBandwidth->getModifier(),"CT") == 0)
   {
      m_sessionBandwidth = atoi(l_sessionBandwidth->getBandwidthValue());
      LogMsg("Session description bandwidth: " + AsString(l_sessionBandwidth->getModifier())
                                               + ":" + AsString(m_sessionBandwidth));
   }



   //For each m= line, process the media information.
   int l_mediaDescriptionCount = a_remoteSdp.mediaDescriptionList()->numItem();

   if(l_mediaDescriptionCount == 0)
   {
        LogMsg(SessionName(), "BuildRemoteCapList - SDP does not contain any m= lines.");
   }
   else  //Check if all ports are 0
   {
      bool l_portsAreZero = true;
      for(int k=0; k < l_mediaDescriptionCount; k++)
      {
         l_sdpMediaDescription = a_remoteSdp.mediaDescriptionList()->getItem(k);
         if(l_sdpMediaDescription->media()->getPort() != 0)
         {
            l_portsAreZero = false;
            break;
         }
      }

      if(l_portsAreZero == true)
      {
         LogMsg(SessionName(), "BuildRemoteCapList -  All m= lines in remote SDP have port=0.");
         return -1;
      }
      else
      {
         //LogMsg(SessionName(), "BuildRemoteCapList -  SDP contains " + AsString(l_mediaDescriptionCount) + "m= lines.");
      }
   }

   // for each media description
   for(int h=0; h < l_mediaDescriptionCount; h++)
   { 
      l_sdpMediaDescription = a_remoteSdp.mediaDescriptionList()->getItem(h);

      l_sdpMedia = l_sdpMediaDescription->media();

      // The SDP connection IP info can exist at the session level, the media level, or both. 
      // Use SDP connection at the media level if it exists, otherwise use the session
      // level info.
      // Try media level first.
      l_sdpMediaConnection = l_sdpMediaDescription->connection();
      if (strlen(l_sdpMediaConnection->getAddress()) == 0)
      {
         // Default to the session level.
         l_sdpMediaConnection = a_remoteSdp.connection();
      }

      // the default mode will be Full Duplex
      eIPM_DATA_DIRECTION l_localMediaDirection = DATA_IP_TDM_BIDIRECTIONAL;

      // Perform sanity check
      l_return = ValidateMediaConnection( *l_sdpMediaDescription,
                                          *l_sdpMediaConnection,
                                          l_localMediaDirection);
      if(l_return != 0)
      {
         return l_return;
      }

      //Skip unsupported m= lines
      if(l_localMediaDirection == DATA_IP_DIR_NONE)
      {
         continue;
      }

      //Retrieve the Connection IP address
      l_connectionIpAddr.assign(l_sdpMediaConnection->getAddress());

      // check for bandwidth parameter
      sdpBandwidth* l_mediaBandwidth = l_sdpMediaDescription->bandwidth();
      if (strcmp(l_mediaBandwidth->getModifier(),"AS") == 0)
      {
         sdpMedia* l_media = l_sdpMediaDescription->media();
         if (strcmp(l_media->getMedia(), "audio") == 0)
         {
            m_audioBandwidth = atoi(l_mediaBandwidth->getBandwidthValue());
            LogMsg(AsString(l_media->getMedia()) + " media description bandwidth: " 
                                               + AsString(l_mediaBandwidth->getModifier()) 
                                               + ":" + AsString(m_audioBandwidth));
         }
         else if(strcmp(l_media->getMedia(), "video") == 0)
         {
            m_videoBandwidth = atoi(l_mediaBandwidth->getBandwidthValue());
            LogMsg(AsString(l_media->getMedia()) + " media description bandwidth: " 
                                               + AsString(l_mediaBandwidth->getModifier()) 
                                               + ":" + AsString(m_videoBandwidth));
         }
      }

      //Get the number of media formats stored in this m= line.
      l_mediaFormatCount = l_sdpMedia->getNumFormat();

      //Reset the vector
      l_mediaFormats.clear();

      //Store the media formats in the vector, It's important to perserve the order. 
      for(int j=0; j<l_mediaFormatCount; j++)
      {
         l_mediaFormats.push_back(atoi(l_sdpMedia->getFormat(j)));
      }

      l_sdpAttributeList = l_sdpMediaDescription->attributeList();
      l_attributeCount = l_sdpAttributeList->numItem();

      //printf("BuildRemoteCapList - Media descriptor #%d = %s\n",h+1,l_sdpMedia->getValue());
      //printf("BuildRemoteCapList - Media descriptor #%d contains %d media format(s)\n",h+1,l_mediaFormatCount);
      //printf("BuildRemoteCapList - Media descriptor #%d contains %d attribute(s)\n",h+1,l_attributeCount);

      // Process all rtpmap entries first to ensure
      // the creation of new media entries. Preserve the
      // order of the format to maintain requested priority
      // Associate the rtpmap with the corrsponding media format 

      // For each media format, look for an associated rtpmap entry. If found,
      // call ImportRtpmap() which will create a AMediaCoder as needed. 
      for(int j=0; j < l_mediaFormatCount; j++)
      {
         //printf("BuildRemoteCapList - Media format #%d = %s\n",j+1,l_sdpMedia->getFormat(j));StaticPayloadName(l_mediaFormats[j]);

         l_created = false;
         for(int k=0; k < l_attributeCount; k++)
         {
            l_sdpAttribute = l_sdpAttributeList->getItem(k);

            //We are only looking for attributes with property type "rtpmap"
            if (!strcmp(l_sdpAttribute->getProperty(), "rtpmap"))
            {
               l_attribute.assign(l_sdpAttribute->getPropertyValue());

//               printf("BuildRemoteCapList - rtpmap Attribute #%d = %s.\n",k+1,l_sdpAttribute->getValue());

               // create media entries as necessary
               l_return =ImportRtpmap(l_attribute, l_mediaFormats[j]);
               if(l_return == 0)
               {
                  l_created = true;
               }
               else
               {
                  continue;
               }
            }
         } // for l_AttributeCount
         // account for static media format without rtpmap attributes. (Note: Dynamic
         // payload types are required to include an rtpmap attribute.) If the
         // rtpmap for a static payload was not included in the SDP, then 
         // conditionally create a AMediaCoder if the format type is something 
         // we support.
         if(l_created == false)
         {
            if ((l_mediaFormats[j] < DYNAMIC_PAYLOAD_MIN) &&
                (strcmp(StaticPayloadName(l_mediaFormats[j]), "")!=0))
            {
               std::string l_payloadName;
               l_payloadName.assign(StaticPayloadName(l_mediaFormats[j]));
               //printf("BuildRemoteCapList - media type: (%d)\n", l_mediaFormats[j]);

               l_return = CreateRemoteMediaCoder(l_payloadName, l_mediaFormats[j]);
               if(l_return != 0)
               {
//                  printf("BuildRemoteCapList - Unrecognized media type: (%d)\n", l_mediaFormats[j]);
                  continue;
               }
            }
            else
            {
//               printf("BuildRemoteCapList - Unsupported media type: (%d)\n", l_mediaFormats[j]);
               continue;
            }
         }
      }  // for l_iMediaFormatCnt

      // add ip address, port value, and media direction for each Remote coder associated
      // with this m= line.
      long l_remotePort = l_sdpMedia->getPort();

      for(int j=0; j < l_mediaFormatCount; j++)
      {
         unsigned int l_payloadType = (unsigned int) atoi(l_sdpMedia->getFormat(j));
         for(unsigned int k=0; k < m_remoteMediaCoders.size(); k++) 
         {
            if (l_payloadType == m_remoteMediaCoders[k]->GetPayloadType())
            {
               
               //printf("BuildRemoteCapList - Media format #%d = %s, IP=%s, port=%ld and direction=%d\n",
               //                   j+1,
               //                   l_sdpMedia->getFormat(j),
               //                   l_connectionIpAddr.c_str(),
               //                   l_remotePort,
               //                   l_localMediaDirection);
               
               m_remoteMediaCoders[k]->SetRtpPortAndIpAddress(l_remotePort, l_connectionIpAddr);
               m_remoteMediaCoders[k]->SetMediaDirection(l_localMediaDirection);
               l_found = true;
               break;
            }
         }
         if(l_found == false)
         {
            LogMsg(SessionName(), AsString("BuildRemoteCapList - Unrecognized media type:") + AsString(l_payloadType));
            continue;
         }
      }

      // now reprocess the media attributes looking for fmtp and ptime.
      for(int j=0; j<l_attributeCount; j++)
      {
         l_sdpAttribute = l_sdpAttributeList->getItem(j);

         if (!strcmp(l_sdpAttribute->getProperty(), "fmtp"))
         {
            l_attribute.assign(l_sdpAttribute->getPropertyValue());

//            printf("BuildRemoteCapList - fmtp Attribute #%d = %s.\n",
//                               j+1,
//                               l_sdpAttribute->getValue());

            l_return=ImportSdpFmtp(l_attribute);
            if( l_return != 0)
            {
               LogMsg(SessionName(), AsString("BuildRemoteCapList - Unrecognized attribute:") + l_attribute);
               continue;
            }
         }

         // generic attribute for all formats, e.g. ptime.
         // Currently, do not handle other generic attributes.
         else if (!strcmp(l_sdpAttribute->getProperty(), "ptime"))
         {
            unsigned int k;
            for(k=0; k < m_remoteMediaCoders.size(); k++)
            {
               const static unsigned int l_attributeLength = 512;
               char l_attributeData[l_attributeLength];
               memset(l_attributeData,0,l_attributeLength);
               sprintf(l_attributeData, "%d ptime=%s", 
                       m_remoteMediaCoders[k]->GetPayloadType(), 
                       l_sdpAttribute->getPropertyValue());
               l_attribute.assign(l_attributeData);

//               printf("BuildRemoteCapList - ptime Attribute #%d = %s.\n",
//                                  j+1, 
//                                  l_attribute.c_str()); 

               m_remoteMediaCoders[k]->ImportSdpFmtp(l_attribute);
            }
         }
      }
   }  // for l_iNumMediaDescr
   return 0;
}

const char* CSipSession::StaticPayloadName( unsigned int a_payloadType)
{
   switch(a_payloadType)
   {
   case 0:
      return "PCMU";
   case 4:
      return "G723";
   case 8:
      return "PCMA";
   case 9:
      return "G722";
   case 18:
      return "G729";
   case 34:
      return "H263";
   }
   return "";
}


//*****************************************************************************
// Function: int CSipSession::ValidateMediaConnection(sdpMediaDescription &a_sdpRemoteMediaDescription, sdpConnection &a_sdpRemoteMediaConnection, eIPM_DATA_DIRECTION &a_localMediaDirection)
// Description: Validate the connection in an SDP media descriptor
// Return: int 
// Parameters: sdpMediaDescription &a_sdpRemoteMediaDescription 
//             sdpConnection &a_sdpRemoteMediaConnection 
//             eIPM_DATA_DIRECTION &a_localMediaDirection 
//*****************************************************************************
int CSipSession::ValidateMediaConnection(sdpMediaDescription& a_sdpRemoteMediaDescription,
                            sdpConnection& a_sdpRemoteMediaConnection,
                            eIPM_DATA_DIRECTION& a_localMediaDirection)
{
   //The default mode will be full duplex
   a_localMediaDirection = DATA_IP_TDM_BIDIRECTIONAL;

   //Validate the number of ports is correct
   sdpMedia* l_sdpRemoteMedia = a_sdpRemoteMediaDescription.media();

   //We only support 1 media port value. 
   if (l_sdpRemoteMedia->getNumPorts() != 1)
   {
//      printf("FindLocalMediaMode - Unsupported number of media Ports (%d) .\n", l_sdpRemoteMedia->getNumPorts());
      a_localMediaDirection = DATA_IP_DIR_NONE;
      return -1;
   }

   //Validated that the media is of type "audio" or "video"
   if( strcmp(l_sdpRemoteMedia->getMedia(),"audio") &&
       strcmp(l_sdpRemoteMedia->getMedia(),"video") )
   {
//      printf("FindLocalMediaMode - unsupported media = %s\n",l_sdpRemoteMedia->getMedia());
      a_localMediaDirection = DATA_IP_DIR_NONE;
      return -1;
   }

   // if connection IP Address = 0.0.0.0, then the new
   // default mode will be RECVONLY
   if (!strcmp(a_sdpRemoteMediaConnection.getAddress(),"0.0.0.0"))
   {
//      printf("FindLocalMediaMode - Connection address = 0.0.0.0. Changing default mode to RECV_ONLY\n");
      a_localMediaDirection = DATA_IP_RECEIVEONLY;
   }

   sdpAttributeList* l_remoteAttributeList = a_sdpRemoteMediaDescription.attributeList();
   // Verify if the match is specific to Recv Only, Send Only, or Send and Receive
   unsigned int l_attributeCount = l_remoteAttributeList->numItem();
   for(unsigned int j=0; j<l_attributeCount; j++)
   {
      sdpAttribute* l_remoteAttribute = l_remoteAttributeList->getItem(j);

      if (!strcmp(l_remoteAttribute->getProperty(),"sendrecv"))
      {
         // make sure there is a valid connection address
         // otherwise this mode is invalid.
         // log a warning and treat as INACTIVE
         if (!strcmp(a_sdpRemoteMediaConnection.getAddress(),"0.0.0.0"))
         {
//            printf(  "FindLocalMediaMode - Received direction attribute=sendrecv without a valid connection address. "\
//                     "This mode of operation is not supported\n");
            a_localMediaDirection = DATA_IP_INACTIVE;
         }
         else
         {
            a_localMediaDirection = DATA_IP_TDM_BIDIRECTIONAL;
         }
         break;
      }
      else if (!strcmp(l_remoteAttribute->getProperty(),"recvonly"))
      {
         // make sure there is a valid connection address
         // otherwise this mode is invalid.
         // log a warning and treat as INACTIVE
         if (!strcmp(a_sdpRemoteMediaConnection.getAddress(),"0.0.0.0"))
         {
//            printf(  "FindLocalMediaMode() Received direction attribute=recvonly without a valid connection address. "\
//                     "This mode of operation is not supported\n");
            a_localMediaDirection = DATA_IP_DIR_NONE;
         }
         else
         {
            // local direction wrt to remote end.
            a_localMediaDirection = DATA_IP_SENDONLY;
         }
         break;
      }
      else if(!strcmp(l_remoteAttribute->getProperty(),"sendonly"))
      {
         // local direction wrt to remote end.
         a_localMediaDirection = DATA_IP_RECEIVEONLY;
         break;

      }
      else if(!strcmp(l_remoteAttribute->getProperty(),"inactive"))
      {
         a_localMediaDirection = DATA_IP_INACTIVE;
         break;
      }
   }  // each a=

   //Set any m= line to ENUM_INVALID if the port value is 0 and the media mode is not 
   //ENUM_RECV_ONLY
   if( (l_sdpRemoteMedia->getPort() == 0) && (a_localMediaDirection != DATA_IP_RECEIVEONLY) )
   {
      a_localMediaDirection = DATA_IP_DIR_NONE;
   }

   return 0;  
}

//*****************************************************************************
// Function: int CSipSession::ImportRtpmap(std::string &a_attribute, unsigned int a_payloadType)
// Description: Import an SDP rtpmap and create a remote media coder object
// Return: int 
// Parameters: std::string &a_attribute 
//             unsigned int a_payloadType 
//*****************************************************************************
int CSipSession::ImportRtpmap(std::string& a_attribute, unsigned int a_payloadType)
{
   int l_tokenCount = 0;
   char l_separator[] = " /";
   char *l_token;
   int l_return = 0;

   unsigned int l_payloadType=0xffff;
   bool l_found = false;
   std::string   l_payloadCoderName;
   unsigned int l_clockRate = 0;
   unsigned int l_channel = 0;
   char l_attribute[AMediaCoder::MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = 0;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= AMediaCoder::MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("ImportRtpmap - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",a_attribute.size(),AMediaCoder::MEDIA_ATTRIB_SIZE_MAX);
      return -1;
   }

   // Make a copy of the Attribute value because DLGC_STRTOK will modify
   // the char array.. 
   strcpy(l_attribute, a_attribute.c_str());
   //Add NULL termination
   l_attribute[l_attributeSize] = '\0';

   l_token = DLGC_STRTOK(l_attribute, l_separator, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_payloadType =  atoi(l_token);
         if(a_payloadType != l_payloadType)
         {
            return -1;
         }
         l_found = true;
         break;

      case 1:
         l_payloadCoderName.assign(l_token);
         break;

      case 2:
         l_clockRate = atoi(l_token); // currently not used
         break;
         
      case 3:
         l_channel = (unsigned int)atoi(l_token);   // currently not used
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(0, l_separator, &l_tokenizerBuffer);
      l_tokenCount++;
   }
   
   if (l_found)
   {
      //printf("ImportRtpmap - found rtpmap for (%d)\n", l_payloadType);
      l_return=CreateRemoteMediaCoder(l_payloadCoderName, l_payloadType);
   }

   return l_return;
}

//*****************************************************************************
// Function: int CSipSession::CreateRemoteMediaCoder(std::string &a_payloadCoderName, unsigned int a_payloadType)
// Description: Create a Media Coder object based on the name and type
// Return: int 
// Parameters: std::string &a_payloadCoderName 
//             unsigned int a_payloadType 
//*****************************************************************************
int CSipSession::CreateRemoteMediaCoder(std::string& a_payloadCoderName, unsigned int a_payloadType)
{
   //printf("CreateRemoteMediaCoder - PAYLOAD=%s, Format=%d\n", a_payloadCoderName.c_str(), a_payloadType);

   AMediaCoder* l_mediaCoder = 0;
   std::string l_payloadCoderName = a_payloadCoderName;

   // convert Payload coser name to all uppercase
   for(unsigned int i = 0; i < l_payloadCoderName.length(); i++)
   {
      l_payloadCoderName[i] = toupper(l_payloadCoderName[i]); 
   }

   //Check for duplicates. If found, ignore.
   if(a_payloadType < DYNAMIC_PAYLOAD_MIN)
   {
      for(unsigned int i=0; i < m_remoteMediaCoders.size(); i++) 
      {
         if (a_payloadType == m_remoteMediaCoders[i]->GetPayloadType())
         {
            printf("CreateRemoteMediaCoder - Duplicate static Rtpmap entry ignored: %s(%d)\n", 
                               l_payloadCoderName.c_str(), 
                               a_payloadType);

            return -1;
         }
      }
   }
   else if( (a_payloadType >= DYNAMIC_PAYLOAD_MIN) &&
            (a_payloadType < DYNAMIC_PAYLOAD_MAX ))
   {
      for(unsigned int i=0; i < m_remoteMediaCoders.size(); i++)
      {
         if (!strcmp(m_remoteMediaCoders[i]->GetPayloadCoderName(), l_payloadCoderName.c_str()))
         {
            printf("CreateRemoteMediaCoder - Duplicate dynamic Rtpmap entry ignored: %s(%d)\n", 
                               l_payloadCoderName.c_str(), 
                               a_payloadType);

            return -1;
         }
      }
   }

   // check for both upper and lower case matches
   if (!strcmp(l_payloadCoderName.c_str(), "PCMA"))
   {
      l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG711a());
      m_remotePayloads[a_payloadType] = a_payloadType;
   } 
   else if (!strcmp(l_payloadCoderName.c_str(), "PCMU")) 
   {
      l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG711u());
      m_remotePayloads[a_payloadType] = a_payloadType;
   }
   else if (!strcmp(l_payloadCoderName.c_str(), "G723"))
   {
      CMediaCoderG723* l_g723Coder = (new CMediaCoderG723());
      m_remotePayloads[a_payloadType] = a_payloadType;
      l_g723Coder->EnableVAD(); // by default VAD is enabled unless explicitly disabled.
      l_mediaCoder = dynamic_cast<AMediaCoder *> (l_g723Coder);
   }
   else if (!strcmp(l_payloadCoderName.c_str(), "G729"))
   {
      CMediaCoderG729* l_g729Coder = (new CMediaCoderG729());
      m_remotePayloads[a_payloadType] = a_payloadType;
      l_mediaCoder = dynamic_cast<AMediaCoder *> (l_g729Coder);
   }
   else if (!strcmp(l_payloadCoderName.c_str(), "AMR"))
   {
      if (m_remotePayloads[a_payloadType] == -1)
      {
         CMediaCoderAmrNb* l_amrCoder = new CMediaCoderAmrNb(a_payloadType);
         m_remotePayloads[a_payloadType] = a_payloadType;
         l_amrCoder->EnableAllModeSet(); // by default all mode set is enabled unless explicitly changed
         l_mediaCoder = dynamic_cast<AMediaCoder *> (l_amrCoder);
      }
      else
      {
         printf("CreateRemoteMediaCoder - %s=%d in use.\n", l_payloadCoderName.c_str(), a_payloadType);
         return -1;
      }
   }
   else if (!strcmp(l_payloadCoderName.c_str(), "AMR-WB"))
   {
      if (m_remotePayloads[a_payloadType] == -1)
      {
         CMediaCoderAmrWb* l_amrCoder = new CMediaCoderAmrWb(a_payloadType);
         m_remotePayloads[a_payloadType] = a_payloadType;
         l_amrCoder->EnableAllModeSet(); // by default all mode set is enabled unless explicitly changed
         l_mediaCoder = dynamic_cast<AMediaCoder *> (l_amrCoder);
      }
      else
      {
         printf("CreateRemoteMediaCoder - %s=%d in use.\n", l_payloadCoderName.c_str(), a_payloadType);
         return -1;
      }
   }

   else if (!strcmp(l_payloadCoderName.c_str(), "G722")) 
   {
      l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG722());
      m_remotePayloads[a_payloadType] = a_payloadType;
   }
   else if (!strcmp(l_payloadCoderName.c_str(), "TELEPHONE-EVENT"))
   {
      if (m_remotePayloads[a_payloadType] == -1)
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderRfc2833(a_payloadType));
         m_remotePayloads[a_payloadType] = a_payloadType;
      }
      else
      {
         printf("CreateRemoteMediaCoder - %s=%d in use.\n", l_payloadCoderName.c_str(), a_payloadType);
         return -1;
      }
   }
   else if (!strcmp(l_payloadCoderName.c_str(), "H263"))
   {
      l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderH263());

   }
   else if (!strcmp(l_payloadCoderName.c_str(), "MP4V-ES"))
   {
      if (m_remotePayloads[a_payloadType] == -1)
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderMp4v(a_payloadType));
         m_remotePayloads[a_payloadType] = a_payloadType;
      }
      else
      {
         printf("CreateRemoteMediaCoder - %s=%d in use.\n", l_payloadCoderName.c_str(), a_payloadType);
         return -1;
      }
   }
#ifdef USE_H264
   else if (!strcmp(l_payloadCoderName.c_str(), "H264"))
   {
      if (m_remotePayloads[a_payloadType] == -1)
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderH264(a_payloadType));
         m_remotePayloads[a_payloadType] = a_payloadType;
      }
      else
      {
         printf("CreateRemoteMediaCoder - %s=%d in use.\n", l_payloadCoderName.c_str(), a_payloadType);
         return -1;
      }
   }
#else
   else if (!strcmp(l_payloadCoderName.c_str(), "H264"))
   {
      printf("Unsupported Media Format: H264 Codec support is not compiled in.\n HMP4.1 only: Please re-compile with USE_H264 in makefile.\n");
      return -1;
   }

#endif
   else
   {
      printf("CreateRemoteMediaCoder - Unsupported Media Format: %s(%d)\n", l_payloadCoderName.c_str(), a_payloadType);
      return -1;
   }

   //by default direction is bidiretional unless attribute overwrite it.
   //printf("CreateRemoteMediaCoder - %s(%d)\n", l_mediaCoder->PayloadName(), l_mediaCoder->PayloadNumber());

   //Add remote coder to vector
   m_remoteMediaCoders.push_back(l_mediaCoder);
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::ImportSdpFmtp(const std::string &a_attribute)
// Description: Find a coder in the remote  capabilities list and import ab SDP
//   fmtp line
// Return: int 
// Parameters: const std::string &a_attribute 
//*****************************************************************************
int CSipSession::ImportSdpFmtp(const std::string& a_attribute)
{

   //printf("ImportSdpFmtp - Attrib=%s\n", a_attribute.c_str());

   unsigned int i = 0;
   unsigned int l_format = 0;
   unsigned int l_tokenCount=0;
   char l_separator[] = "= ;";
   char *l_token=NULL;
   bool l_found = false;
   char l_attribute[AMediaCoder::MEDIA_ATTRIB_SIZE_MAX];
   char* l_tokenizerBuffer = NULL;

   size_t l_attributeSize = a_attribute.size();
   if(l_attributeSize >= AMediaCoder::MEDIA_ATTRIB_SIZE_MAX)
   {
      printf("ImportSdpFmtp - Attribute.size()= %u is greater than MEDIA_ATTRIB_SIZE_MAX=%u\n",
                         a_attribute.size(),
                         AMediaCoder::MEDIA_ATTRIB_SIZE_MAX);
      return -1;
   }

   strcpy(l_attribute, a_attribute.c_str());
   //Add NULL termination
   l_attribute[l_attributeSize] = '\0';


   l_token = DLGC_STRTOK(l_attribute, l_separator, &l_tokenizerBuffer);
   while (l_token != NULL)
   {
      switch (l_tokenCount)
      {
      case 0:
         l_format = atoi(l_token);

         for(i=0; i < m_remoteMediaCoders.size(); i++)
         {
            if (m_remoteMediaCoders[i]->GetPayloadType() == l_format)
            {
               m_remoteMediaCoders[i]->ImportSdpFmtp(a_attribute);
               l_found = true;
               break;
            }
         }
         if (!l_found)
         {
            LogMsg(SessionName(), AsString("ImportSdpFmtp -Unrecognized media type:") + AsString(l_format));
            return -1;
         }
         break;

      default:
         break;
      }

      l_token = DLGC_STRTOK(0, l_separator, &l_tokenizerBuffer);
      l_tokenCount++;
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::BuildLocalCapList()
// Description: Build the list of local capabilities
// Return: int 
// Parameters: none 
//*****************************************************************************
int CSipSession::BuildLocalCapList()
{

   int l_localAudioCoderCount = sizeof(g_systemAudioCoderCapabilities_HMP)/sizeof(IPM_AUDIO_CODER_INFO);
   for (int i =0; i<l_localAudioCoderCount; i++)
   {
      CreateLocalMediaCoder(g_systemAudioCoderCapabilities_HMP[i]);
   }

   AMediaCoder* l_rfc2833Coder = new CMediaCoderRfc2833(101);
   if (l_rfc2833Coder)
   {
      m_localMediaCoders.push_back(l_rfc2833Coder);
   }

   int l_localVideoCoderCount = sizeof(g_systemVideoCoderCapabilities_HMP)/sizeof(IPM_VIDEO_CODER_INFO);
   for (int i =0; i<l_localVideoCoderCount; i++)
   {
      CreateLocalMediaCoder(g_systemVideoCoderCapabilities_HMP[i]);
   }

   return 0;
}

//*****************************************************************************
// Function: int CSipSession::CreateLocalMediaCoder(IPM_AUDIO_CODER_INFO &l_ipmAudioCoderInfo)
// Description: Create a local audio coder from a populates IPM_AUDIO_CODER_INFO structure
// Return: int 
// Parameters: IPM_AUDIO_CODER_INFO &l_ipmAudioCoderInfo 
//*****************************************************************************
int CSipSession::CreateLocalMediaCoder(IPM_AUDIO_CODER_INFO& l_ipmAudioCoderInfo)
{
   AMediaCoder* l_mediaCoder = 0;
   switch (l_ipmAudioCoderInfo.eCoderType)
   {
   case CODER_TYPE_G711ALAW64K:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG711a(l_ipmAudioCoderInfo));
      }
      break;

   case CODER_TYPE_G711ULAW64K:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG711u(l_ipmAudioCoderInfo));
      }
      break;

   case CODER_TYPE_G7231_5_3K:
   case CODER_TYPE_G7231_6_3K:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG723(l_ipmAudioCoderInfo));
      }
      break;

   case CODEC_AUDIO_G729ANNEXAWANNEXB:
   case CODEC_AUDIO_G729WANNEXB:
   case CODEC_AUDIO_G729ANNEXA:
   case CODEC_AUDIO_G729:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG729(l_ipmAudioCoderInfo));
      }
      break;

   case CODER_TYPE_AMRNB_4_75k: 
   case CODER_TYPE_AMRNB_5_15k:
   case CODER_TYPE_AMRNB_5_9k:
   case CODER_TYPE_AMRNB_6_7k:
   case CODER_TYPE_AMRNB_7_4k:
   case CODER_TYPE_AMRNB_7_95k:
   case CODER_TYPE_AMRNB_10_2k:
   case CODER_TYPE_AMRNB_12_2k:
   case CODER_TYPE_AMRNB_NONE:
      l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderAmrNb(l_ipmAudioCoderInfo));
      break;

   case CODER_TYPE_AMRWB_6_6K: 
   case CODER_TYPE_AMRWB_8_85K:
   case CODER_TYPE_AMRWB_12_65K:
   case CODER_TYPE_AMRWB_14_25K:
   case CODER_TYPE_AMRWB_15_85K:
   case CODER_TYPE_AMRWB_18_25K:
   case CODER_TYPE_AMRWB_19_85K:
   case CODER_TYPE_AMRWB_23_05K:
   case CODER_TYPE_AMRWB_23_85K:
   case CODER_TYPE_AMRWB_NONE:
      l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderAmrWb(l_ipmAudioCoderInfo));
      break;

   case CODER_TYPE_G722_64K:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderG722(l_ipmAudioCoderInfo));
      }
      break;

   default:
      LogMsg(SessionName(), AsString("unhandled coder ") + AMediaCoder::GetDialogicCoderTypeDesc(l_ipmAudioCoderInfo.eCoderType));
   }

   if (l_mediaCoder)
   {
      m_localMediaCoders.push_back(l_mediaCoder);
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::CreateLocalMediaCoder(IPM_VIDEO_CODER_INFO &l_ipmVideoCoderInfo)
// Description: Create a local video coder from a populates IPM_AUDIO_CODER_INFO structure
// Return: int 
// Parameters: IPM_VIDEO_CODER_INFO &l_ipmVideoCoderInfo 
//*****************************************************************************
int CSipSession::CreateLocalMediaCoder(IPM_VIDEO_CODER_INFO& l_ipmVideoCoderInfo)
{
   AMediaCoder* l_mediaCoder = 0;
   switch (l_ipmVideoCoderInfo.eCoderType)
   {
   case CODER_TYPE_H263:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderH263(l_ipmVideoCoderInfo));
      }
      break;

   case CODER_TYPE_MP4V_ES:
      {
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderMp4v(l_ipmVideoCoderInfo));
      }
      break;

   case CODER_TYPE_H264:
      {
#ifdef USE_H264
         l_mediaCoder = dynamic_cast<AMediaCoder *> (new CMediaCoderH264(l_ipmVideoCoderInfo));
#else

         LogMsg(SessionName(), AsString("unhandled coder ") + AMediaCoder::GetDialogicCoderTypeDesc(l_ipmVideoCoderInfo.eCoderType));
#endif
      }
      break;

   default:
      LogMsg(SessionName(), AsString("unhandled coder ") + AMediaCoder::GetDialogicCoderTypeDesc(l_ipmVideoCoderInfo.eCoderType));
   }

   if (l_mediaCoder)
   {
      m_localMediaCoders.push_back(l_mediaCoder);
   }
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::GetHostInformation()
// Description: Get the local hostname and IP address information
// Return: int 
// Parameters: none 
//*****************************************************************************
int CSipSession::GetHostInformation()
{
   // Get IP information
   int l_return;
#ifdef WIN32
   WSADATA l_wsaData;
   // In Windows you must initialize the socket library
   l_return = WSAStartup(MAKEWORD(2,2), &l_wsaData);
   if(l_return != 0)
   {
      LogMsg(SessionName(), AsString("Call to WSAStartup() failed, returned ") + AsString(l_return));
      return -1;
   }
#endif

   memset(m_hostName,0,IP_NAME);
   l_return = gethostname(m_hostName, IP_NAME);
   if (l_return != 0)
   {
#ifdef WIN32
      LogMsg(SessionName(), AsString("Call to gethostname() failed, returned ") + AsString(l_return) + AsString("WSAGetLastError=") + AsString(WSAGetLastError()));
#else
      LogMsg(SessionName(), AsString("Call to gethostname() failed, returned ") + AsString(l_return));
#endif
      return -1;
   }

   struct hostent *l_hostent;
   if ((l_hostent = (struct hostent *)gethostbyname(m_hostName)) == 0)
   {
#ifdef WIN32
      LogMsg(SessionName(), AsString("Call to gethostbyname() failed, returned NULL, WSAGetLastError=") + AsString(WSAGetLastError()));
#else
      LogMsg(SessionName(), AsString("Call to gethostbyname() failed, returned NULL"));
#endif
      return -1;
   }

   struct in_addr  l_addr;
	// network byte order
   memcpy(&l_addr, l_hostent->h_addr_list[0], sizeof(struct in_addr));

   // convert binary inet address into dot format.
   memset(m_hostIpAddress,0,IP_NAME);
   strcpy(m_hostIpAddress,inet_ntoa(l_addr));

   LogMsg(SessionName(), AsString("InitializeSystem(): HostName=") + AsString(m_hostName) + AsString("IpAddr=") + AsString(m_hostIpAddress));
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::InitializeAnswerSdp(sdpSessionDescription &a_sdp)
// Description: Initialize an answer SDP
// Return: int 
// Parameters: sdpSessionDescription &a_sdp 
//*****************************************************************************
int CSipSession::InitializeAnswerSdp(sdpSessionDescription& a_sdp)
{
   GetHostInformation();

   a_sdp.clear();

   time_t l_time;
   time(&l_time);
   char l_cTime[100];
   memset(l_cTime,0,sizeof(l_cTime));
   sprintf(l_cTime,"%d %d",(int)l_time,(int)l_time);

   a_sdp.version()->setVersion("0");
   a_sdp.origin()->setUserName("VideoConfDemo");
   a_sdp.origin()->setNetworkType("IN");
   a_sdp.origin()->setAddressType("IP4");
   a_sdp.origin()->setAddress(m_hostIpAddress);
   a_sdp.origin()->setSessionId(l_cTime);
   a_sdp.sessionName()->setName("Dialogic_demo");

   sdpTimeDescription *l_sdpTimeDescription = a_sdp.timeDescriptionList()->addItem();
   l_sdpTimeDescription->time()->setStart(0);
   l_sdpTimeDescription->time()->setStop(0);

   return 0;
}

//*****************************************************************************
// Function: int CSipSession::BuildAnswerSDP(sdpSessionDescription &a_remoteSdp, sdpSessionDescription &a_localSdp)
// Description: Build an answer ADP
// Return: int 
// Parameters: sdpSessionDescription &a_remoteSdp 
//             sdpSessionDescription &a_localSdp 
//*****************************************************************************
int CSipSession::BuildAnswerSDP(sdpSessionDescription& a_remoteSdp, sdpSessionDescription& a_localSdp)
{
   if(a_remoteSdp.timeDescriptionList()->numItem() > 0)
   {
      sdpTimeDescription* l_remoteTimeDescription = a_remoteSdp.timeDescriptionList()->getItem(0);

     // update one set in InitializeSdp()
      sdpTimeDescription* l_localTimeDescription = a_localSdp.timeDescriptionList()->getItem(0);

      // Set the "t=" line based on the Remoteed Sdp value
      l_localTimeDescription->time()->setStart(l_remoteTimeDescription->time()->getStart());
      l_localTimeDescription->time()->setStop(l_remoteTimeDescription->time()->getStop());
   }
   
   //JM - required for Vippie Android
   //session c= line
   sdpConnection* l_sessionConnection = a_localSdp.connection();
   l_sessionConnection->setNetworkType("IN");
   l_sessionConnection->setAddressType("IP4");
   l_sessionConnection->setAddress(m_hostIpAddress);

   if (m_sessionBandwidth)
   {
      sdpBandwidth* l_sessionBandwidth = a_localSdp.bandwidth();
      l_sessionBandwidth->clear();
      l_sessionBandwidth->setModifier("AS");
      char l_bandwidth[32];
      sprintf(l_bandwidth,"%d",m_sessionBandwidth);
      l_sessionBandwidth->setBandwidthValue(l_bandwidth);
   }
   
   // clear media description list
   sdpMediaDescriptionList* l_mediaDescriptionList = a_localSdp.mediaDescriptionList();
   l_mediaDescriptionList->clear();

   // Audio Coder
   if (m_matchedAudioCoder)
   {
      IPM_PORT_INFO& l_localAudioPortInfo = m_ipmDevice->LocalAudioPortInfo();
      
      sdpMediaDescription* l_audioMediaDescription = l_mediaDescriptionList->addItem();
      l_audioMediaDescription->connection()->setNetworkType("IN");
      l_audioMediaDescription->connection()->setAddressType("IP4");
      l_audioMediaDescription->connection()->setAddress(l_localAudioPortInfo.cIPAddress);
      
      m_matchedAudioCoder->ExportSdpMediaDescription(l_audioMediaDescription,l_localAudioPortInfo.unPortId);
      
      if (m_matchedRfc2833Coder)
      {
         m_matchedRfc2833Coder->ExportSdpMediaDescription(l_audioMediaDescription,l_localAudioPortInfo.unPortId);
      }
      if (m_audioBandwidth)
      {
         sdpBandwidth* l_audioBandwidth = l_audioMediaDescription->bandwidth();
         l_audioBandwidth->clear();
         l_audioBandwidth->setModifier("AS");
         char l_bandwidth[32];
         sprintf(l_bandwidth,"%d",m_audioBandwidth);
         l_audioBandwidth->setBandwidthValue(l_bandwidth);
      }
   }
   
   // Video coder
   if (m_matchedVideoCoder)
   {
      IPM_PORT_INFO& l_localVideoPortInfo = m_ipmDevice->LocalVideoPortInfo();

      sdpMediaDescription* l_videoMediaDescription = l_mediaDescriptionList->addItem();
      l_videoMediaDescription->connection()->setNetworkType("IN");
      l_videoMediaDescription->connection()->setAddressType("IP4");
      l_videoMediaDescription->connection()->setAddress(l_localVideoPortInfo.cIPAddress);
      
      m_matchedVideoCoder->ExportSdpMediaDescription(l_videoMediaDescription,l_localVideoPortInfo.unPortId);
      if (m_videoBandwidth)
      {
         sdpBandwidth* l_videoBandwidth = l_videoMediaDescription->bandwidth();
         l_videoBandwidth ->clear();
         l_videoBandwidth ->setModifier("AS");
         char l_bandwidth[32];
         sprintf(l_bandwidth,"%d",m_videoBandwidth);
         l_videoBandwidth ->setBandwidthValue(l_bandwidth);
      }
   }


   return 0;  
}

//*****************************************************************************
// Function: int CSipSession::PrimAddCallerIdOverlay()
// Description: Handle an add caller ID image overlay command
// Return: int 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimAddCallerIdOverlay()
{
   LogMsg(SessionName(), "PrimAddCallerIdOverlay");
   int res(-1);
   if (!m_callerIdOverlay)
   {
      strcpy(m_callerIdOverlayInfo.imgName, m_systemConfiguration.MediaFilePath().c_str());
      strcat(m_callerIdOverlayInfo.imgName, g_callerIdFiles[m_callerId]);

      m_callerIdOverlay = new COverlay(&m_callerIdOverlayInfo);
      res = m_callerIdOverlay->addOverlay(m_ipmDevice->Device(),eSM_OVERLAY_DIRECTION_DEVICE);
   }
   return res;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnAddCallerIdOverlay(METAEVENT &a_metaEvent)
// Description: Handle an add caller ID image overlay completion event
// Return: int 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CSipSession::PrimOnAddCallerIdOverlay(METAEVENT& a_metaEvent)
{  
   LogMsg(SessionName(), "PrimOnAddCallerIdOverlay");
   long evtlen = sr_getevtlen();
   void* evtContext = sr_getUserContext();
   void* pEvtData = sr_getevtdatap();
   return m_callerIdOverlay->OnOverlayAdd(a_metaEvent.evtlen, a_metaEvent.evtdatap, evtContext);
}

//*****************************************************************************
// Function: int CSipSession::PrimRemoveCallerIdOverlay()
// Description: Handle a remove caller ID image overlay command
// Return: int 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimRemoveCallerIdOverlay()
{
   LogMsg(SessionName(), "PrimRemoveCallerIdOverlay");
   int res = m_callerIdOverlay->removeOverlay(m_ipmDevice->Device());
   return res;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnRemoveCallerIdOverlay(METAEVENT &a_metaEvent)
// Description: Handle a remove the caller ID image overlay completion event
// Return: int 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CSipSession::PrimOnRemoveCallerIdOverlay(METAEVENT& a_metaEvent)
{  
   LogMsg(SessionName(), "PrimOnRemoveCallerIdOverlay");
   long evtlen = sr_getevtlen();
   void* evtContext = sr_getUserContext();
   void* pEvtData = sr_getevtdatap();
   int res = m_callerIdOverlay->OnOverlayRemove(a_metaEvent.evtlen, a_metaEvent.evtdatap, evtContext);
   delete m_callerIdOverlay;
   m_callerIdOverlay = 0;
   return res;
}

//*****************************************************************************
// Function: int CSipSession::PrimAddConferenceLayoutOverlay(int a_layout)
// Description: Handle an add conference layout description image overlay 
// command
// Return: int 
// Parameters: int a_layout 
//*****************************************************************************
int CSipSession::PrimAddConferenceLayoutOverlay(int a_layout)
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnAddConfernceLayoutOverlay(METAEVENT &a_metaEvent)
// Description: Handle a remove conference layout description image overlay 
//                completion event
// Return: int 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CSipSession::PrimOnAddConfernceLayoutOverlay(METAEVENT& a_metaEvent)
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimRemoveConferenceLayoutOverlay()
// Description: Handle a remove conference layout description image overlay 
//                command
// Return: int 
// Parameters: none 
//*****************************************************************************
int CSipSession::PrimRemoveConferenceLayoutOverlay()
{
   return 0;
}

//*****************************************************************************
// Function: int CSipSession::PrimOnRemoveConferenceLayoutOverlay(METAEVENT &a_metaEvent)
// Description: Handle a remove conference layout description image overlay 
//                completion event
// Return: int 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
int CSipSession::PrimOnRemoveConferenceLayoutOverlay(METAEVENT& a_metaEvent)
{
   return 0;
}

//*****************************************************************************
// Function: void CSipSession::ProcessEvent(METAEVENT &a_metaEvent)
// Description: Process event
// Return: void 
// Parameters: METAEVENT &a_metaEvent 
//*****************************************************************************
void CSipSession::ProcessEvent(METAEVENT& a_metaEvent)
{
   switch (a_metaEvent.evttype)
   {
      case APPEVENT_MM_OPEN_COMPLETE:
         break;

      case APPEVENT_MM_GET_RX_PORT_INFO_COMPLETE:
         OnGetMmRxPortInfoComplete();
         break;

      case APPEVENT_MM_GET_TX_PORT_INFO_COMPLETE:
         OnGetMmTxPortInfoComplete();
         break;

      case APPEVENT_MM_PORT_CONNECTED:
         OnMmPortConnected();
         break;

      case APPEVENT_MM_PORT_DISCONNECTED:
         OnMmPortDisconnected();
         break;

      case APPEVENT_IPM_GET_RX_PORT_INFO_COMPLETE:
         OnGetIpmRxPortInfoComplete();
         break;

      case APPEVENT_IPM_GET_TX_PORT_INFO_COMPLETE:
         OnGetIpmTxPortInfoComplete();
         break;

      case APPEVENT_IPM_PORT_CONNECTED:
         OnIpmPortConnected();
         break;

      case APPEVENT_IPM_PORT_DISCONNECTED:
         OnIpmPortDisconnected();
         break;

      case APPEVENT_PLAY_COMPLETE:
         OnPlayComplete();
         break;

      case APPEVENT_PLAY_FAIL:
         break;

      case APPEVENT_RECORD_COMPLETE:
         OnRecordComplete();
         break;

      case APPEVENT_RECORD_FAIL:
         break;

      case APPEVENT_ON_CLOSE_CNF_PARTY_COMPLETE:
         OnCloseConferencePartyComplete();
         break;

      case APPEVENT_ON_OPEN_CNF_PARTY_COMPLETE:
         OnOpenConferencePartyComplete();
         break;

      case APPEVENT_ON_JOIN_CNF_COMPLETE:
         OnJoinConferenceComplete();
         break;

      case APPEVENT_ON_LEAVE_CNF_COMPLETE:
         OnLeaveConferenceComplete();
         break;

      case APPEVENT_CNF_CONNECTED:
         OnConnectToConferencePartyComplete();
         break;

      case APPEVENT_CNF_DISCONNECTED:
         OnDisconnectFromConferencePartyComplete();
         break;

      case APPEVENT_CNF_GET_RX_PORT_INFO_COMPLETE:
         OnGetCnfRxPortInfoComplete();
         break;

      case APPEVENT_CNF_GET_TX_PORT_INFO_COMPLETE:
         OnGetCnfTxPortInfoComplete();
         break;

      case APPEVENT_CNF_PORT_CONNECTED:
         OnCnfPortConnected();
         break;

      case APPEVENT_CNF_PORT_DISCONNECTED:
         OnCnfPortDisconnected();
         break;

      case APPEVENT_RESERVE_RESOURCE_COMPLETE:
         OnReserveResourceComplete();
         break;

      case APPEVENT_RELEASE_RESOURCE_COMPLETE:
         OnReleaseResourceComplete();
         break;

      case APPEVENT_GET_LOCAL_MEDIA_INFO_COMPLETE:
         OnGetLocalMediaInfoComplete();
         break;

      case APPEVENT_START_MEDIA_COMPLETE:
         OnStartMediaComplete();
         break;

      case APPEVENT_STOP_MEDIA_COMPLETE:
         OnStopMediaComplete();
         break;

      case APPEVENT_DTMF_DIGIT:
         OnDtmf((int) a_metaEvent.evtdatap);
         break;

      case SMEV_ADD_OVERLAY:
         LogMsg(SessionName(), "SMEV_ADD_OVERLAY");
         OnAddImageOverlay(a_metaEvent);
         break;

      case SMEV_REMOVE_OVERLAY:
         LogMsg(SessionName(), "SMEV_REMOVE_OVERLAY");
         OnRemoveImageOverlay(a_metaEvent);
         break;

      case SMEV_ADD_OVERLAY_FAIL:
         LogMsg(SessionName(), "SMEV_ADD_OVERLAY_FAIL");
         break;

      case SMEV_REMOVE_OVERLAY_FAIL:
         LogMsg(SessionName(), "SMEV_REMOVE_OVERLAY_FAIL");
         break;

      case APPEVENT_DISPLAY_CALLERID_COMMAND:
         LogMsg(SessionName(), "APPEVENT_DISPLAY_CALLERID_COMMAND");
         AddCallerIdOverlay();
         break;

      case APPEVENT_DISPLAY_LAYOUT_COMMAND:
         LogMsg(SessionName(), "APPEVENT_DISPALY_LAYOUT_COMMAND");
//         AddConferenceLayoutOverlay((int)a_metaEvent.evtdatap);
         break;

	  case APPEVENT_DROPCALL_COMMAND:
         LogMsg(SessionName(), "APPEVENT_DROPCALL_COMMAND");
		 StopSession();
		  break;

      default:
         LogMsg(SessionName(), "unhandled event:" + AsHexString(a_metaEvent.evttype));
   }
}

//*****************************************************************************
// Function: void CSipSession::StopSession()
// Description: Stop the session
// Return: void CSipSession::void 
// Parameters: none 
//*****************************************************************************
void CSipSession::StopSession()
{
   m_sipDevice->PutEvent(APPEVENT_DROPCALL_COMMAND,0,0);
}
