/**
* @file LogUtil.h
* @brief Logging Utility decleration
* @date No.v 15, 2007
*
* DIALOGIC CONFIDENTIAL
* Copyright  2007 Dialogic Corporation. All Rights Reserved.
* 
* The source code contained or described herein and all documents related to
* the source code (Material) are owned by Dialogic Corporation or its suppliers
* or licensors. Title to the Material remains with Dialogic Corporation or its
* suppliers and licensors. The Material contains trade secrets and proprietary
* and confidential information of Dialogic or its suppliers and licensors. The
* Material is protected by worldwide copyright and trade secret laws and treaty
* provisions. No part of the Material may be used, copied, reproduced,
* modified, published, uploaded, posted, transmitted, distributed, or disclosed
* in any way without Dialogic's prior express written permission.
* 
* No license under any patent, copyright, trade secret or other intellectual
* property right is granted to or conferred upon you by disclosure or delivery
* of the Materials, either expressly, by implication, inducement, estoppel or
* otherwise. Any license under such intellectual property rights must be
* express and approved by Dialogic in writing.
*/

#ifndef LogUtil_h
#define LogUtil_h

#ifdef WIN32
#pragma warning(disable:4311 4312 4267 4018 4244 4996 4065 4101) 
#endif 

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include "gclib.h"

using std::string;
using std::ostringstream;

//*****************************************************************************
// Function: string ::AsString(T a_value)
// Description: Template function to convert a value to a string
// Return: string 
// Parameters: T a_value 
//*****************************************************************************
template< typename T> string AsString(T a_value)
{
   ostringstream l_tempStream;
   l_tempStream << a_value;
   return l_tempStream.str();
}

//*****************************************************************************
// Function: std::string ::AsHexString(T a_value, int a_width=4)
// Description: Template function to convert a value to a string hex representation
// Return: std::string 
// Parameters: T a_value 
//             int a_width=4 
//*****************************************************************************
template< typename T> std::string AsHexString( T a_value, int a_width = 4)
{
   std::ostringstream l_stream;
   l_stream << "0x" << std::hex << std::setw( a_width) << std::setfill('0') << std::uppercase << a_value;
   return l_stream.str();
}

//*****************************************************************************
// Description: Logging functions
//*****************************************************************************
void LogMsg(const string& msg);
void LogMsg(const int dev, const string& msg);
void LogMsg(const string& devName, const string& msg);
int GcInfoMsg(const GC_INFO& info, const LINEDEV gcDev, const string& msgType, const string& msg);
void GcResultMsg(const LINEDEV gcDev, const METAEVENT& metaevent);
int GcErrMsg(const LINEDEV gcDev, const string& msg);
int ErrMsg(const int dev, const string& msg);

void WriteToLog(std::stringstream& msg);

extern std::ofstream g_logfile;

#endif

